/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("com.ibm.asq.common.web.client.internal._catalog.RootAttribute");
dojo.require("com.ibm.asq.common.web.client.internal._catalog._AttributeBase");
dojo.declare("com.ibm.asq.common.web.client.internal._catalog.RootAttribute", com.ibm.asq.common.web.client.internal._catalog._AttributeBase, 
{
    constructor: function(document, parent, instanceRoot, templateObj, path){
        if (instanceRoot != null) {
            this.instanceObj = instanceRoot;
        }
        this.path = "root";
        this.partCache = new Array();
        this._buildPartCache();
        if(parent instanceof com.ibm.asq.common.web.client.internal._catalog.Asset){
            this._isArray = false;
        } else {
            this._isArray = true;
        }
    },
    // Get the type of the attributes value.
    getType: function(){
        return "ConfigurationPartType";
    }, 
    // This method can be called to get the specific name of 
    // the part.  Then getTemplate(partName) can be called to
    // get the detailed information about part names and 
    // attributes.
    getPartTypeName: function(){
        return this.parent.typeName;
    },
    // Get the label for one of the assets
    getLabelByTag: function(tag){
        return this.getLabel(this.getIndexByTag(tag));
    },
    // Get the label for one of the assets
    getLabel: function(index){
        var valArray = this._util._getArrayObject(this.instanceObj, "Assets", "AssetInstance");
        return valArray[index].Label;
    },
    // Set the label for one of the assets
    setLabelByTag: function(tag, label){
        return this.setLabel(this.getIndexByTag(tag), label);
    },
    // Set the label for one of the assets
    setLabel: function(index, label){
        var valArray = this._util._getArrayObject(this.instanceObj, "Assets", "AssetInstance");
        valArray[index].Label = label;
    },
    // Get the identifier for one of the assets
    getIdentifierByTag: function(tag){
        return this.getIdentifier(this.getIndexByTag(tag));
    },
    // Get the identifier for one of the assets
    getIdentifier: function(index){
        var valArray = this._util._getArrayObject(this.instanceObj, "Assets", "AssetInstance");
        return valArray[index].identifier;
    },
    // Set the identifier for one of the assets
    setIdentifierByTag: function(tag, identifier){
        return this.setIdentifier(this.getIndexByTag(tag), identifier);
    },
    // Set the identifier for one of the assets
    setIdentifier: function(index, identifier){
        var valArray = this._util._getArrayObject(this.instanceObj, "Assets", "AssetInstance");
        valArray[index].identifier = identifier;
    },
    _getTemplate: function(){
        var cat = new com.ibm.asq.common.web.client.internal.Catalog();
        var templateName = this.parent.typeName;
        return cat.getTemplate(templateName);
    },
    // This method can be called to get all the names of 
    // all parts in the family. 
    getAllPartTypeNames: function(){
        return this._getTemplate().getPartNames();
    }, 
    // This method can be called to get the root part for the attribute
    // type from the type's template. 
    getRootPart: function(){
        return this._getTemplate().getRoot();
    }, 
    getNumberOfParts: function(){
        return this.partCache.length;  
    },
    // Determine if this attribute can contain an array of 
    // selected values.  If this method is true, the 
    // getArrayValues(), addArrayValue(), getArrayValue(),
    // setArrayValue() and removeArrayValue() methods become
    // valid. If this method is false, the getValue() and 
    // setValue() methods become valid. 
    isArray: function(){
        return this._isArray;
    },
    // Get the name of the attribute
    getName: function(){
        return "root";
    },
    // Determine if this attribute is an enumeration. 
    isEnum: function(){
        return false;
    },
    _getValue: function(returnNullObjects){
        if(returnNullObjects && this.instanceObj.Assets["null"] == "") {
            return {"null":""};
        }
        if(this.isArray()) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> getValue called on an array type root");
            throw new Error("getValue called on an array type root");
        }
        if(this.partCache.length == 0){
            return null;
        }
        else{
            return(this.partCache[0]);
        }
    },
    // Get the value of the attribute at the given array
    // index.  This method is only valid if the attribute
    // is an array.
    getArrayValue: function(index){
        return this.partCache[index];
    },
    // Set the value of an attribute.  This method is only 
    // valid if the attribute is not an array. It is an error 
    // to attempt to set the attribute's value to an object of 
    // the incorrect type.  If there is an existing value 
    // present, it will be replaced by the new value.
    setValue: function(object){
        if(this.isArray()) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> setValue called on an array type root");
            throw new Error("setValue called on an array type root");
        }
        if(object instanceof com.ibm.asq.common.web.client.internal._catalog.Part){
            return this._setValuePart(object);
        }
        return this._setValueRaw(object);
    },
    // set the value of an attribute of type part
    _setValuePart: function(part){
        // Save the old part in case it's swapped in later
        if (this.partCache.length != 0) {
            var oldPart = this.partCache[0];
            this.document_setSwappedPart(this.getName(), oldPart.getName(), oldPart);
        }
        this.partCache = [part];           
        this.instanceObj.Assets = {
            AssetInstance: part.instanceRoot
        };
    },
    // set the value of an attribute of type string, number or date
    _setValueRaw: function(object){
        // Check if we can delete that attribute
        if(object == null) {
            if (this.instanceObj != null) {
                this.instanceObj.Assets = null;
            }
            this.partCache = [];
            return;
        }
        // Get a formatted obj
        // Get the existing part if there is one
        var oldPart = this.getValue();
        // Check if new and old are the same
        if(oldPart != null){
            if (oldPart.getName() == object) {
                // Nothing to do
                return;
            }
        }
        // Get the part if it's available from the swap
        var newPart = this.document._getSwappedPart(this.getName(), object);
        // Save the old part in case it's swapped in later
        if (oldPart != null) {
            this.document._setSwappedPart(this.getName(), oldPart.getName(), oldPart);
        }
            
        // Check if we have this part saved
        if (newPart == null) {
            var part = object;
            if (dojo.isString(part)) {
                // Get the correct part from the template
                var template = this._getTemplate();
                part = template.getPart(object);
                if (part == null) {
                    console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> Part " + object + " not found in " + template.getName() + " template!");
                    throw new Error("Part not found in Template!");
                }
            }
            // Create a Part for the partCache
            var newPart = part._createInstancePart(this.document, null, this.parent.path);
            // Set the values from those in the old part
            if (oldPart != null) {
                newPart._setValuesFromPart(oldPart);
            }
        }
        this.partCache= [newPart];           
        var obj = this._createPartObj(newPart, false);
            
        // set the obj in the document
        if(this.instanceObj == null) {
            this.parent.instanceObj.Attributes.Attribute.push(obj);
            this.instanceObj = obj;
        } else {
            this.instanceObj.Assets = obj.value;
        }
    },
    // Set the value of the attribute at the given array
    // index.  If this method is called with a value of null,
    // the part at the corresponding index will be removed.
    // This method is only valid if the attribute
    // is an array.
    setArrayValue: function(index, value){
        if(value == null) {
            return this.removeArrayValue(index);
        }
        if(!(this.isArray())) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> getValues called on a non array type root");
            throw new Error("getValues called on a non array type root");
        }
        // return the array of values.  This may not be
        // an array type in the obj!
        
        // Get the part from the cache
        var part = this.partCache[index];
        // Check if new and old are the same
        if(part != null){
            if (part.getName() == value) {
                // Nothing to do
                return;
            }
        }
        // get the tag to trasfer to the new part
        var tag = part.tag;
        // Get the part if it's available from the swap
        var newPart = this.document._getSwappedPart(this.getName(), value);
        // Save the part in case it's swapped in later
        this.document._setSwappedPart(this.getName(), part.getName(), part);
        part.instanceRoot.remove = true;
        // Remove the object from the AssetInstance array
        var removed = false;
        var aiArr = this._util._getArrayObject(this.instanceObj, "Assets", "AssetInstance");
        for(var i=0;i<aiArr.length && !removed;i++){
            if(aiArr[i].remove == true){
                var newArr = this._util._removeFromArray(aiArr, index);
                this.instanceObj.Assets.AssetInstance = newArr;
                var ai = this.instanceObj.Assets.AssetInstance;
                if(ai.length == 0){
                    delete this.instanceObj.Assets.AssetInstance;
                }
                removed = true;
            }
        }
        // Check if we have this part saved
        if (newPart == null) {
            // Create a json object for the new attribute value
            var templatePart = value;
            if(dojo.isString(templatePart)) {
                // Get the correct part from the template
                var template = this._getTemplate();
                templatePart = template.getPart(value);
                if(templatePart == null) {
                    console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> Part not found in " + template().getName() + " template!");
                    throw new Error("Part not found in Template!");
                }
            }
            // Create a Part for the partCache
            newPart = templatePart._createInstancePart(this.document, null, this.parent.path);
            newPart.mark = this.markCounter;
            this.markCounter++;
        } else {
            // remove the swapped part 
            this.document._clearSwappedPart();
        }
        this.partCache[index] = newPart;           
        
        // Add the new json the the unresolved
        var val = this.instanceObj.Assets;
        if (val.AssetInstance == null) {
            val.AssetInstance = new Array();
        }
        
        // Set the values from those in the old part
        newPart._setValuesFromPart(part);
        newPart.tag = tag;
        
        aiArr = this._util._getArrayObject(this.instanceObj, "Assets", "AssetInstance");
        aiArr.push(newPart.instanceRoot);
    },
    // Remove the value of the attribute at the given array
    // index.  This method is only valid if the attribute
    // is an array.  This will re-index all of the values
    // in the array.
    removeArrayValue: function(index){
        if(!(this.isArray())) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> getValues called on a non array type root");
            throw new Error("getValues called on a non array type root");
        }
        // return the array of values.  This may not be
        // an array type in the obj!

        // Get the part from the cache
        var part = this.partCache[index];
        // Save the part in case it's swapped in later
        this.document._setSwappedPart(this.getType(), part.getName(), part);

        // Remove the object from the AssetInstance array
        part.instanceRoot.remove = true;
        var removed = false;
        var aiArr = this._util._getArrayObject(this.instanceObj, "Assets", "AssetInstance");
        for(var i=0;i<aiArr.length && !removed;i++){
            if(aiArr[i].remove == true){
                var newArr = this._util._removeFromArray(aiArr, index);
                this.instanceObj.Assets.AssetInstance = newArr;
                var ai = this.instanceObj.Assets.AssetInstance;
                if(ai.length == 0){
                    delete this.instanceObj.Assets.AssetInstance;
                }
                removed = true;
            }
        }
        // Remove the part from the cache
        this.partCache = this._util._removeFromArray(this.partCache, index);           
    },
    // Get the current values of the attribute.
    getArrayValues: function(){
        return(this.partCache);
    },
    // Add a value to the selection of values for the attribute. 
    // This method is only valid if the attribute is an array. 
    addArrayValue: function(object){
        if(object instanceof com.ibm.asq.common.web.client.internal._catalog.Part){
            return this._addArrayValuePart(object);
        }
        return this._addArrayValueRaw(object);
    },
    // Add a part to the selection of values for the attribute. 
    _addArrayValuePart: function(part){
		for(var j=0;j<this.partCache.length;j++){
			//dont add duplicates
			if(this.partCache[j].tag == part.tag){
				return;
			}
		}
        this.partCache.push(part);           
            if(this.instanceObj.Assets == null || this.instanceObj.Assets["null"] == ""){
                this.instanceObj.Assets = "";
            }
            var valArray = this._util._getArrayObject(this.instanceObj, "Assets", "AssetInstance");
            valArray.push(part.instanceRoot);
    },
    // Add a value to the selection of values for the attribute. 
    // This method is only valid if the attribute is an array. 
    _addArrayValueRaw: function(object){
        
        if (this.instanceObj.Assets == null || this.instanceObj.Assets["null"] == "") {
            this.instanceObj.Assets = "";
        }
        var valArray = this._util._getArrayObject(this.instanceObj, "Assets", "AssetInstance");
        var part = object;
        if (dojo.isString(part)) {
            // Get the correct part from the template
            var cat = new com.ibm.asq.common.web.client.internal.Catalog();
            var template = this._getTemplate();
            part = template.getPart(object);
            if (part == null) {
                console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> Part " + object + " not found in " + template.getName() + " template!");
                throw new Error("Part not found in Template!");
            }
        }
        // Create a Part for the partCache
        var newPart = part._createInstancePart(this.document, null, this.parent.path);
        this.partCache.push(newPart);
        valArray.push(newPart.instanceRoot);
    },
    _createPartObj: function(_part, isArray){
        var val = null;
        if(_part == null) {
            val = {'null':""};
        } else {
            if(isArray){
                val = { AssetInstance: [_part.instanceRoot] };
            } else {
                val = { AssetInstance: _part.instanceRoot };
            }
        }
        return {
            name: this._getTemplate().getName(),
            type: "ConfigurationPartType",
            value: val };
    },
    _buildPartCache: function(){
        // Need to get both resolved and unresolved
        
        // Get the correct template
        var template = this._getTemplate();
            
        var value = this.instanceObj.Assets;
        if (value == null) {
            return;
        }
        
        // Add the AssetInstance
        if (value.AssetInstance != null) {
            var aiArr = this._util._getArrayObject(this.instanceObj, "Assets", "AssetInstance");
            for (var i = 0; i < aiArr.length; i++) {
                if (aiArr[i] != null) {
                    // Get the correct part from the template
                    var at = aiArr[i].AssetType;
                    var part = template.getPart(at.name);
                    var instancePart = part._createInstancePart(this.document, aiArr[i], this.parent.path);
                    this.partCache.push(instancePart);
                }
            }
        }
    }
}   
)
})();
