/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//
// Author: Paul Beaulieu
//
// Summary:
//

(function(){
	
dojo.require("com.ibm.rational.test.lm.web.client.internal.lmContainer");
dojo.provide("com.ibm.rational.test.lm.web.client.internal.config");

/*
 * Non-API object used for interacting with the server's import REST service. 
 */
dojo.declare("com.ibm.rational.test.lm.web.client.internal.config", 
			  com.ibm.rational.test.lm.web.client.internal.lmContainer, 
    {
		constructor: function()
		{
			this.setServiceInterface('com.ibm.rational.test.lm.catalog.common.ISavedConfigurationRestService');
		},
		
		_getProjectAreaArgs: function()
		{
		  	return { projectAreaItemId: com.ibm.team.process.web.projectArea.itemId };
		},
		
        // Error string "property constraint: name" in the errorMessage field of the response indicates a name constraint violation.
		createConfiguration: function( responseHandler, config, configName, configDesc, configSummary )
		{
			var jsonConfig = config.toJson();
            if(!configSummary){
                configSummary = "";
            }

			var args = this._getProjectAreaArgs();
			
			args["configuration"] = jsonConfig;
			args["name"] = configName;
			args["description"] = configDesc;
			args["summary"] = configSummary; 

			return( this._invokeService( responseHandler, 'postCreateConfiguration', args ) );	
		},

        // Error string "property constraint: name" in the errorMessage field of the response indicates a name constraint violation.		
		updateConfiguration: function( responseHandler, id, stateId, config, configName, configDesc, configSummary )
		{
			var jsonConfig = config.toJson();
            if(!configSummary){
                configSummary = "";
            }
			var args = {'id': id, 'stateId': stateId, configuration: jsonConfig, name: configName, description: configDesc, summary: configSummary };
			return( this._invokeService( responseHandler, 'postUpdateConfiguration', args ) );				
		},
		
		deleteConfigurations: function( responseHandler, uuid )
		{
			var arrUUID = [];
			
			if( !dojo.isArray( uuid ) )
			{
				arrUUID[ 0 ] = uuid;
			}
			else
			{
				arrUUID = uuid;
			}
			
			var args = {'uuid': arrUUID};
			return( this._invokeService( responseHandler, 'postDeleteConfigurations', args ) );				
		},
		
		retrieveConfiguration: function( responseHandler, uuid )
		{
			return( this._retrieveConfiguration( responseHandler, 'getRetrieveConfiguration', uuid ) ); 
		},

		retrieveConfigurationWithDetails: function( responseHandler, uuid )
		{
			var args = {'uuid': uuid};
			return( this._retrieveConfiguration( responseHandler, 'getRetrieveConfigurationWithDetails', uuid ) ); 
		},
		
		_retrieveConfiguration: function( responseHandler, method, uuid )
		{
			var args = {'uuid': uuid};
			return( this._invokeService( responseHandler, method, args ) );			
		},
		
        getConfigurationsWithDetails: function( responseHandler, uuids )
        {
            var args = {'uuid': uuids};
            return( this._invokeService( responseHandler, 'getConfigurationsWithDetails', args ) ); 
        },
        
		getMatchingConfiguration: function( responseHandler, config )
		{
			var jsonConfig = config.toJson();
			var args = this._getProjectAreaArgs();
			
			args["configuration"] = jsonConfig;
			
			return( this._invokeService( responseHandler, 'getMatchingConfiguration', args ) );			
		},
		
		getAllWithDetails: function( responseHandler )
		{
			return( this._invokeService( responseHandler, 'getAllWithDetails', this._getProjectAreaArgs() ) );						
		},
		
		getAll: function( responseHandler )
		{
			return( this._invokeService( responseHandler, 'getAll', this._getProjectAreaArgs() ) );						
		}

	}
);

})();
