/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.asq.common.web.client.internal.transport.RawServiceRequest");

dojo.declare("com.ibm.asq.common.web.client.internal.transport.RawServiceRequest", null, {
    
    // -------------------------------------------------------------------------
    // API Methods
    // -------------------------------------------------------------------------
    
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    /**
     * Represents an request for a Raw service.
     */
    constructor: function(/* object */ serviceInterface, 
	                      /* string */ method, 
						  /* string */ requestParams, 
                          /* object */ additionalHeaders, 
						  /* string */ data) {
        var srClass= com.ibm.team.repository.web.transport.ServiceRequest;
        var srClassName= "com.ibm.team.repository.web.transport.ServiceRequest()";
        
        if (arguments.length < 4 || arguments.length > 5) {
            throw new Error(srClassName + ": invalid number of parameters: " + arguments.length);
        }
        
        // validate input; this will throw an exception if illegal input is discovered
        srClass._validateInput(serviceInterface, method, requestParams);
        
        this._serviceInterface= serviceInterface;
        this._method= method;
        this._params= srClass._makeAllParamsArrays(requestParams);
        this._additionalHeaders= additionalHeaders;
		this._data= data;
    },
    
    /**
     * Aborts the service request, causing the response to be ignored and
     * freeing the connection. The response handler will not be called.
     * Has no effect if called on a request that hasn't yet been issued or
     * has already received its response.
     */
    abort: function() {
        if (this._xhr) {
            this._xhr.abort();
        }
    },
    
    /**
     * Returns an array of strings containing the names of the parameters
     * contained in this request. If the request has no parameters, the method
     * returns an empty array.
     *
     * @return an array of strings containing the requests request parameter
     *  names, or an empty array if the request has no parameters.
     */
    getParameterNames: function() {
        var paramNames= [];
        for (var prop in this._params) {
            paramNames.push(prop);
        }
        return paramNames;
    },
    
    /**
     * Returns the extra path info that follows a request's team server service
     * context path (e.g. /jazz/service) but precedes its query string.  Always
     * starts with the '/' character.
     */
    getMethod: function() {
        return this._method;
    },
    
    /**
     * Returns this service request's method type as a string; either "GET" or
     * "POST".
     *
     * @return this service request's method type as a string. 
     */
    getMethodType: function() {
        var regex= /^(get|post)/;
        var result= this._method.match(regex)[0].toUpperCase();
        return result;
    },
    
    /**
     * Returns an array of strings containing all of the values the given
     * request parameter has, or <code>null</code> if the parameter does not
     * exist.
     *
     * @return an array of strings containing the parameter's values.
     */
    getParameterValues: function(name) {
        var srcParamVals= this._params[name];
        if (!srcParamVals) {
            return null;
        }
        var copyParamVals= [];
        for (var i=0; i<srcParamVals.length; i++) {
            copyParamVals.push(srcParamVals[i]);
        }
        return copyParamVals;
    },
    
    /**
     * Returns addtional headers.
     *
     * @return the additionalHeaders object.
     */
    getAdditionalHeaders: function() {
        return this._additionalHeaders;
    },
    
    /**
     * Returns the data that will be used in the request body.
     *
     * @return the data string.
     */
    getData: function() {
        return this._data;
    },
    
    /**
     * Returns the extra path info that follows a request's team server service
     * context path (e.g. /jazz/service) but precedes its query string.  Always
     * starts with the '/' character.
     */
    getServiceInterface: function() {
        return this._serviceInterface;
    },
    
    /**
     * Returns the request's paremeters, represented as a query string, or
     * <code>null</code> if the request does not contain request parameters.
     * Encodes any illegal URL characters with their ASCII code equivalents.
     * <p>
     * TODO: Need to consider providing an optional encoding parameter to
     * support both ASCII and UTF-8 URL encoding (see dojo.io.argsFromMap for an
     * example).
     * </p>
     *
     * @return the request's query string, or <code>null</code> if the request
     *  does not have request parameters.
     */
    getQueryString: function() {
        return dojo.objectToQuery(this._params);
    },

    // ----------------------------------------------------------------------------
    // Internal methods.  *Not* API - do not use!
    // ----------------------------------------------------------------------------
    
    /*
     * (Internal) Called when the request is initiated to set the associated
     * XMLHttpRequest, which is needed to support the abort() feature.
     */
    _setXMLHttpRequest: function(/* XMLHttpRequest */ xhr) {
        this._xhr = xhr;
    }
});

