/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.asq.common.web.client.internal.transport.RawTeamServerClient");

dojo.require("net.jazz.ajax.core.internal.util");
dojo.require("com.ibm.team.repository.web.ui.internal.LoginWidget");
dojo.require("dojo.cookie");

//
// Provides a generic remoting mechanism to the team server's Raw services.
//

//
// Aliases
//

var callContributedCode = net.jazz.ajax.core.internal.util.callContributedCode;

/**
 * Invokes the service specified by <code>serviceRequest</code>
 * asyncronously and calls the appropriate function specified by
 * <code>serviceResponseHandler</code> in the context of
 * <code>serviceResponseHandler</code>.
 *
 * If the service returns successfully, invokeService will call back to
 * the function defined by handler.getCallbackName(); if the service call
 * results in an error, invokeService will call back to the function defined
 * by handler.getErrorCallbackName().
 */
function invokeService(serviceRequest, serviceResponseHandler) {
    var bindArgs = _getBindArgs(serviceRequest, serviceResponseHandler);
    // check for the secret sync parameter for unit testing
    if (arguments[2] === true) {
        bindArgs.sync= true;
    }
    var xhr = null;
    if (serviceRequest.getMethodType() == "POST") {
        bindArgs.postData = serviceRequest.getData();
        xhr = dojo.rawXhrPost(bindArgs).ioArgs.xhr;
    }
    else {
        xhr = dojo.xhrGet(bindArgs).ioArgs.xhr;
    }
    serviceRequest._setXMLHttpRequest(xhr);
}

com.ibm.asq.common.web.client.internal.transport.RawTeamServerClient.invokeService = invokeService;

//
// Internal consts and functions
//

var SERVICE_PREFIX   = dojo.baseUrl + "../../service/";
var X_AUTH_MSG = "X-com-ibm-team-repository-web-auth-msg";
var USERID_COOKIE = "JazzLoginCookie";

function _getBindArgs(serviceRequest, serviceResponseHandler) {
    var bindArgs = {
        url: _assembleURL(serviceRequest),
        handleAs: "json",
        headers: _createHeaders(serviceRequest.getAdditionalHeaders()),
        preventCache: dojo.isIE,
		postData: serviceRequest.getData(),
        handle: function(response, ioArgs){
            if (response instanceof Error) {
                // For work item 18631 ('Propogate server exception details to
                // REST service clients'), we can access the error response's
                // body via xhr.responseText.  Will probably have to manually
                // demarshall this data here, since I don't believe Dojo
                // provides any auto-demarshalling on error paths.

                // temporary fix for Defect 32729
                // test data (what we expect WebSphere to send back on ITEs)
                //ioArgs.xhr = {
                //  status: 500,
                //  statusText: "Internal Server Error",
                //  responseText: "Error 500: -- &#39;Save Work Item&#39; was blocked by process.\nThe work item is not assigned to a category.\n"
                //}
                var errorObj = {
                    status: ioArgs.xhr.status,
                    message: null,
                    exception: undefined
                };
                if (ioArgs.xhr.statusText === "Internal Server Error" || ioArgs.xhr.statusText === "Bad Request") {
                    var errorResponse = ioArgs.xhr.responseText || "";
                    errorResponse = dojo.fromJson(errorResponse);
                    if (dojo.isObject(errorResponse)) {
                        errorObj.message = "["+errorResponse.errorCode + "] : " + errorResponse.errorMessage;
                        errorObj.exception = errorResponse;
                    }
                    else {
                        errorObj.message = errorResponse;
                        if (errorObj.message.indexOf("-- ") > 0 && errorObj.message.length > 3) {
                            errorObj.message = errorObj.message.substring(errorObj.message.indexOf("-- ") + 3);
                        }
                        errorObj.message = errorObj.message.replace(/&#39;/g, "\"");
                    }
                } else if (ioArgs.xhr.status == 401 && ioArgs.args.url.match(/^.*IWebUIInitializerRestService\/initializationData.*$/)) {
                    // Capture the scenario where the user cancels out of a Basic auth prompt and display a helpful message
                    // rather than a blank "Loading..." screen
                    var errMsg = "Authentication is required to access the requested resource.  Please refresh the page to attempt authentication again." 
                    errorObj.message = errMsg;                  
                } else if (ioArgs.xhr.status == 403 && ioArgs.args.url.match(/^.*IWebUIInitializerRestService\/initializationData.*$/)) {
                    // Capture the scenario where a user account does not belong to the groups required to access the Web UI
                    // Provide a helpful error message rather than a blank "Loading..." screen.
                    var errMsg = "Your account does not belong to the groups that are authorized to access the Web UI.  Please contact your server administrator.";
                    errorObj.message = errMsg;
                } else {
                    errorObj.message = ioArgs.xhr.statusText;
                }
                
                errorObj.message = dojo.string.trim(unescape(errorObj.message).replace(/\+/g, " ").replace(/^(-*)/g, ""));
                    
                serviceResponseHandler.doErrorCallback(errorObj);
            } else if (ioArgs.xhr.getResponseHeader(X_AUTH_MSG) == "authrequired") {
                _loginPrompt(this, serviceResponseHandler, serviceRequest.getMethodType().toLowerCase());
            } else {
                callContributedCode( function() {
                    serviceResponseHandler.doCallback(response);
                });
            }           
        }
    };
    return bindArgs;
}

function _loginPrompt(bindArgs, serviceResponseHandler, requestMethod, noQueue, errMessage) {
    var workbenchNode = dojo.byId("workbench");
    var dialogWidget = dijit.byId("com-ibm-team-repository-internal-LoginWidgetDialog");
    if (dialogWidget == null) {
        var loginWidget = new com.ibm.team.repository.web.ui.internal.LoginWidget(null, workbenchNode);
        dialogWidget = new dijit.Dialog({id:"com-ibm-team-repository-internal-LoginWidgetDialog",
            execute: dojo.hitch(this, function(dialogArgs) {
                // allow the dialogWidget to be closed
                dialogWidget._alreadyInitialized = true;
                dialogWidget.hide();
                dialogWidget.loginState = "inactive";
                if (dialogArgs.rememberUserID == "on") {
                    dojo.cookie(USERID_COOKIE, dialogArgs.userid, { expires: 365 });
                } else {
                    dojo.cookie(USERID_COOKIE, "");
                }
                _login(dialogArgs.userid, dialogArgs.password, bindArgs, serviceResponseHandler, requestMethod);
            })
        }, loginWidget.domNode);
        dialogWidget.loginWidget = loginWidget;
        dialogWidget.loginState = "inactive";
        loginWidget.connect(loginWidget.loginButton,"onclick",function(event) {
            dialogWidget.execute(dialogWidget.getValues());
        });
        loginWidget.connect(loginWidget.passwordNode.domNode,"onkeypress",function (event) {
            // If the user hits "Enter" in the password field, submit the data
            if (event.keyCode == dojo.keys.ENTER) {
                dialogWidget.execute(dialogWidget.getValues());
            }
        });
    }
    if (errMessage != null) {
        dialogWidget.loginWidget.showErrorMessage(errMessage);
    }
    else {
        dialogWidget.loginWidget.hideErrorMessage();
    }
    var storedUserId = dojo.cookie(USERID_COOKIE);
    if (storedUserId) {
        dialogWidget.loginWidget.setUserId(storedUserId);
        dialogWidget.loginWidget.rememberUserIdNode.setChecked(true);
        var i = 1;
    } else {
        var i = 0;
    }
    if (dialogWidget.loginState == "inactive") {
        document.title = "Login Required - Jazz Team Server";
        dialogWidget.show();
        setTimeout(dojo.hitch(dialogWidget, function(){
            dijit.focus(dojo.query('input', this.domNode)[i]); 
        }), 50);
        // prevent the user from closing the dialogWidget without submitting
        dialogWidget._alreadyInitialized = false;
        dialogWidget.loginState = "active";
    }
    if (dialogWidget.queuedRequests == null) {
        dialogWidget.queuedRequests = new Array();
    }
    if (noQueue == null || !noQueue) {
        dialogWidget.queuedRequests.push({
            bindArgs: bindArgs,
            requestMethod: requestMethod,
            responseHandler: serviceResponseHandler
        });
    }
}

function _login(userid, password, bindArgs, serviceResponseHandler, requestMethod) {
    this._previousRequest = bindArgs;
    var bindArgs= {
        url: "j_security_check",
        content: {
            j_username: userid,
            j_password: password
        },
        handleAs: "json",
        headers: {
            accept: "text/json"
        },
        handle: dojo.hitch(this, function(response, ioArgs){
            if (response instanceof Error) {
                var errorObj= {
                    status: ioArgs.xhr.status
                };
                if (ioArgs.xhr.status == 403 && this._previousRequest.url.match(/^.*IWebUIInitializerRestService\/initializationData.*$/)) {
                    // Capture the scenario where a user account does not belong to the groups required to access a requested resource
                    // Provide a helpful error message rather than a blank "Loading..." screen.
                    var errMsg = "Your account does not belong to the groups that are authorized to access the Web UI.  Please contact your server administrator";
                    alert(errMsg);
                    errorObj.message = errMsg;
                }
                // format detailed message (unescape URI-encoding formatting, remove leading dashes, and trim whitespace)
                errorObj.message= dojo.trim(unescape(ioArgs.xhr.statusText).replace(/\+/g, " ").replace(/^(-*)/g, ""));
                serviceResponseHandler.doErrorCallback(errorObj);
            }
            else if (ioArgs.xhr.getResponseHeader(X_AUTH_MSG) == "authfailed") {
                var dialogWidget = dijit.byId("com-ibm-team-repository-internal-LoginWidgetDialog");
                dialogWidget.loginWidget.passwordNode.domNode.value = "";
                _loginPrompt(bindArgs, serviceResponseHandler, requestMethod, true, "Incorrect User name or Password");
            }
            else {
                var dialogWidget = dijit.byId("com-ibm-team-repository-internal-LoginWidgetDialog");
                if (dialogWidget.queuedRequests != null) {
                    while (dialogWidget.queuedRequests.length > 0) {
                        var request = dialogWidget.queuedRequests.shift();
						var xhr;
                        if (request.requestMethod == "post") {
                            xhr = dojo.rawXhrPost(request.bindArgs);
                        }
                        else {
                            xhr = dojo.xhrGet(request.bindArgs);
                        }
                    }
                }
            }
        })
    };
    dojo.xhrPost(bindArgs);
}

/*
 * Assembles a URL, appending a query string to GET queries.  Handles both
 * non-secure and secure contexts.
 */
function _assembleURL(serviceRequest) {
    var method = serviceRequest.getMethod();
    var methodType = serviceRequest.getMethodType();
    var urlResource =
        method.substring(methodType.length, methodType.length + 1).toLowerCase() +
        method.substring(methodType.length + 1);
    
    // assemble basic url
    var url = _getServicePrefix() + serviceRequest.getServiceInterface() + "/" + urlResource;
    
    // append the query string (this is used for the projectAreaId
    var tmpQueryString = serviceRequest.getQueryString();
    var queryString = tmpQueryString ? ("?" + tmpQueryString) : "";
    url += queryString;

    return url;
}

/*
 * Returns the correct service prefix based on the browser's security
 * context (e.g. http or https).
 */
function _getServicePrefix() {
    return SERVICE_PREFIX;
}

/*
 * Creates the headers adding in any addional headers.
 */
function _createHeaders( additionalHeaders ) {
	var headers = {};
	if (additionalHeaders != null) {
		headers = additionalHeaders;
	}
    headers["accept"] = "text/json";
    headers["content-type"] = "text/json; charset=utf-8";
    return headers;
}

// export for the sake of workitem.web, which depends on this (though it shouldn't)
com.ibm.asq.common.web.client.internal.transport.RawTeamServerClient._getServicePrefix = _getServicePrefix;

})();