/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.page.print.PrintPageView");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper")

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.page.print.PrintPageController");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.ArtifactModel");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.planning.TestPlanEditor");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.TestCaseEditor");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.TestScriptEditor");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient");

(function()
{	
	var PlatformUI			= net.jazz.ajax.ui.PlatformUI;	
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var Constants           = com.ibm.rqm.planning.web.client.internal.Constants;
	var ArtifactModel       = com.ibm.rqm.planning.web.ui.internal.model.ArtifactModel;
	var TestScriptClient    = com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient;
	var root                = "com.ibm.rqm.planning.printview.";
	
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.page.print.PrintPageView", [dijit._Widget, dijit._Templated],
	{      
		controller: null,      
		templateString: '<div class="com-ibm-rqm-printable-artifact-view">' +
		                '    <div dojoAttachPoint="_contentNode"></div>' +
		                '</div>',
		
		_actions: ["testPlan", "testCase", "testScript"],			
			
		/*
		 * Required by Jazz web framework to make hyperlinking and back/forward
		 * work.
		 */
		getController: function() {
			return this.controller;
		},
		
		// initialization      
		postCreate: function() {
			this.controller = new com.ibm.rqm.planning.web.ui.internal.page.print.PrintPageController(this);
            
			// register actions			
            var actionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			for (var i = 0; i < this._actions.length; i++) {
				var actionName = this._actions[i];
		        actionRegistry.registerAction(root + actionName, this.controller, actionName);
			}					
            //dojox.data.dom.removeChildren(this._contentNode);									
		},
		
        viewTestPlan: function(uriArgs) {		
            var artifactModel = new ArtifactModel(Constants.TYPE_PLAN);
            var editor = new com.ibm.rqm.planning.web.ui.internal.view.planning.TestPlanEditor({
                parentController: this,
                model: artifactModel,
                kwArgs: uriArgs,
				printable: true
            });
            dojox.data.dom.removeChildren(this._contentNode);
            this._contentNode.appendChild(editor.domNode);
        },
        
        viewTestCase: function(uriArgs) {		
            var artifactModel = new ArtifactModel(Constants.TYPE_CASE);
            var editor = new com.ibm.rqm.planning.web.ui.internal.view.construction.TestCaseEditor({
                parentController: this,
                model: artifactModel,
                kwArgs: uriArgs,
                printable: true
            });
            dojox.data.dom.removeChildren(this._contentNode);
            this._contentNode.appendChild(editor.domNode);			
        },
		
        viewTestScript: function(uriArgs) {       
            // create the editor to show loading in progress in a new tab and reinitialize the editor
            // after the data is fetched from the server
            var artifactModel = new ArtifactModel(Constants.TYPE_SCRIPT);
            var editor = new com.ibm.rqm.planning.web.ui.internal.view.construction.TestScriptEditor();
            editor.setArtifactModel(artifactModel);
            var self = this;
            var handler = {
                success: function(result) {
	                if (!result || result.length != 1) {
	                    alert(Constants.getMessage("errRetrieveTestScript"));
	                    return;
	                }
					if (dojo.isArray(result)) {
	                    result = result[0]; 
	                }
	                // initialize will implicitly update the tab button label
	                editor.initialize({
						parentController: self, 
						testScript: result, 
						printable: true
					});							
                },
                error: function(e) {
                    self._error(e);
                }
            }
            var responseHandler = new ServiceResponseHandler(handler, "success", "error");          
            TestScriptClient.getTestScript(responseHandler, uriArgs.id);			
			
            dojox.data.dom.removeChildren(this._contentNode);
            this._contentNode.appendChild(editor.domNode);          
        },
		
		_error: function(e) {
			console.error(this.declaredClass + ": Error occurred: [" + e.message);
		}
	});
})();
