/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.reports.web.ui.internal.view.ReportViewer");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.app.view.report.ReportViewer");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.reports.web.ui.internal.view.SaveReport");

(function()
{   
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var PlatformUI = net.jazz.ajax.ui.PlatformUI;
    var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;    
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    
    dojo.declare("com.ibm.rqm.reports.web.ui.internal.view.ReportViewer",  
    [dijit._Widget, dijit._Templated, com.ibm.team.reports.web.app.view.report.ReportViewer], 
    {
                   
        templateString:   '<div class="com-ibm-rqm-reports">' +
                          '    <div dojoAttachPoint="_renderedReportNode" class="com-ibm-team-reports">' +
                          '        <!-- Copied from Jazz ReportsViewer.js template -->' +
                          '        <div class="report-viewer">' +
                          '            <div id="parametersDiv" class="parametersDiv" dojoAttachPoint="parametersDiv"></div>' +
                          '            <div id="loadingDiv" class="loading" dojoAttachPoint="loadingDiv"></div>' +
                          '            <div id="renderedReportDiv" class="renderedReport" dojoAttachPoint="renderedReportDiv"></div>' +
                          '        </div>' +                          
                          '    </div>' +
                          '</div>',       
		
        labelLoading: Constants.getMessage("labelLoading"),
		labelLoadingReport: Constants.getMessage("labelLoadingReport"),
		
        /*
         * Required by Jazz web framework to make hyperlinking and back/forward
         * work.
         */
        constructor: function() {
            this.autoRun = false;     //auto-run report on load with default args
            this.autoExpand = true;   //auto-expand the parameters section; 
            this.isReport = false;
            this._gotParametersResponse = false;
            this._gotRenderedResponse = false;
            
            this.uriArgs = null;
            this._view = null;
            this.reportUUID = null;
            this.queryUUID = null;
            this.title = null;          
            this._viewer = null;
            this._includeArchived = false;
        },
        
        //@Override from Jazz's ReportsViewer.js
        postCreate: function() {			
            this.controller = this;
			//INTENTIONAL GLOBAL VARIABLE USAGE - set global reportsController for Jazz reports widgets 
			reportController = this.controller;			
			
            if (this.reportUUID != null) {
                this.isReport = true
            }			
			if (this.autoRun) {
				this.autoExpand = false;
			}
			this._setHasResponse(false);			
        },

        initialize: function() {
            this.renderParameters();			   
            if (this.autoRun) {
                this.render();
            }  
        },
                    
        setView: function(view) {
            this._view = view;          
            this._updateView();
        },      
        
        _updateView: function() {
            if (this._view && this._view.controlButton) {
                //apply the tab class
                dojo.addClass(this._view.controlButton.containerNode, "report-icon-tab");
                //TODO update the tab label
            }           
        },
		
		_showSaveAs: function() {
			var saveAsWidget = new com.ibm.rqm.reports.web.ui.internal.view.SaveReport({
                reportUUID: this.reportUUID,
				queryUUID: this.queryUUID,
				onOk: dojo.hitch(this, "_saveAsFinished") 	
			});			
		},
		
		_saveAsFinished: function(newQueryUUID) {
			//if a new query has been created, need to re-draw the parameters section,
			//in order to enable the 'Save' function
			if (newQueryUUID != null) {
				console.debug(this.declaredClass + ": created new Query: " + newQueryUUID);
				this.isReport = false;
				this.reportUUID = null;
				this.autoRun = false; //do not re-run report
				this.autoExpand = false;
				this.queryUUID = newQueryUUID;
				this.initialize();
			}
		},
        
        renderParameters: function(runReport) {
			this._started = new Date();
			console.info(this.declaredClass + ": renderParameters: entry --> " + this._started);
            this._removeChildren(this.parametersDiv);       
            this._gotParametersResponse = false;
			this.loadingDiv.innerHTML = this.labelLoading;   
            var responseHandler = new ServiceResponseHandler(this, "_displayParametersSection", "_remoteError");            
            if (this.isReport) {
                var renderArgs = this._getRenderReportArgs();
                ReportsClient.getReportParametersSection(responseHandler, renderArgs);
            } else {
                var renderArgs = this._getRenderQueryArgs();
                ReportsClient.getQueryParametersSection(responseHandler, renderArgs);               
            }     
        },
        
        _displayParametersSection: function() {
			console.info(this.declaredClass + ": _displayParametersSection: entry --> " + (new Date().getTime() - this._started.getTime()) + " ms");         
			this.inherited('_displayParametersSection', arguments);
			console.info(this.declaredClass + ": _displayParametersSection: finished setting dom --> " + (new Date().getTime() - this._started.getTime()) + " ms");
			dojox.data.dom.removeChildren(this.loadingDiv);   
			this._setHasResponse(this._gotRenderedResponse); 
			dojo.connect(null, "handleRun", null, dojo.hitch(this, "_setHasResponse", false));
			dojo.connect(null, "mySuccessFunction", null, dojo.hitch(this, "_setHasResponse", true));
			
			//must be performed AFTER parameters html added to DOM
			if (this.autoExpand) {
				this._expandParametersSection();
			}
			this._showCSHelp();

            //redefine saveAs
            saveAs = dojo.hitch(this, "_showSaveAs"); //INTENTIONAL GLOBAL NAMESPACE USAGE	

            // Not using the set default action. Remove it from
            // view.
            var setDefAction = dojo.byId("setAsDefault");
            if(setDefAction)
                dojo.style(setDefAction, "display", "none");
        },  
		
        _expandParametersSection: function() {
            var twisty = dojo.byId("twistyLink"); //$NON-NLS$
            if (twisty) {
                if (document.createEvent) {
                    var event = document.createEvent('MouseEvents');
                    event.initEvent('click', true, true);
                    twisty.dispatchEvent(event);                    
                } else if (document.createEventObject) {
                    //var event = document.createEventObject();
                    twisty.click();
                    
                } else {
                    return; //unsupported browser
                }
            }           
        },		
		
		_showCSHelp: function() {
            var cshelp = new com.ibm.asq.common.web.ui.internal.util.CSHelp({
                //contextID: "rtlm_rpt_req_by_state.html",
				contextID: "report_" + this.reportId + ".html"
            });
            
            var dirtyMarker = document.getElementById("dirtyMarker");
            var helpNode = dirtyMarker.parentNode;
			helpNode.appendChild(cshelp.domNode);   			
		},
				
		_setHasResponse: function(hasResponse) {
			this._gotRenderedResponse = hasResponse;
			if (this._gotParametersResponse) {
				var exportImg = dojo.byId("export");
				exportImg.style.display = this._gotRenderedResponse ? "inline" : "none";
			}
		}, 
        
        render: function() {
            if (this.isReport) {
                this.renderReport();
            } else {
                this.renderQuery();
            }           
        },

        /*
         * Create or Refresh the Report widget 
         */
        renderReport: function() {
            // Handle non-query reports         
			this.loadingDiv.innerHTML = this.labelLoadingReport;
            dojox.data.dom.removeChildren(this.renderedReportDiv);
            this._setHasResponse(false);
            var responseHandler = new ServiceResponseHandler(this, "_displayRenderedQuery", "_remoteError");
            var renderedReportArgs = this._getRenderReportArgs();
            ReportsClient.postRenderReport(responseHandler, renderedReportArgs);
            return;
        },
                    
        /*
         * Create or Refresh the Report widget 
         */
        renderQuery: function() {
            /*
            dojox.data.dom.removeChildren(this._renderedReportNode);
            this._viewer = new ReportViewer({queryUUID: this.queryUUID, isReport: false, controller: this});
            this._renderedReportNode.appendChild(this._viewer.domNode);
            */
            
            // Now handle regular query
			this.loadingDiv.innerHTML = this.labelLoadingReport;
            dojox.data.dom.removeChildren(this.renderedReportDiv);            
            this.controller.startOperation("Loading Report");
            this._setHasResponse(false);
            var responseHandler = new ServiceResponseHandler(this, "_displayRenderedQuery", "_remoteError");
            var renderQueryArgs = this._getRenderQueryArgs();
            ReportsClient.getRenderedQuery(responseHandler, renderQueryArgs);           
        },
                
        _getRenderReportArgs: function() {
            var renderReportArgs = {
                reportUUID: this.reportUUID,
                parameterName: this.parameterName ? this.parameterName : [],
                parameterValue: this.parameterValue ? this.parameterValue : [],
                showArchived: this.controller.getIncludeArchived(),
                webui: true
            };          
            return renderReportArgs;
        },
        
        _getRenderQueryArgs: function() {
            var renderedQueryArgs = {
                queryUUID : this.queryUUID,
                showArchived: this.controller.getIncludeArchived(),
                webui: true
            };     
            return renderedQueryArgs;
        },  
        
        //*******************************************************************************
        //             Overridde from Jazz's ReportsController to allow reuse of 
        //                    Jazz's ReportsViewer widget
        //*******************************************************************************
        /**
         * Marks the start of a long-running operation (needs more documentation).
         * May be called by code within the com.ibm.team.workitem.web bundle.
         */
        startOperation: function(message) {
            PlatformUI.getWorkbench().showMessage(message);
        },
    
        /**
         * Marks the end of a long-running operation (needs more documentation).
         * May be called by code within the com.ibm.team.workitem.web bundle.
         */
        finishOperation: function(operation) {
            PlatformUI.getWorkbench().hideMessageArea();
        },
        
        /**
         * Return whether archived things should be
         * included in reports
         */
        getIncludeArchived: function() {
            return this._includeArchived;
        },
        
        /**
         * Set whether archived things should be
         * included in reports
         */
        setIncludeArchived: function(includeArchived) {
            this._includeArchived = includeArchived;
        },      
        
        refreshNavigator: function() {
            //do nothing; not using navigator
        },
        
        setDefaultQuery: function(defaultQuery) {
            //do nothing; not using default
        },
        //*******************************************************************************
                
        _error: function(e) {
            console.error(this.declaredClass + ": Error: " + e.message);
        }   

    });
})();
