/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.reports.web.ui.internal.view.SaveReport");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");

(function(){
    
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
	
    dojo.declare("com.ibm.rqm.reports.web.ui.internal.view.SaveReport", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.reports.web", "ui/internal/view/templates/SaveReport.html"),
		
		labelName: Constants.getMessage("labelName"),
		labelDescription: Constants.getMessage("labelDescription"),
        labelOk: Constants.getMessage("labelOk"),
		labelCancel: Constants.getMessage("labelCancel"),
		labelTheme: Constants.getMessage("labelTheme"),
		labelLoading: Constants.getMessage("labelLoading"),
		labelMyReports: Constants.getMessage("labelMyReports"),

        width: "650px",
        title: Constants.getMessage("labelSaveReportAs"),
		
        constructor: function(args) {
			this.onOk = null;
			this.isExistingReport = false;
			this.reportUUID = null;
			
			//internal storage
			this._responses = {}; //track which responses we've received
			this._queries = null;
			this._folders = null;
        },
		
		postCreate: function() {     
            this._messages.style.display = "none";
			if (this.reportUUID == null && this.queryUUID == null) {
                throw new Error("One of 'reportUUID' or 'queryUUID' is REQUIRED.");
            }
			
            var args = {
                // Project Area will be added by ReportsClient
                flags: ReportsClient.ALL
            };			
		
            //have to get all - no API for getting single queryDescriptor               
            var queryHandler = new ServiceResponseHandler(this, "_setQueries", "_error");            
            ReportsClient.getQueryDescriptors(queryHandler, args);
			
			var folderArgs = {
                // Project Area will be added by ReportsClient
                flags: ReportsClient.SHARED
            };            
            var foldersHandler = new ServiceResponseHandler(this, "_setSharedFolders", "_error");
            ReportsClient.getFolderDescriptors(foldersHandler, folderArgs);			
			
			
            var reportsHandler = new ServiceResponseHandler(this, "_setReports", "_error");
            ReportsClient.getReportDescriptors(reportsHandler, args);   
		},
		
        _setQueries: function(queries) {
            //console.debug(this.declaredClass + ":_setQueries --> entry.");
			this._responses.queries = true;
            this._queries = queries ? queries : [];
            if (this._isDataInitialized()) {
                this._initialize();
            }
        },
		
        _setReports: function(reports) {
            //console.debug(this.declaredClass + ":_setReportss --> entry.");
            this._responses.reports = true;
            this._reports = reports ? reports : [];
            if (this._isDataInitialized()) {
                this._initialize();
            }
        },		
        
		_setSharedFolders: function(folders) {
			console.debug(this.declaredClass + ":_setSharedFolders --> entry.");
			this._folders = folders || [];
			var folderArgs = {
                // Project Area will be added by ReportsClient
                flags: 4,  //private folder
				ownerUUID: com.ibm.rqm.planning.web.ui.internal.util.EditorUtil.getCurrentContributor().itemId
            };
			var foldersHandler = new ServiceResponseHandler(this, "_setFolders", "_error");
            ReportsClient.getFolderDescriptors(foldersHandler, folderArgs);
		},
		        
        _setFolders: function(privateFolders) {
            console.debug(this.declaredClass + ":_setFolders --> entry.");
			this._responses.folders = true;
			this._folders = this._folders.concat(privateFolders || []);
			this._folders.sort(this._sortFolders)
            if (this._isDataInitialized()) {
                this._initialize();
            }           
        },	
                
        _isDataInitialized: function() {
            return (this._responses.folders && this._responses.queries && this._responses.reports);
        },      
		
		
		_initialize: function() {			
            if (this.queryUUID != null) {
                this._query = this._findQuery(this.queryUUID, "queryUUID");
            } else {
				var report = this._findReport(this.reportUUID, "reportUUID", "reportId");
                this._query = this._findQuery(report ? report.reportUUID : "", "reportUUID");
				if (this._query == null) {
					this._error({message: "Unable to locate Report or Query Descriptor for reportId: " + this.reportUUID});
				}		 	
			}
			
			//set initial form values
            this._name.value = this._query.name;
			this._description.value = this._query.description;
			
			//show the dialog
            this._showAsDialog();
            setTimeout(dojo.hitch(this, function(){
				this._name.focus();
			}), 250);		
			
			//show folder selection
            dojox.data.dom.removeChildren(this._folderSelect);
            //if new & no 'private' folders exist, show 'My Reports' placeholder (null folder will be sent to server)
			if (!this.isExistingReport && !this._hasPrivateFolders()) {
				this._folderSelect.options[0] = new Option(this.labelMyReports, "");
			}
            for (var i = 0; i < this._folders.length; i++) {
				var folder = this._folders[i];
				//show only shared folders (e.g. Defect) and non-shared system folders (e.g. My Reports)
				if (this._showFolder(folder)) {
					var index = this._folderSelect.options.length;
					this._folderSelect.options[index] = new Option(folder.name, folder.folderUUID);
					//if not a new report (via save as), select the report's folder
					if (this.isExistingReport && folder.folderUUID == this._query.folderUUID) {
						this._folderSelect.options[index].selected = true;
					}
				}
            }				
		},		
		
		_showFolder: function(folder) {
            if (this.isExistingReport) {
				return ((folder.shared && !folder.system) || folder.folderUUID == this._query.folderUUID);
			} else {
			    return true;	
			} 	
			return false;		
		},
		
		_hasPrivateFolders: function() {
            for (var i = 0; i < this._folders.length; i++) {
                if (this._folders[i].shared == false) {
					return true;
                }
            }	
			return false;		
		},
		
        _showAsDialog: function(){
            var dialogArgs = {
				primaryTitle: this.title,
                okCallback: dojo.hitch(this, "_onOk"),
                cancelCallback: dojo.hitch(this, "_onCancel")
            };
            
            this.floatingPane = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
            this.floatingPane.setWidth(this.width);
            this.floatingPane.contentContainer.appendChild(this.domNode);
            this.floatingPane.getOkButton().disabled = false;    
            this.floatingPane.open();      
        },		
		
        _onCancel: function(){
            //this.onCancel();
        },
		
        _onOk: function(){
			var invalid = this._validate(); 
			if (invalid != null) {
				this._messages.style.display = "inline";
				this._messages.innerHTML = invalid;
				return false;
			} else {
				this._messages.style.display = "none";
				this._save();
			}
        },
		
		_validate: function() {
			var name = this._name.value;
            if (name == null || dojo.trim(name).length == 0) {
                return Constants.getMessage("labelReportNameRequired");
            } else if (!this.isExistingReport && this._nameExists(name)) {
				return Constants.getMessage("labelReportNameExists", name);
			}	
			return null;	
		},
		
		_save: function() {
			//save the report			
		    var args = {
		        name: this._name.value,				
		        reportUUID: this._query.reportUUID
		    };		
			if (this.queryUUID && this.isExistingReport) {
				//update existing report
				args.queryUUID = this.queryUUID;
				//use existing parameters, if specified
				if (this._query.parameterNames && this._query.parameterValues) {
					args.parameterName = this._query.parameterNames;
					args.parameterValue = this._query.parameterValues;
				}				
			} else {
				//get new parameters from form
				var newParameters = prepareParameters(allParameterValues);
                args.parameterName = newParameters.names;
                args.parameterValue = newParameters.values;	
			}
			
			if (this._description.value != null && dojo.trim(this._description.value).length > 0) {
                args.description = this._description.value;
			}
			var folderUUID = this._folderSelect.value;
			if (folderUUID != null && dojo.trim(folderUUID).length > 0) {
				args.folderUUID = this._folderSelect.value;
			}
			
			var handler = new ServiceResponseHandler(this, "_saved", "_error");				
			ReportsClient.postSaveQueryDescriptor(handler, args);
			
		},
		
		_saved: function(newQueryUUID) {
            this._messages.style.display = "inline";
            this._messages.innerHTML = Constants.getMessage("labelReport") + " '" + this._name.value + "' " + Constants.getMessage("msgSavedSuccessfully");
			if (dojo.isFunction(this.onOk)) {
				this.onOk(newQueryUUID);
			}						
		},
		
        _findQuery: function(uuid, propertyName) {
            for (var i = 0; i < this._queries.length; i++) {
                if (this._queries[i][propertyName] == uuid) {
                    return this._queries[i];
                }
            }
            return null;
        },
		
        _findReport: function(uuid, propertyName, altPropertyName) {
            for (var i = 0; i < this._reports.length; i++) {
                if (this._reports[i][propertyName] == uuid) {
                    return this._reports[i];
                } else if (altPropertyName && this._reports[i][altPropertyName] == uuid) {
					return this._reports[i];
				}
            }
            return null;
        },		
		
        _findFolder: function(uuid) {
            for (var i = 0; i < this._folders.length; i++) {
                if (this._folders[i].folderUUID == uuid) {
                    return this._folders[i];
                }
            }
            return null;
        }, 		
		
		_nameExists: function(name) {
			return this._queryNameExists(name) || this._reportNameExists(name);
		},
		
        _queryNameExists: function(name) {
            for (var i = 0; i < this._queries.length; i++) {
                if (this._queries[i].name.toUpperCase() == name.toUpperCase()) {
                    return true;
                }
            }
            return false;
        },  	
		
        _reportNameExists: function(name) {
            for (var i = 0; i < this._reports.length; i++) {
                if (this._reports[i].name.toUpperCase() == name.toUpperCase()) {
                    return true;
                }
            }
            return false;
        }, 		
		
		_sortFolders: function(a, b) {
			//sort by name, but show personal (not shared) folders first
			var val1 = a.shared ? a.name : "0" + a.name;
			var val2 = b.shared ? b.name : "0" + b.name;
            if (val1 < val2) {
                return -1;
            } else if (val1 > val2) {
                return 1;
            }
            return 0;			
		},		
		
		_onKeyup: function(e){
			if (e.keyCode == dojo.keys.ENTER){
				this.floatingPane.getOkButton().onclick();
			}
		},
		
        _error: function(e) {
            console.error(this.declaredClass + ": Error: " + e.message);
            if (dojo.isFunction(this.onOk)) {
                this.onOk(e.message);
            }                       
        }        
    });
})();
