/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.common.EWIConfigSelector");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.parser");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.FilteredListChooser");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");
dojo.require("com.ibm.rqm.process.web.client.internal.timebox.TimeboxClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField");

(function() {
var ServiceResponseHandler          = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var Constants                       = com.ibm.rqm.planning.web.client.internal.Constants;
var TimeBoxClient                   = com.ibm.rqm.process.web.client.internal.timebox.TimeboxClient;
var PlanClient                      = com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
var SelectUtility                   = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
var EditorUtil          = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
var FormNumericField    = com.ibm.asq.common.web.ui.internal.widgets.layout.FormNumericField;

dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.common.EWIConfigSelector",
             [dijit._Widget, dijit._Templated], 
{
        
    templatePath: dojo.moduleUrl("com.ibm.rqm.execution.web", "ui/internal/view/templates/common/EWIConfigSelector.html"),
    widgetsInTemplate:true,

    labelAddTestCase: Constants.getMessage("labelAddTestCase"),
    labelSetConfigDefault: Constants.getMessage("labelAssociateResultsToPlan"),
    labelOk: Constants.getMessage("labelOk"),
    labelCancel: Constants.getMessage("labelCancel"),
	labelowner: Constants.getMessage("labelowner"),
	labelweight: Constants.getMessage("labelweight"),
    labelPoints: Constants.getMessage("labelPoints"),
	
    height: "450px", 
    width: "500px", 
    
    postCreate: function(){
        this._configTitle.innerHTML = Constants.getMessage("msgChangeExecProp");
        this._labelTestEnvironment.innerHTML = Constants.getMessage("labelConfiguration");
		EditorUtil.initContributors(this._owner, null, true);
		
		dojox.data.dom.removeChildren(this._weight);
        this.weightWidget = new FormNumericField({
            fieldLabel: this.labelweight,
            required: false
        });
        
        this.weightWidget.onValueChanged = dojo.hitch(this, "validate");
        this.weightWidget.setErrorMessage(Constants.getMessage("msgNumericWeightRequired"));
        this.weightWidget.setPromptMessage(Constants.getMessage("msgWeightRequired"));
        this._weight.appendChild(this.weightWidget.domNode);
    },
    
    initialize: function(args){
        this.testCases = args.testCases;
        this.configs = args.configs;
        if(!this.testCases || !this.configs){
            dojo.style(this._overrideSection, "display", "none");
            dojo.style(this._configNode, "display", "none");
            dojo.style(this._propertiesSection, "display", "none");
            this.messagesNode.innerHTML = Constants.getMessage("labelCreateTCorConfiguration"); 
            dojo.style(this.messagesNode, "display", "inline");
            this._showAsDialog();
            return;
        }
        this.allPlans = [];
        this.testPlansForTestCaseReceivedCount = 0;
        this._TPLabel.innerHTML = Constants.getMessage("labelTestPlan");
        this._IterationLabel.innerHTML = Constants.getMessage("labelIteration");
        if(this.testCases && this.testCases.length >0){
            for(var i=0;i<this.testCases.length;i++){
                var responseHandler = new ServiceResponseHandler(this, "_fillPlans", "_error");
                PlanClient.getRelatedTestPlanByCaseId(responseHandler, this.testCases[i].id);
            }
        }   
        for(var i=0;i<this.configs.length;i++){
            this.addListItem(this._configSection, this.configs[i].id,this.configs[i].name);
            if(args.defaultConfigId != null && this.configs[i].id == args.defaultConfigId)
                this._configSection.selectItem(i);
        }
        
        dojo.style(this._configSection.fsButtonTableNode, "display", "none");
        this._configSection.setMultiSelect(false);
        this._configSection.selectionList.size="6";
        
        this._showAsDialog();
    },
    
    _fillPlans: function(plans){
        if(plans && plans.length > 0){
            for(var i=0;i<plans.length;i++){
                var found = false;
                for(j=0;j<this.allPlans.length;j++){
                    if(this.allPlans[j].plan.id == plans[i].id){
                        found = true;
                        break;
                    }
                }
                if(found){
                    this.allPlans[j].count = this.allPlans[j].count + 1; 
                }else{
                    this.allPlans[j] = {};
                    this.allPlans[j].count = 1;
                    this.allPlans[j].plan = plans[i];
                }
            }
        }
        this.testPlansForTestCaseReceivedCount++;
        if(this.testPlansForTestCaseReceivedCount == this.testCases.length){
            this.plansToShow = [];
            for(var i=0;i<this.allPlans.length;i++){
                if(this.allPlans[i].count == this.testCases.length){
                    this.plansToShow.push(this.allPlans[i].plan);
                }
            }
            this._fillTPCombo(this.plansToShow);
            var responseHandler= new ServiceResponseHandler(this, "_appendIterations", "_error");
            TimeBoxClient.getTestPhasesDTO(responseHandler, {});
        }
    },
    
    _fillTPCombo:function(plans){
        dojox.data.dom.removeChildren(this._TP);
        if(!plans || plans.length == 0){
            var option = new Option("", "global");
            SelectUtility.add(this._TP, option);
        }else{
            var option = new Option(Constants.getMessage("labelUnassigned"), "global");
            SelectUtility.add(this._TP, option);
        }   
        if (plans && plans.length > 0) {
            for (var i = 0; i < plans.length; i++) {
                var planoption = new Option(plans[i].name, plans[i].id);
                SelectUtility.add(this._TP, planoption);
            }
        }
    },
    
    _appendIterations: function(iterations){
        if(iterations && iterations.length > 0){
            for(var j=0;j<iterations.length;j++){
                for(var i=0;i<this.plansToShow.length;i++){
                    if(!this.plansToShow[i].iterations)
                        this.plansToShow[i].iterations = [];
                    if(this.plansToShow[i].versionableItemId === iterations[j].targetTestPlanUUID){
                        this.plansToShow[i].iterations.push(iterations[j]);
                    }   
                }
            }
        }
    },
    
    _updateIterationCombo: function(){
        var planId = this._TP.value;
        dojox.data.dom.removeChildren(this._Iteration);
        if(planId ==="global"){
            this._Iteration.disabled = true;
            return;
        }
        this._Iteration.disabled = false;
        for(var i=0;i<this.plansToShow.length;i++){
            var id = parseInt(planId, 10);
            if(this.plansToShow[i].id === id){
                var planIterations = this.plansToShow[i].iterations;
                if(planIterations){
                    for(var j=0;j<planIterations.length;j++){
                        var opt = new Option(planIterations[j].name, planIterations[j].itemId);
                        SelectUtility.add( this._Iteration, opt );
                    }
                    break;  
                }else
                    this._Iteration.disabled = true;
            }
        }
    },
    
    _showAsDialog: function()
    {
        if (!this._dialog) {
            var dialogArgs = {
                okCallback: dojo.hitch(this, "_okAction"),
                cancelCallback: dojo.hitch(this, "_cancelAction")
            };
            
            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
            this._dialog.setTitle(Constants.getMessage("labelChangeEWIProperties"));
			this._dialog.setWidth(this.width);
            this._dialog.contentContainer.appendChild(this.domNode);
            this._dialog.getOkButton().disabled = false;                                
        }
        this._dialog.open(false);
    },  
    
    addListItem :function(domNode, id, displayValue){
        domNode.addOption(id, displayValue);
    },
    
    _okAction: function(){
        var args = {};
        var testConfig = this._configSection.getSelectedItems();

        args.testConfig = testConfig.length > 0? testConfig[0].id : null;
        args.testPlanItemId = this._TP.value != "global" ? this.getPlanInfo(this._TP.value, "vItemId") : null;
		if (args.testPlanItemId){
			args.testPhase = this._Iteration.value ? this._Iteration.value : "unassigned";
		}
        
		if (dojo.trim(this.weightWidget.getValue()).length != 0) {
			args.weight = Number(dojo.trim(this.weightWidget.getValue()));
		}
		
		var option = this._owner.options[this._owner.selectedIndex];
        args.owner = option.text != Constants.getMessage("labelUnassigned")? option.value : null;

        if (this.okCallback) {
            this.okCallback(args);
        }
    },
    
    getPlanInfo: function(planId, attributeName){
        for(var i=0;i<this.plansToShow.length; i++){
            if(this.plansToShow[i].id == planId){
                if(attributeName == "name"){//non-nls
                    return this.plansToShow[i].name;
                }else if(attributeName == "vItemId"){ //non-nls
                    return this.plansToShow[i].versionableItemId;
                }//Add more if need be
            }
        }
    },
    
    getPhaseInfo: function(planId, iterationItemId, attributeName){
        for(var i=0;i<this.plansToShow.length; i++){
            if(this.plansToShow[i].id == planId){
                var iterations = this.plansToShow[i].iterations
                for(var j=0;j<iterations.length;j++){
                    if(iterations[j].itemId == iterationItemId){
                        if(attributeName == "name"){//non-nls
                            return iterations[j].name;
                        }//Add more if need be
                    }   
                }
            }
        }       
    },
    
    _cancelAction: function(){
        if (this.cancelCallback) {
            this.cancelCallback();
        }
    },
    
    _checkBoxStateChanged: function(){
        if(this.allowChangingOfPlanIteration){
            //Enable Section, else keep disabled
        }
    },
    
    onSelectTP: function(){
        this._updateIterationCombo();
    },
        
	validate: function() {
        if (dojo.trim(this.weightWidget.getValue()).length != 0 && this._dialog) {
            var weight = Number(dojo.trim(this.weightWidget.getValue()));
            if (weight != NaN && weight >= 0 ) {
                this._dialog._yesButton.disabled = false;
            }
            else { 
                this._dialog._yesButton.disabled = true;
            }
        }
    },
	
    _error: function(errorObj){
        console.error("Error in EWIConfigSelector " + errorObj.message);
    }
    
});
})();
    
