/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

EditorSection = dojo.provide("com.ibm.asq.common.web.ui.internal.view.common.EditorSubSection");
EditorSection.ACTION_ICON_ONLY = 0;
EditorSection.ACTION_LABEL_ONLY = 1;
EditorSection.ACTION_LABEL_AND_ICON = 2;

dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSubSection");

(function() { // module container

var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;

dojo.declare("com.ibm.asq.common.web.ui.internal.view.common.EditorSubSection",
com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSubSection,
{    
    constructor: function() {
        this.editable = false;  
        this.title = null;
        this.summary = null;
				
		//AbstractArtifactEditorSection
		this.sectionProvider = null;
    },    

    postCreate: function() {
		this.inherited('postCreate', arguments);		
		if (!this.sectionProvider) {
			throw new Error("EditorSection: parameter 'sectionProvider' is REQUIRED.");
		}		
		this.renderContent(true);		
    },
	
	renderContent: function(updateContent) {  
        //set up the content based on the provider (AbstractArtifactEditorSection)
        this.setEditable(this.editable);
        
        //title information
        this.setTitle(this.sectionProvider.getConfig().name);
        this.setDisplayTitle(this.sectionProvider.getDisplayTitle());

        //main content
        if (this.open && updateContent) {
            this.setContent(this.sectionProvider.getMainContent(this.editable));
        }
        
        //context-sensitive help
        var helpID = this.sectionProvider.getHelpContextID();
        if (helpID) {
            this.setCSHelp(helpID);
        }          
	},
	
	toggleEditable: function() {
		this.editable = !this.editable;
		this.renderContent(true);
	},
	
    setSummary: function(summary) {
        this.summary = summary;
        if (this._summaryTextNode) {
            this._summaryTextNode.innerHTML = summary;
        }
        if (this.summaryEditable && this._summaryEditableNode) {
            this._summaryEditableNode.value = this.summary;
        }
    },	
	
    _onTitleClick: function() {
		//lazily load main content
		if (!this.initialized) {
			this.setContent(this.sectionProvider.getMainContent(this.editable));
			this.initialized = true;
		}
		//toggle open/closed state
		this.toggle();        		
		
		//nofity sectionProvider's of expand/collapse (if provided) 
		var handler = this.open ? this.sectionProvider.getExpandHandler() : this.sectionProvider.getCollapseHandler();
		if (handler) {
			handler();
		}   
        dojo.publish("com-ibm-asq.common-CollapsibleSection-sectionSizeChanged", null);
    },	

    setEditable: function(editable) {
        this.editable = editable;
        
        // set appropriate summary node to visible
        if (this.summaryEditable) {
            this._summaryTextNode.style.display = editable ? "none" : "block";
            this._summaryEditableNodeContainer.style.display = editable ? "block" : "none";
        }				
    },
	
    //////////////////////////////////////////////////////////////////////////////////////
    //     Addt'l API for Editor section support
    //////////////////////////////////////////////////////////////////////////////////////        
    isEditable: function() {
        return this.editable;
    },
	
	getProvider: function() {
		return this.sectionProvider;
	},
    
    setCSHelp: function(helpID) {
        var help = new CSHelp({contextID: helpID});
        dojox.data.dom.removeChildren(this.titleHelpNode);
        this.titleHelpNode.appendChild(help.domNode);        
    }
});

})();
