/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

/*
 * Example for usage
 data = [ 
		[ "normal", false, "new", 'But are not followed by two hexadecimal', 29.91, 10, false ],
		[ "important", false, "new", 'Because a % sign always indicates', 9.33, -5, false ],
		[ "important", false, "read", 'Signs can be selectively', 19.34, 0, true ],
		[ "note", false, "read", 'However the reserved characters', 15.63, 0, true ],
		[ "normal", false, "replied", 'It is therefore necessary', 24.22, 5.50, true ],
		[ "important", false, "replied", 'To problems of corruption by', 9.12, -3, true ],
		[ "note", false, "replied", 'Which would simply be awkward in', 12.15, -4, false ]
	];
	
	var rows = 2;
	for(var i=0, l=data.length; i<rows-l; i++){
		data.push(data[i%l].slice(0));
	}
	
	//Sample comparator to be used for sorting
	compare2ndLetter = function(inA, inB) {
			// Sample Comparator for second alphabet
			// return <0, 0, >0
			return inA.charCodeAt(1) - inB.charCodeAt(1);
		}
	
	model = new dojox.grid.data.Table([{}, {}, {}, {compare: compare2ndLetter}], data);
	
	//model = new dojox.grid.data.Table(null, data);//Use if sorting is not required
	
	//Formatting data
	formatMoney = function(inDatum) {
		return isNaN(inDatum) ? '...' : '$' + parseFloat(inDatum).toFixed(2);
	}

	// Layout
	statusCell = { field: 2, name: 'Status', width: '83px', styles: 'text-align: center;', editor: dojox.grid.editors.Select, options: [ "new", "read", "replied" ] };
	gridLayout = [{
		type: 'dojox.GridRowView', width: '20px'
	},{
		defaultCell: { editor: dojox.grid.editors.Input, styles: 'text-align: right;'  },
		rows: [[
			{ name: 'Id',width: '90px', get: function(inRowIndex) { return inRowIndex+1;} },
			{ name: 'Priority',width: '106px', styles: 'text-align: center;', editor: dojox.grid.editors.Select, options: ["normal", "note", "important"]},
			{ name: 'Mark', width: '112px', styles: 'text-align: center;', editor: dojox.grid.editors.Bool },
			statusCell,
			{ name: 'Message', width: '249px',styles: ''},
			{ name: 'Amount', width: '127px', formatter: formatMoney },
			{ name: 'Amount', field: 4, width: '200px', formatter: formatMoney }
		]]
	}];
	
	//
	//	Note the data[] and the rows[] differ in size. 
	//  ie. If there is data to be hidden from the UI but is needed during service call, append the values in data. 
	//	
		
		var grid = new com.ibm.rqm.planning.web.ui.internal.view.common.GridContainer({
			model: model,
			structure: gridLayout,
			rowCount:"10", elasticView:"2",
			defaultStaticRow: [ "normal", false, "new", 'Now is the time for all good men to come to the aid of their party.', 99.99, 9.99, false ] //Used for adding new row.
		});
		this._testGrid.appendChild(grid.domNode);
		grid.render();
		grid.initialize();
 
 <div>
 	<div dojoAttachPoint="_testGrid" style="height: 500px; width: 500px;"></div>
 </div>	
 */


dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.GridContainer");

dojo.require("dojox.grid.Grid");
dojo.require("dojox.grid.VirtualGrid");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");

(function() { // module container

var ImageCache		= com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache;
var Constants       = com.ibm.rqm.planning.web.client.internal.Constants;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.GridContainer",
dojox.Grid,
{
	templateString: '<div class="com-ibm-rqm-planning-common-gridcontainer"><div style="text-align: right" dojoAttachPoint="headerRow"></div><div class="dojoxGrid" hidefocus="hidefocus" role="wairole:grid"><div class="dojoxGrid-master-header" dojoAttachPoint="headerNode"></div><div class="dojoxGrid-master-view" dojoAttachPoint="viewsNode"></div><span dojoAttachPoint="lastFocusNode" tabindex="0"></span></div></div>',
	classTag: 'dojoxGrid',
	postCreate: function(){
		this.inherited(arguments);
		
	},
	
	/*
	 * Should be called after grid.render is called. Call only once for a given grid.
	 * 
	 * Buttons are for initial implementation only. They will be replaced by Icons
	 */
	
	initialize: function()
	{
		this._addActions();
	},
	
	/*
	 * returns the control selected rows
	 */
	
	getSelectedRows: function()
	{
		var rowIndicies = this.selection.getSelected();
		var selectedRows = [];
		var sampleString = "Get Selected APi called (# is delimiter)\n";
		for(var i=0;i<rowIndicies.length;i++)
		{
			var row = this.model.getRow(rowIndicies[i]);
			sampleString +=row+" # ";
			selectedRows.push(row);
		}
		alert(sampleString);
		return selectedRows;
	},
	
	/*
	 * removes the CTRL+selected rows
	 */
	
	deleteSelectedRows: function()
	{
		this.removeSelectedRows();
		this.deleteButton.setDisabled(true);
	},
	
	/*
	 * Appends a new row below the current row with the defaultStaticRow passed during creation.
	 * Add Row button will not appear if this.defaultStaticRow is not passed during creation.
	 */
	
	appendRow: function()
	{
		var rowData = this.defaultStaticRow.slice(0);
		var i = this.selection.getFirstSelected() + 1 || this.model.getRowCount();
		this.addRow(rowData, i);
	},
	
	/*
	 * Returns all data in the Grid
	 */
	
	getAllRows: function()
	{
		this.edit.editorApply();
		var rowCount = this.model.getRowCount();
		var rowData = []
		for(var i=0; i<rowCount; i++)
			rowData.push(this.model.getRow(i));
		
		return rowData;	
	},
	
	/*
	 * Overriden from Grid, fired when focus is released from editing control
	 */
//	doApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
//		this.model.setDatum(inValue, inRowIndex, inFieldIndex);
//		this.onApplyCellEdit(inValue, inRowIndex, inFieldIndex);
//		this.onchange();
//	},
	
	dokeyup: function(e){
		if(this.cellContentUpdated(e)){
				this.onchange();
//				this.model.setDatum(value, this.focus.rowIndex, this.focus.cell.fieldIndex);
		}
	},
	
	cellContentUpdated: function(e){
		if(this.edit.isEditing()){
			var value = this.focus.cell.editor.getValue(this.focus.rowIndex);
			var oldValue = this.model.getDatum(this.focus.rowIndex, this.focus.cell.fieldIndex);
			if(value != oldValue){
				return true;
			}else{
				return false;
			}
		}
	},
	
	/*
	 * Overriden from dojox.grid._grid.publicEvents
	 */
	onCellClick: function(e){
		this.inherited("onCellClick", arguments);
		if(this.cellContentUpdated(e)){
				this.onchange();
		}
		this.deleteButton.setDisabled(false);
	},
	
	/*
	 * Overriden from dojox.grid._grid.publicEvents
	 */
	onRowClick: function(e) {
		this.inherited("onRowClick", arguments);
		this.deleteButton.setDisabled(false);
	},
	
	_placeHolderIconsClicked: function()
	{
//		alert("Not Yet Implemented");
	},
	
	
	_error: function()
	{
		alert("Error in Grid Container");
	},
	
	_addActions: function() {
		var self = this;
		var arg = {
            label: Constants.getMessage("labelAddRow"),
            onClick: function(e) {
                self.appendRow();
				self.onchange();    
            }
        };
        arg["iconClass"] = "add-table-row-icon-image";
        arg["showLabel"] = false;
        var button = new com.ibm.asq.common.web.ui.internal.widgets.Button(arg);
        button.setDisabled(false);
        this.headerRow.appendChild(button.domNode);
		
		var arg = {
            label: Constants.getMessage("labelDeleteRow"),
            onClick: function(e) {
                self.deleteSelectedRows();
				self.onchange();    
            }
        };
        arg["iconClass"] = "delete-table-row-icon-image";
        arg["showLabel"] = false;
        this.deleteButton = new com.ibm.asq.common.web.ui.internal.widgets.Button(arg);
        this.deleteButton.setDisabled(true);
        this.headerRow.appendChild(this.deleteButton.domNode);
	},
	
	resize: function(){
		if( dojo.isIE && !this.domNode.parentElement )
		{
			return;
		}
		else if(!dojo.isIE && !this.domNode.parentNode){
			return;
		}

		this.inherited( "resize", arguments );		
	}

});

})();
