/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemContent");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestGroupClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient");

(function() {

var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var TestGroupClient		    = com.ibm.rqm.planning.web.client.internal.construction.TestGroupClient;
var TestCaseClient		    = com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
var TestScriptClient		= com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient;
var Constants    			= com.ibm.rqm.planning.web.client.internal.Constants;

// delay between user input and running a search (in milliseconds)
var DELAY					= 750;

// cap max number of query results at 50
var MAX						= 50;

var RUNNING_SEARCH_MSG		= Constants.getMessage("labelRunningSearch");
var FETCH_ERR_MSG			= Constants.getMessage("msgErrorRunningQuery");
var KEY_ESCAPE			= 27;
var KEY_RETURN			= 13;
var KEY_ENTER			= 14;
var KEY_SPACE			= 32;
// var KEY_DELETE			= 46; // future

dojo.declare(
	"com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemContent",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/SelectItemContent.html"),
		
		labelAdd: Constants.getMessage("labelAdd"),
		labelClose: Constants.getMessage("labelClose"),
		labelMatchingItems: Constants.getMessage("labelMatchingItems"),
		
		postCreate: function() {
			this.exclusionMap= {};
			this._connectEvents();
			// mark that keypresses from this control should be ignored w/r/t dirtying the editor
			//this.queryInput.__jazzIgnoreKeystroke = true;
			this.setIsOKCancel(false);
			this.doSearch();
		},
		
		setInitialFocus: function() {
			setTimeout(dojo.hitch(this,this.setFocus),200);
		},
		setFocus: function() {
			//this.queryInput.focus();
			this.queryResults.focus();
		},
		
		reset: function() {
			//this.queryInput.value= "";
			this.queryResults.length= 0;
			this._clearStatusMessage();
			this.addCmd.disabled= true;
			this.addAndCloseCmd.disabled= true;
		},
		
		setKey: function(key) {
			this.key= key;
		},
		
		setAllowsMultiple: function(allowsMultiple) {
			this.allowsMultiple= allowsMultiple;
		},
		/*
		 *  The dialog can either be OK Cancel, or Add, Add and Close, Close 
		 */
		setIsOKCancel: function(isOKCancel) {
			this.isOKCancel = isOKCancel;
			if(isOKCancel) {
				this.addCmd.style.display = "none";
				this.addAndCloseCmd.value = Constants.getMessage("labelOk");
				this.addAndCloseCmd.innerHTML = Constants.getMessage("labelOk");
				this.cancelCmd.value = Constants.getMessage("labelCancel");
				this.cancelCmd.innerHTML = Constants.getMessage("labelCancel");
			} else {
				this.addCmd.style.display = "inline";
				this.addAndCloseCmd.value = "Add & Close";
				this.addAndCloseCmd.innerHTML = Constants.getMessage("labelAddAndClose");
				this.cancelCmd.value = Constants.getMessage("labelClose");
				this.cancelCmd.innerHTML = Constants.getMessage("labelClose");
			}
		},
		
		setExclusionList: function(exclusionList) {
			this.exclusionMap= {};
			for (var i=0; i<exclusionList.length; i++) {
				var id= exclusionList[i];
				this.exclusionMap[id]= true;
			}
		},
		
		destroyRendering: function() {
			var nodesToClean=
				[this.addCmd, this.addAndCloseCmd, this.queryResults, this.cancelCmd, /*this.queryInput,*/ this.queryResults];
			EditorUtil._cleanFormElements(nodesToClean);
		},
		
		_connectEvents: function() {
			dojo.connect(this.addCmd, "onclick", this, this._handleAdd);
			dojo.connect(this.addAndCloseCmd, "onclick", this, this._handleAddAndClose);
			//dojo.connect(this.showAllCmd, "onclick", this, this._handleShowAll);
			dojo.connect(this.queryResults, "ondblclick", this, this._handleAdd);
			dojo.connect(this.cancelCmd, "onclick", this, this._handleCancel);
			dojo.connect(this.addCmd, "onkeyup", this, this._handleEscapeKeyUp);
			dojo.connect(this.addAndCloseCmd, "onkeyup", this, this._handleEscapeKeyUp);
			dojo.connect(this.cancelCmd, "onkeyup", this, this._handleEscapeKeyUp);
			//dojo.connect(this.queryInput, "onkeyup", this, this._handleInputKeyUp);
			dojo.connect(this.queryResults, "onkeyup", this, this._handleListKeyUp);
			dojo.connect(this.queryResults, "onchange", this, this._handleSelectionChange);
		},
		
		_handleAdd: function() {
			var retArr= [];
			var options= this.queryResults.options;
			for (var i=0; i<options.length; i++) {
				if (options[i].selected) {
					retArr.push(this._backingItems[options[i].value]);
					this.exclusionMap[this._backingItems[options[i].value]] = true;
					if(dojo.isIE) {
						options[i].style.color = "#CCCCCC";
					}
					options[i].disabled = true;
					options[i].selected = false;
				}
			}
			this.onOk(retArr, this.key);
			if(this.isOKCancel) {
				this._handleCancel();
			}
		},
		
		_handleAddAndClose: function() {
			this._handleAdd();
			this._handleCancel();
		},

		_handleCancel: function() {
			this.onCancel();
			this.reset();
		},
		
		_handleSelectionChange: function() {
			// disable the 'OK' button if nothing is selected
			this.addCmd.disabled= this.queryResults.selectedIndex === -1;
			this.addAndCloseCmd.disabled= this.addCmd.disabled;
			if (!this.allowsMultiple) {
				this._deselectPreviouslySelected();
			}
		},
		
		_deselectPreviouslySelected: function() {
			
			var options= this.queryResults.options;
			var prevOption= this._currentOption ? this._currentOption : null;
			var newOptionSet= false;

			for (var i=0; i<options.length; i++) {
				if (!options[i].selected) {
					continue;
				} else if (options[i] == prevOption || newOptionSet) {
					options[i].selected= false;
				} else {
					this._currentOption= options[i];
					newOptionSet= true;
				}
			}
		},
		
		_handleEscapeKeyUp:function(e) {
			if(e.keyCode == KEY_ESCAPE) {
				this._handleCancel();
				return;
			}
		},
		_handleListKeyUp:function(e) {
			if(e.keyCode == KEY_ESCAPE) {
				this._handleCancel();
				return;
			}
			if(e.keyCode == KEY_RETURN || e.keyCode == KEY_ENTER) {
				this._handleAdd();
				return;
			}
			if(e.keyCode == KEY_SPACE) {
				if(dojo.isIE) {
					this.queryResults.click();
				};
				return;
			}
		},
		_handleInputKeyUp:function(e) {
			if(e.keyCode == KEY_ESCAPE) {
				this._handleCancel();
				return;
			}
			// If they hit enter with one item on the list, take it
			if(e.keyCode == KEY_RETURN || e.keyCode == KEY_ENTER) {
				if(this.queryResults.options.length >= 1) {
					this.queryResults.options[0].selected = true;
					this._handleAdd();
					this._handleCancel();
					return;
				}
			}
			this._doCancellableSearch();
		},
		_doCancellableSearch: function() {
			if (this._currentId) {
				clearTimeout(this._currentId);
			}
			var self= this;
			var doSearchFunc= function() {
				self._doSearch();
			};
			this._currentId= setTimeout(doSearchFunc, DELAY);
		},
		
		/*
		_doSearch: function() {
			// if there's no input, do nothing
			var value= this.queryInput.value;
			if (dojo.trim(value) === "") {
				this.queryResults.length= 0;
				this._clearStatusMessage();
				return;
			}
			var srh = new ServiceResponseHandler(this, "_loadQueryResults", "_queryError");			
			var params;

			value = value.replace(/\* /g,"%"); //remove space between * and / when uncommenting
			value = value.replace(/\?/g,"_");
			params= { nameFilter: value };

			this._setStatusMessage(RUNNING_SEARCH_MSG);
			WorkItemClient.getContributors(srh, params);
		},
		*/
		
		doSearch: function() {
			var srh = new ServiceResponseHandler(this, "_loadQueryResults", "_queryError");
			this._setStatusMessage(RUNNING_SEARCH_MSG);
			
			if ( this.itemType == Constants.TYPE_GROUP ) {
				TestGroupClient.getTestGroups(srh);
			} else if ( this.itemType == Constants.TYPE_CASE ) {
				TestCaseClient.getTestCases(srh);
			} else if ( this.itemType == Constants.TYPE_SCRIPT ) {
				TestScriptClient.getTestScriptsSummary(srh);
			}
			/*
			else if( this.itemType == Constants.TYPE_REQUIREMENT) {
				var requirementClient = new RequirementClient();
				requirementClient.getAllRequirements(this, "_loadQueryResults");
			}
			*/
		},
		
		_handleShowAll: function(e) {
			e.preventDefault();
			//this.queryInput.value = "*";
			this._doSearch();
		},
		_loadQueryResults: function(itemList) {
			itemList = itemList ? itemList : [];
			itemList.sort(function(a,b) { return a.name.toLowerCase() < b.name.toLowerCase() ? -1 : 1; } );
			// contrib.name, contrib.itemId
			this._backingItems= {};
			this.queryResults.length = 0;
			var displayedResults = itemList.length;
			for (var i=0; i<itemList.length; i++) {
				var title = itemList[i].name;
				var option = new Option(title, itemList[i].itemId);
				if (this.exclusionMap[itemList[i].itemId]) {
					if(dojo.isIE) {
						option.style.color = "#CCCCCC";
						}
					option.disabled = true;
				}
				this._addOption(this.queryResults, option, /*isDefault*/false);
				this._backingItems[itemList[i].itemId]= itemList[i];
			}
			this._setStatusMessage(displayedResults + " results");
		},
		
		// adds an option to the end of a list; works both in IE and Firefox
		_addOption: function(select, option, isDefault) {
			var optionArr= select.options;
			optionArr[optionArr.length]= option;
			option.selected= (isDefault === true) ? true : false;
		},
	
		_queryError: function(errorObj) {
			this._setStatusMessage(FETCH_ERR_MSG, /*isError*/ true);
		},
		
		_clearStatusMessage: function() {
			this.msgArea.innerHTML= "";
		},
		
		_setStatusMessage: function(msg, isError) {
			this.msgArea.innerHTML= msg;
			this.msgArea.className= isError ? "ErrorStatus" : "NormalStatus";
		}
	}
);

})();
