/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.TestScriptsSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptTableView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableDialog");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.NewItemPopup");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");

(function(){ // module container
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    var TestScriptTableView = com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptTableView;
    var SelectItemTableDialog   = com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableDialog;
    var NewItemPopup = com.ibm.rqm.planning.web.ui.internal.view.construction.NewItemPopup;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var TestScriptClient = com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient;
	
    // global save context
    var SaveParticipant         = com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant;
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.TestScriptsSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: "<div dojoAttachPoint=\"_testScriptsSection\" class=\"com-ibm-rqm-planning-web-ui-internal-view-construction-extension-TestScriptsSection\"></div>",
        
        constructor: function(args){
            this._items = [];
            this._itemType = Constants.TYPE_SCRIPT;
            this._itemsAdded = [];
            this._itemsRemoved = [];
            this._referencesAddedType = "testCaseExecutionScriptsAdded";
            this._referencesRemovedType = "testCaseExecutionScriptsRemoved";
			this._initialized = false;
        },
        
        postCreate: function(){            
            this.inherited('postCreate', arguments);
            this.addWorkItemSupport();           
        },
        
        initialize: function(){
            this._itemsAdded = [];
            this._itemsRemoved = [];
            this.showTestScriptTable();
            this.refreshTestScripts();
			this.setSectionDirty(false);
        },
		
        refreshTestScripts: function() {
            if (this.getParentEditor().isNew() && !this.getParentEditor().isCopy()) {
                return;          
            }
            var srh = new ServiceResponseHandler(this, "_setTestScripts", "_error");
            TestScriptClient.getTestCaseTestScripts(srh, {
                versionedTestCaseItemId: this.getArtifact().versionableItemId
            });
        },
        
        _setTestScripts: function(testScripts) {
            
            this._items = testScripts ? testScripts : [];
            this.showTestScriptTable();
            this._initialized = true;
            //if new item was clicked before save, open it
            this._openNewlySavedItem();			           
        },		
        
        showTestScriptTable: function() {
            var artifact = this.getArtifact();
            
            if(!this._scriptTableViewer) {
                dojox.data.dom.removeChildren(this._testScriptsSection)
                this._scriptTableViewer = new TestScriptTableView({
                	editable: this.isEditable(),
					printable: this.isPrintView(),
                    enableRowSelection: true,
                    scripts: this.getItems(),
                    actions: this.getTableActions(),
					refreshCallback: dojo.hitch(this, "refreshTestScripts"),
					newItemLinkCallback: dojo.hitch(this, "_handleNewItemLink")
                });
                this._testScriptsSection.appendChild(this._scriptTableViewer.domNode);
            }
            else {
                this._scriptTableViewer.refresh(this.getItems());
            }
        },
        
        getItems: function() {
            return this._items;            
        },
        
        setItems: function(items) {
            this._items = items;            
        },
               
        //actions to be displayed in the table header        
        getTableActions: function(){
            var actions = [];
			
            if (this.isRolePermitsEdit()) {
                actions.push({
                    label: Constants.getMessage("labelAddExistingTestScripts"),
                    actionHandler: dojo.hitch(this, "_handleAddClick"),
                    iconClass: "add-icon-image",
                    disabledOnReadOnly: true
                });
            }
            var ts_type = com.ibm.asq.common.web.client.internal.PermissionsClient.OP_TEST_SCRIPT;
            var perm = com.ibm.asq.common.web.client.internal.PermissionsClient.getStoredPermission(ts_type);
            if (!perm || perm.create) {
                actions.push({
                    label: Constants.getMessage("labelAddNewTestScript"),
                    actionHandler: dojo.hitch(this, "_handleAddNewClick"),
                    iconClass: "add-new-testscript-image",
                    disabledOnReadOnly: true
                });
            }
            if (this.isRolePermitsEdit()) {
                actions.push({
                    label: Constants.getMessage("labelRemoveTestScripts"),
                    actionHandler: dojo.hitch(this, "_handleRemoveClick"),
                    iconClass: "remove-enabled-icon-image",
                    disabledOnReadOnly: true
                });
            }                         
            return actions;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        getAttributes: function(){
            return null;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(){
            this.inherited('saveFinished', arguments);
            this.initialize();     
            this.addWorkItemSupport();	
			this.updateRefreshable();
        },
		
        //@Overridden from AbstractArtifactEditorSection
        setSectionDirty: function(dirty) {
            this.inherited('setSectionDirty', arguments);
            this.updateRefreshable();
        },
        
        updateRefreshable: function() {
            if (this._scriptTableViewer && this._scriptTableViewer.refreshAction) {
                this._scriptTableViewer.refreshAction.setDisabled(this.isSectionDirty());
            }           
        },		
        
        _handleAddClick: function() {
			this.getParentEditor().setUri(window.location.hash);
            this._showItemsDialog();         
        },
        
        _handleAddNewClick: function() {
            var args = {
                itemType: Constants.MODEL_TYPE_SCRIPT,
                onOkCallback: dojo.hitch(this, "_handleAddNewItem")                
            }
            this._newItemDialog = new NewItemPopup(args);        
        },        
		
        _handleNewItemLink: function(participantId) {
			var self = this;
			var args = {
				message: Constants.getMessage("msgSaveRequiredToView", Constants.TYPE_CASE, Constants.TYPE_SCRIPT),
				yesCallback: function(){
					self._newItemToOpen = self._getItemIndexByParticipantId(participantId);
            		self.getParentEditor().onSave();    
					return true;          
				},
				noCallback: function() {
					return;
				}
			}
			var dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
       	 	dialog.setSecondaryTitle(Constants.getMessage("labelConfirmation"));
			dialog.open();         
        },		
		
		_openNewlySavedItem: function() {
            //if a new item was clicked, open it
            if (this._newItemToOpen) {
                var savedItem = this._items[this._newItemToOpen];
                this._newItemToOpen = null;
                if (savedItem != null) {
                    window.location = Utility.getTestScriptURI(savedItem.id);
                }
            }			
		},

        _handleRemoveClick: function() {
            this._removeItems(this._scriptTableViewer.getSelectedItems());      
			if (this._scriptTableViewer.deleteActionNode){
				this._scriptTableViewer.deleteActionNode.setDisabled(true);
			}   
        },               
               
        _showItemsDialog: function() {
            var params = {
                title: Constants.getMessage("labelAddItems", this._itemType),
                exclusionList: this._getAddExclusionList(),                    
                showQueryFilters: true,
                itemType: this._itemType,               
                onOk: dojo.hitch(this, "_handleAddedItems")
            };
            var itemListWidget = new SelectItemTableDialog(params);
            itemListWidget.setExclusionList(this._getAddExclusionList());
            itemListWidget.show();    
        },
        
        _getAddExclusionList: function() {
            var excludedItems = this.getItems(); //[].concat(this.getItems(), this._excludeFromAdd);
            var excludedItemIds = [];
            
            for ( var i = 0; i < excludedItems.length; i++) {
                var excludedItem = excludedItems[i];
                var itemId = excludedItem.versionableItemId != null ? excludedItem.versionableItemId : excludedItem.itemId;
                excludedItemIds.push(itemId);
            }
            return excludedItemIds; 
        },
        
        _handleAddNewItem: function() {
            var newItemParticipant = this._newItemDialog.getItemSaveParticipant();            
            this.getSaveContext().addParticipant(newItemParticipant);
            this.getSaveContext().createReference(this.getParentEditor(), this._referencesAddedType, newItemParticipant);    
            
            var newItem = this._newItemDialog.getItemForDisplay();
            newItem.participantId = newItemParticipant.getParticipantId();
            this._items.push(newItem);  
			this.showTestScriptTable();
			this.setDirty();              
        },
        
        _handleAddedItems: function(addedItems) {
            for (var i = 0; i < addedItems.length; i++) {
                var item = addedItems[i];
                var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
                if (this._containsItemId(this._itemsRemoved, itemId)) {
                    this._itemsRemoved = this._removeItemId(this._itemsRemoved, itemId);
                    this.getSaveContext().unremoveReference(this.getParentEditor(), this._referencesRemovedType, itemId);
					this._items.push(item);
                } else if (!this._contains(item)) {
                    this._items.push(item);
                    this._itemsAdded.push(itemId);                              
                } else {
                    alert(Constants.getMessage("msgIsPartOfList", item.name));
                }
                //append the item to the list
                this._addItem(item);
            }      
            this.showTestScriptTable();
			this.setDirty();              
        },
        
        _addItem: function(item) {
            //create a SaveParticipant for the item
            var versionableItemId = item.versionableItemId ? item.versionableItemId : null;
            var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;
            var saveParticipant = this.getSaveContext().findParticipant(itemId);
            if (saveParticipant == null)
            {
                saveParticipant = new SaveParticipant();
                var itemType = Utility.getModelType(item);
                saveParticipant.setDirty(false);
                saveParticipant.getItemId = function () { return itemId; };
                saveParticipant.getVersionableItemId = function() { versionableItemId };
                saveParticipant.getItemType = function() { return itemType; };
                this.getSaveContext().addParticipant(saveParticipant);
            }
            var parent = this.getParentEditor();
            // treat it as a new reference if the parent is a copy
            if (parent != null && parent.isCopy != null && parent.isCopy.call(parent)) {
                this.getSaveContext().createReference(parent, this._referencesAddedType, saveParticipant);
            }
            else if (this._containsItemId(this._itemsAdded, itemId) && parent!=null)
            {
                this.getSaveContext().createReference(parent, this._referencesAddedType, saveParticipant);
            }
        },
        
        _removeItems: function(items) {            
            if ( !items || items.length == 0) {
//                alert("Please select one or more " + this._itemType + "s to remove.");
				var msg = Constants.getMessage("msgDeleteTypeRequired", this._itemType);
				this.getParentEditor().setHeaderStatus(Constants.OPERATION_WARNING, Constants.HEADER_STATUS_WARNING, msg);
                return;
            }
            var confirmMsg = "";
            if (items.length > 1) {
                confirmMsg = Constants.getMessage("msgRemoveItem2", items.length, this._itemType);
            } else {
                confirmMsg = Constants.getMessage("msgRemoveItem", this._itemType, items[0].name);
            }
			var self = this;
			var	args = {
				message: confirmMsg,
				yesCallback: function(){
					for (var i = 0; i < (items ? items.length : 0); i++) {
                		self._removeItem(items[i]);
            		}
            		self._scriptTableViewer.refresh(self.getItems());
					self.setDirty();           
					return true;          
				},
				noCallback: function() {
					return;
				}
			}
			var dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
        	dialog.setSecondaryTitle(Constants.getMessage("labelConfirmation"));
			dialog.open();   
        },        
        
        _removeItem: function(item) {       
            var updatedItems = [];        
            var items = this.getItems();  
            for (var i = 0; i < items.length; i++) {
                //if this item is not new && its itemId doesn't match the item to be removed's itemId
                if (items[i].itemId && items[i].itemId != item.itemId) {
                    updatedItems.push(items[i]);
                //if this item is new and its participantId doesn't match the item to be removed's participantId
                } else if (items[i].newItem && items[i].participantId != item.participantId) {
                    updatedItems.push(items[i]);
                } else {                 
                    //handle item removal from save context   
                    var saveContext = this.getSaveContext();
                    var itemId = items[i].versionableItemId ? items[i].versionableItemId : items[i].itemId;                    
                    if (items[i].newItem) {
                        //handle new item CREATED during this saveContext session
                        saveContext.uncreateReference(this.getParentEditor(), this._referencesAddedType, items[i].participantId);
                        saveContext.removeParticipant(saveContext.findParticipant(items[i].participantId));
                    } else if (this._containsItemId(this._itemsAdded, itemId)) {
                        //handle existing items ADDED during this saveContext session
                        this._itemsAdded = this._removeItemId(this._itemsAdded, itemId);                        
                        saveContext.uncreateReference(this.getParentEditor(), this._referencesAddedType, itemId);
                    } else {
                        //handle existing references, add them 'removed'
                        this._itemsRemoved.push(itemId);
                        saveContext.removeReference(this.getParentEditor(), this._referencesRemovedType, itemId);
                    }
                }
            }
            this.setItems(updatedItems);
        },
        
        _containsItemId: function(array, itemId) {
            for (var i = 0; i < array.length; i++) {
                if (array[i] == itemId) {
                    return true;
                }
            }
            return false;
        },
        
        _removeItemId: function(array, itemId) {
            var updatedArray = [];
            for (var i = 0; i < array.length; i++) {
                if (array[i] != itemId) {
                    updatedArray.push(array[i]);
                }
            }
            return updatedArray;
        },
        
        _contains: function(item) {
            var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;         
            return this._getItemById(itemId) != null;
        },  
        
        _getItemById: function(id) {
            for (var i = 0; i < this._items.length; i++) {
                var item = this._items[i];
                var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;         
                if (itemId == id) {
                    return this._items[i];
                }
            }
            return null;
        },
        
        _getItemIndexByParticipantId: function(participantId) {
            for (var i = 0; i < this._items.length; i++) {
                if (this._items[i].participantId == participantId) {
                    return i;
                }
            }
            return null;
        },      
        
        _getItemByName: function(name) {
            for (var i = 0; i < this._items.length; i++) {    
                if (this._items[i].name == name) {
                    return this._items[i];
                }
            }
            return null;
        },
		
		validateStateTransition: function(initialState, action, endState, editable){
			//The only transiton we are interested in is to approved
			if (action == Constants.STATETRANSITION_ACTION_APPROVE && EditorUtil.applyTransactionConstraint(Constants.STATETRANSITION_PREFERENCE_ALLCHILDSCRIPTSAPPROVED)) {
				if (!this._initialized) //Is that the base way to know if the section has been initialized?
					this.initialize();
				
				//Now loop over all the items (test cases) if any to ensure that they are all approved
				if (this._items && this._items.length > 0) {
					for (var i = 0; i < this._items.length; i++) {
						if (this._items[i].state != Constants.STATETRANSITION_STATE_APPROVED) //Not approved
						{
							return Constants.getMessage("msgCannotTransitionNotAllTestScriptsApproved", EditorUtil.getStateNameFromID(endState));
						}
					}
				}
			}
			//If we got here than nothing offensive happened so return null which indicates that it is ok to transition
			return null;
		},
		
        _error: function(error) {
            console.error(this.declaredClass + ": Error occurred: " + error.message);
        }		
        
    })
    
})();
