/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.adapter.appscan.web.ui.internal.view.execution.ASEExecutionLog");
dojo.provide("com.ibm.rqm.adapter.appscan.web.ui.internal.view.execution.ASEExecutionLog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

// Utlities
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.execution.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.appscan.web.client.internal.ScanClient");
dojo.require("com.ibm.rqm.appscan.web.ui.internal.view.common.ASELink");
dojo.require("com.ibm.rqm.appscan.web.client.internal.Constants");
//Clients

(function() { // module container

var ImageCache				= com.ibm.rqm.execution.web.ui.internal.view.common.ImageCache;
var Utility					= com.ibm.rqm.execution.web.ui.internal.util.Utility;
var ScanClient				= com.ibm.rqm.appscan.web.client.internal.ScanClient;
var Constants 				= com.ibm.rqm.appscan.web.client.internal.Constants;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

dojo.declare("com.ibm.rqm.adapter.appscan.web.ui.internal.view.execution.ASEExecutionLog",[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.rqm.adapter.appscan.web", "ui/internal/view/templates/execution/ASEExecutionLog.html"),
		postCreate: function() {
			this._startDate = null; 
			this._initLabels();
		},
		
		/**
		 * Initialize the static labels on the widget
		 */
		_initLabels: function() {
			this._messageLabel.innerHTML = Constants.getMessage("executionlog_message");
			this._thresholdLabel.innerHTML = Constants.getMessage("executionlog_threshold");
			this._issuesLabel.innerHTML = Constants.getMessage("executionlog_issues");
			this._reportsLabel.innerHTML = Constants.getMessage("executionlog_reports");
			this._logLabel.innerHTML = Constants.getMessage("executionlog_scanlog");
		},
		
		initialize: function()
		{
			// Load the variables from script properties
			this._scriptProperties = this.result.script.properties;
			for (var i = 0; i < this._scriptProperties.length; i++) {
				if (this._scriptProperties[i].propertyName == Constants.JOBID) {
					this._jobId = this._scriptProperties[i].propertyValue;
				}
				else if (this._scriptProperties[i].propertyName == Constants.REPORTPACKID) {
					this._reportPackId = this._scriptProperties[i].propertyValue;
				}
				else if (this._scriptProperties[i].propertyName == Constants.SEVERITYTHRESHOLD) {
					this._severityThreshold = this._scriptProperties[i].propertyValue;
				}
			}
			
			// Load the execution result properties
			this._resultProperties = this.result.properties;
			this._issues = [];
			if (this._resultProperties) {
				for (var i = 0; i < this._resultProperties.length; i++) {
					if (this._resultProperties[i].propertyName == Constants.MESSAGE) {
						this._message.innerHTML = Constants.getMessage(this._resultProperties[i].propertyValue);
					}
					else if (this._resultProperties[i].propertyName == Constants.STARTDATE) {
						this._startDate = this._resultProperties[i].propertyValue;
					}
					else if (this._resultProperties[i].propertyName == Constants.ITERATION) {
						this._iteration = this._resultProperties[i].propertyValue;
					}
					else if (this._resultProperties[i].propertyName == Constants.SEVERITY_HIGH_TEXT
						|| this._resultProperties[i].propertyName == Constants.SEVERITY_MEDIUM_TEXT
						|| this._resultProperties[i].propertyName == Constants.SEVERITY_LOW_TEXT
						|| this._resultProperties[i].propertyName == Constants.SEVERITY_INFORMATION_TEXT) {
						this._issues.push({
							severityInt: this._severityToInt(this._resultProperties[i].propertyName),
							severity: this._resultProperties[i].propertyName,
							issue_count: this._resultProperties[i].propertyValue
						})
					}
				}
			}
			
			// Update severity threshold in HTML
			this._effectiveSeverityThreshold.innerHTML = this._lookupSeverityText(this._severityThreshold);
			
			// If the issues table has only one entry, then an error occured
			if (this._issues.length <= 1) {
				this._disableIssuesTable();
				this._disableReportLink();
				
				// If there's a start date, a log might exist so don't give up just yet
				if (!this._startDate) {
					this._disableLogLink();
					return;
				}
			}
			
			// Check if this is the latest run, in which case we can link to the reports and the log (or just the log if there were errors
			var scanSummaryResponseHandler = new ServiceResponseHandler(this, "_loadStartDateSuccess", "_loadStartDateFailure");
			ScanClient.getRunInfo(scanSummaryResponseHandler, {
				jobId: this._jobId,
				iteration: this._iteration				
			});
			
			if (this._issues.length > 1) {
				// Create the issues table	
				this._createIssuesTable();
			}
		},
		
		_issuesort: function(a, b){
			return b.severityInt - a.severityInt;
		},
		
		_eval: function(json){
			return eval( "(" + json + ")" );
		},
		
		_lookupSeverityText: function(severity){
			return Constants.getMessage("issue_severity_" + this._severityToText(severity));
		},
		
		/* 
		 * converts textual representation of severity to numeric
		 */
		_severityToInt: function(severity){
			if (severity == Constants.SEVERITY_HIGH_TEXT)
			return Constants.SEVERITY_HIGH;
			
			if (severity == Constants.SEVERITY_MEDIUM_TEXT)
			return Constants.SEVERITY_MEDIUM;
			
			if (severity == Constants.SEVERITY_LOW_TEXT)
			return Constants.SEVERITY_LOW;
			
			if (severity == Constants.SEVERITY_INFORMATION_TEXT)
			return Constants.SEVERITY_INFORMATION;
		},
		
		/* 
		 * converts numeric representation of severity to textual
		 */
		_severityToText: function(severity){
			if (severity == Constants.SEVERITY_HIGH)
			return Constants.SEVERITY_HIGH_TEXT;
			
			if (severity == Constants.SEVERITY_MEDIUM)
			return Constants.SEVERITY_MEDIUM_TEXT;
			
			if (severity == Constants.SEVERITY_LOW)
			return Constants.SEVERITY_LOW_TEXT;
			
			if (severity == Constants.SEVERITY_INFORMATION)
			return Constants.SEVERITY_INFORMATION_TEXT;
		},
		
		_loadStartDateSuccess: function(result) {
			var currentStartDate = this._eval(result).results[0].StartDate;
			if (currentStartDate != null && currentStartDate == this._startDate) {
				// Only populate reports link if issues were found
				if (this._issues.length > 1) {
					// Populate report link
					var path = "Simple/ViewScanResults.aspx?fiid=" + this._jobId;
					var link = new com.ibm.rqm.appscan.web.ui.internal.view.common.ASELink();
					var title = Constants.getMessage("reports_view_in_ase");
					link.initialize({
						path: path,
						title: title,
						text: title
					});
					this._reportLink.appendChild(link.domNode);
				}
				
				// Get the log count, in order to link to the log, as well
				var logResponseHandler = new ServiceResponseHandler(this, "_loadLogCountSuccess", "_error");
				ScanClient.getScanLogCount(logResponseHandler, {
					jobId: this._jobId,
					iteration: this._iteration
				});
			}
			else {
				this._disableReportLink();
				this._disableLogLink();
			}
		},
		
		_loadStartDateFailure: function(result) {
			this._disableReportLink();
			this._disableLogLink();
		},
		
		_loadLogCountSuccess: function(result){
			var logCount = this._eval(result).logCount;
			
			if (logCount > 0){
				// Populate view log link
				var path = "Jobs/ExportJobLogText.aspx?iteration=" + this._iteration + "&fiid=" + this._jobId;
				var link = new com.ibm.rqm.appscan.web.ui.internal.view.common.ASELink();
				var title = Constants.getMessage("scan_log_entries", logCount);
				link.initialize({ path : path, 
								  title : title, 
								  text: Constants.getMessage("scan_log_download")
								 });
					
				this._logLink.appendChild(link.domNode);
			}
			else {
				this._disableLogLink();
			}
		},
		
		_createIssuesTable: function() {					
			
			// Sort the issues
			this._issues.sort(this._issuesort);		
			
			var issues = this._issues;
						
			dojox.data.dom.removeChildren(this._issuesTable);
			
			var body = document.createElement("tbody");
			var head = document.createElement("thead");
			
			var tr = document.createElement("tr");
			var td = document.createElement("th");
			var label = document.createTextNode(Constants.getMessage("summary_severity"));
			td.appendChild(label);
			tr.appendChild(td);
			td = document.createElement("th");
			label = document.createTextNode(Constants.getMessage("summary_issues"));
			td.appendChild(label);
			tr.appendChild(td);
			head.appendChild(tr);
			this._issuesTable.appendChild(head);
			
			for (var i = 0; i < issues.length; i++) {
				tr = document.createElement("tr");
				td = document.createElement("td");
				
				// Hilight rows causing the test to fail
				if (issues[i].severityInt >= this._severityThreshold && issues[i].issue_count > 0) {
					tr.className = "highlight-row";
				}
				
				var severity = issues[i].severity;
				label = document.createTextNode(this._lookupSeverityText(this._severityToInt(severity)));
				
				var img = document.createElement("img");
				img.src = dojo.moduleUrl("com.ibm.rqm.adapter.appscan.web", "ui/internal/view/templates/common/images/issue_" + severity + ".gif");
				img.title = severity;
				img.style.paddingRight = "5px";
				
				td.appendChild(img);
				td.appendChild(label);
				tr.appendChild(td);
				
				td = document.createElement("td");
				td.align = "right";
				label = document.createTextNode(issues[i].issue_count);
				
				td.appendChild(label);
				tr.appendChild(td);
				
				body.appendChild(tr);
			}
			this._issuesTable.appendChild(body);
		},

		_disableIssuesTable: function()
		{
			var span = document.createElement("span");
			span.innerHTML = Constants.getMessage("error_text_none");
			this._issuesTable.appendChild(span);
		},
						
		_disableReportLink: function()
		{
			dojox.data.dom.removeChildren(this._reportLink);
			var span = document.createElement("span");
			span.style.color = '#CCCCCC';
			span.innerHTML = Constants.getMessage("reports_view_in_ase");
			this._reportLink.appendChild(span);
		},
				
		_disableLogLink: function()
		{
			dojox.data.dom.removeChildren(this._logLink);
			var span = document.createElement("span");
			span.innerHTML = Constants.getMessage("scan_log_download");
			span.title = Constants.getMessage("scan_log_empty");
			span.style.color = '#CCCCCC';
			this._logLink.appendChild(span);
		},
		
		_error: function(e)
		{
			console.debug("An error occurred: [" + e.message + "] @ ASEExecutionLog#_error");
		}
	}
);

}
)();