/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
console.debug("Loading: com.ibm.rqm.adapter.rpt.web.ui.internal.view.execution.PerformanceExecution");
dojo.provide("com.ibm.rqm.adapter.rpt.web.ui.internal.view.execution.PerformanceExecution");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.rqm.adapter.rpt.web.ui.internal.RptUiUtil");

//Dojo widget

(function() { // module container

var RptUiUtil	= com.ibm.rqm.adapter.rpt.web.ui.internal.RptUiUtil;

var RQM_LAUNCHING_STATUS_KEY = "RQM_LAUNCHING_STATUS_KEY";
var STAT_UNSET = "stat_unset";
	
var VALUE_CELL_SUFFIX = '-ValueCellID-';

dojo.declare("com.ibm.rqm.adapter.rpt.web.ui.internal.view.execution.PerformanceExecution",[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.rqm.adapter.rpt.web.", "ui/internal/view/templates/execution/PerformanceExecution.html"),
				
		progressTableInitialized: false,
		pTableData: null,
		
		postCreate: function() 
		{		
			this._labelContainer.appendChild(document.createTextNode(RptUiUtil.getMessage("progressSummary")));
		},
		
		showMessages: function(responseObject)
		{
			dojox.data.dom.removeChildren(this._messages);

			if (!responseObject){
				console.debug("no response received");
				return;
			}
			
			var messages = responseObject.messages;
				
			// Only deal with the last status message, not the accumlative list of messages.
			if (messages.length > 0) {
				
				var statusMessage = messages[messages.length-1].message;
								
				if(!statusMessage){
					statusMessage = "";
				}
				
				// Only show the launching message and tick marks.
				if(!this.progressTableInitialized){
					if (statusMessage.indexOf(RQM_LAUNCHING_STATUS_KEY) == 0) {
						var ticks = statusMessage.substr(RQM_LAUNCHING_STATUS_KEY.length);
						 
						var label = document.createElement('label');
						label.id = "_statusMessage";
						label.innerHTML = RptUiUtil.getMessage("RQM_LAUNCHING_STATUS_KEY") + ticks;
						this._messages.appendChild(label);
						this._messages.appendChild(document.createElement('br'));
					}
				}
			}
			
			this._processStatusUpdate(responseObject.properties);
			
			/****
			// Default growing status area.  RPT not using this anymore.
			var messages = responseObject.messages;
			for(var i=0;i<messages.length;i++)
			{
				var statusMessage = messages[i].message;
				var label = document.createElement('label');
				label.id = "_statusMessage"+i;
				label.innerHTML = statusMessage;
				this._messages.appendChild(label);
				this._messages.appendChild(document.createElement('br'));
			}
			****/
			
		},
		
		_error: function(errorObj) {
				console.error("Error: (message: '" + errorObj.message + "') from Performance Execution#_error");
		},
		
		_processStatusUpdate: function(properties){
			if(properties == null || properties.length < 1){
				return;
			}
				
			if(!this.progressTableInitialized){
				this.handleInitStatProperties(properties);
			}else{
				this.handleUpdateStatProperties(properties);
			}
			
		},
	
		handleInitStatProperties: function(properties){
						
			if(this.progressTableInitialized){
				return;
			}
			
			this.progressTableInitialized = true;
			
			for (var i = 0; i < properties.length; i++) {
				var propName = properties[i].propertyName;
				var propValue = properties[i].propertyValue;
				
				if(propValue == STAT_UNSET){
					propValue = "";
				}
				if(!this.pTableData){
					this.pTableData = new Object();
				}
				
				this.pTableData[propName] = propValue;
			}
			
			this.constructProgressTable();
		},

		handleUpdateStatProperties: function(properties){
						
			if(this.pTableData == null){
				return;
			}
				
			for (var i = 0; i < properties.length; i++) {
				var propName = properties[i].propertyName;
				var propValue = properties[i].propertyValue;
				
				if(propValue == STAT_UNSET){
					propValue = "";
				}
				
				// Get the value cell or this counter and update it's text.
				var statusValueCell = dojo.byId(this.id + VALUE_CELL_SUFFIX + propName);
				if(statusValueCell){
					dojox.data.dom.removeChildren(statusValueCell);
    				statusValueCell.appendChild(document.createTextNode(propValue));
				}	
			}	
		},
						
		// make the progress table visible.
		constructProgressTable: function() {
			if(!this.pTableData || !this._progressTableSection){
				return;	
			}
			
			var _progressTable = document.createElement("TABLE");
			_progressTable.className = "progressTable";
			this._progressTableSection.appendChild(_progressTable);
						
			var tdEven = true;
			
			for( var status_key in this.pTableData )
			{
				var statusText  = RptUiUtil.getMessage("Counter-" + status_key);
				var statusValue = this.pTableData[status_key];
				
				if(!statusText){
					statusText = RptUiUtil.getMessage("UnknownCounter");
				}
				
				if(!statusValue){
					statusValue = "";
				}
				
				statusText += ': ';
				
				var row	= _progressTable.insertRow(-1);
				
    			var statusKeyCell = row.insertCell(-1);
				if (tdEven) {
					statusKeyCell.className = "progressTableTD_Even";
				}else{
					statusKeyCell.className = "progressTableTD_Odd";
				}
    			statusKeyCell.appendChild(document.createTextNode(statusText));
		
		    	var statusValueCell = row.insertCell(-1);
				
				// Prefix with widget id in case multiple RQM tabs are showing different progress tables.
				statusValueCell.id = this.id + VALUE_CELL_SUFFIX + status_key;
				if (tdEven) {
					statusValueCell.className = "progressTableTD_Even";
				}else{
					statusValueCell.className = "progressTableTD_Odd";
				}
    			statusValueCell.appendChild(document.createTextNode(statusValue));	
				
				tdEven = !tdEven;	
			}
		}
		
});
})();	
