/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.rqm.adapter.commandline.web.ui.internal.view.extension.CommandLineScriptEditorSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.RemoteExecutionScriptClient");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.common.RemoteExecutionScriptEditorSection");

(function() { // module container

var   Constants                        = com.ibm.rqm.planning.web.client.internal.Constants;
var   Utility                          = com.ibm.rqm.planning.web.ui.internal.util.Utility;
var   AbstractArtifactEditorSection    = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
var	  EditorUtil					   = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
var   ScriptClient 					   = com.ibm.rqm.execution.web.client.internal.execution.RemoteExecutionScriptClient;
var   COMMAND					       = "command";
var   ARGUMENTS					       = "arguments"
var   REScriptEditorSection			   = com.ibm.rqm.execution.web.ui.internal.view.common.RemoteExecutionScriptEditorSection;

dojo.declare(
   "com.ibm.rqm.adapter.commandline.web.ui.internal.view.extension.CommandLineScriptEditorSection", 
   [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection, REScriptEditorSection], 
   {
   templatePath: dojo.moduleUrl("com.ibm.rqm.adapter.commandline.web", "ui/internal/view/templates/extension/CommandLineScriptEditorSection.html"),
   
   labelCommand: Constants.getMessage("labelCommand"),
   labelArguments: Constants.getMessage("labelArguments"),
   sectionName: Constants.getMessage("labelCmdLineSectionName"),
   
   constructor: function(args) {
      this._script = args.script;
   },
   
   postCreate: function() {},   

   initialize: function(){   
      if (this._script != null) {
		this._arguments.value = this.getScriptProperty(this._script.properties, ARGUMENTS);
	  	if(this.getScriptProperty(this._script.properties, COMMAND) == ""){
			//make the path here
			if(this._script.managedByAdapter){
				this._command.value = this._script.fullPath;
				this._command.readOnly  = false;
			}else{
				this._command.value = this._script.sharePrefix + this.getFileSeperater() + this._script.relativePath;
				this._command.readOnly  = true; 
			}
		}else
			this._command.value = this.getScriptProperty(this._script.properties, COMMAND);
	  }
	  this.createBrowseSection(this._selectionArea, this._browseContainer, this._adapterContainer);
	  if(!this._argumentsHandle)
	  	this._argumentsHandle = dojo.connect(this._arguments, "onkeypress", this, this.setDirty);
	  if(!this._commandHandle && this._script && this._script.managedByAdapter)
	  	this._commandHandle = dojo.connect(this._command, "onkeypress", this, this.setDirty);
	  if(this._script && !this._script.managedByAdapter && this._commandHandle)
	  	dojo.disconnect(this._commandHandle);
		
	
	  dojo.style(this.adapterSelector._labelScriptPath, "display", "none");
	  dojo.style(this.adapterSelector._scriptPath, "display", "none");      	            
   },
   
   setSelectedFiles: function(resource){
		var fileList = resource.fileList;
		if(fileList && fileList.length >0){
			this._command.value = fileList[0].resourceURN;
		}
   },
   
   setAdapterInfo: function(param){
		this._command.value = (param.scriptPath ? param.scriptPath:"") +this.getFileSeperater()+ (param.selectedScript ? param.selectedScript.name: "");
   },
   
   getProperties: function(){
	var props = [];
	props.push({
		propertyType: "string",
		propertyName: COMMAND,
		propertyValue: this._command.value
	});
	props.push({
		propertyType: "string",
		propertyName: ARGUMENTS,
		propertyValue: this._arguments.value
	});
	return props;
   },
   
   //@Overriden from RemoteExecutionScriptEditorSection
   getHelpContextID: function(){
      return null;
   },
   
   //@Overriden from RemoteExecutionScriptEditorSection
   getActions: function(){
      return [];
   },    
   
   validate: function(){
   	if (dojo.trim(this._command.value).length == 0) {
		 this.setValidationError(Constants.getMessage("msgRequiredField", this.labelCommand,  this.sectionName));
		 this._command.focus();
         return false;
      }
	  return true;
   },
     
   //@Overridden from RemoteExecutionScriptEditorSection
   saveFinished: function(testScript){
      this._script = testScript;
	  this.initialize();
	  this.setSectionDirty(false);
   },
   
   updateAdapterRadioClicked: function() {
   		this._command.readOnly  = false;
	},
	
	updateBrowseRadioListener: function() {
		this._command.readOnly  = true;
	}   
   

});

})();