/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.process.web.ui.internal.view.extension.QualityGoalSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextControl");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.GridContainer");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");

(function(){
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;

	dojo.declare("com.ibm.rqm.process.web.ui.internal.view.extension.QualityGoalTableProvider", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider, {
        getNumColumns: function(content){
            return content && content.numOfColumns > 0 ? content.numOfColumns : 1;
        },
        
        getColumnHeading: function(content, index){
            return content && content.numOfColumns > 0 ? content.columns[index].name : "";
        },
 
        
        getColumnWidth: function(content, atColumnIndex){
            return content && content.numOfColumns > 0 ? 100 / content.numOfColumns > 0 : 20;
        },
        
        getContentObjects: function(content){
            return content && content.numOfColumns > 0 ? content.records : [];
        },
 
        getCellDisplayType: function(rowData, atColumnIndex){
            var type = TableConstants.RENDER_TYPE_TEXT;
            return type;
        },
        
        getCellAlignment: function(rowData, atColumnIndex){
            return TableConstants.ALIGN_LEFT;
        },
        
        getCellDisplayData: function(rowData, atColumnIndex){
            return rowData && rowData.cells[atColumnIndex] ? rowData.cells[atColumnIndex].value : "";
        }
    });
	
    dojo.declare("com.ibm.rqm.process.web.ui.internal.view.extension.QualityGoalSection", [dijit._Widget, dijit._Templated, com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection], {
        templateString: '<div id="${id}" dojoAttachPoint="_gridEditorNode" style="height: 200px; width: 100%;"></div>',
        
        postCreate: function(){
            this.inherited('postCreate', arguments);
            
            //mocked data, TODO - parse the data from the section content
            console.debug(this.getSection().content);
            this._data = dojo.fromJson(this.getSection().content);
            this._isInit = false;
            this._isReadOnly = !this.isEditable();
            this.addWorkItemSupport();
        },
        
        //will be lazily loaded by getMainContent
        //@Overridden from AbstractArtifactEditorSection    
        initialize: function(isEditable){
            console.debug("Entering: Quality Goal Section - show editor");
			if (isEditable) {
				this._initilizeGridEditor();
			}
			else {
				this._initializeTableViewer();
			}
            console.debug("Existing: Quality Goal Section - show editor");
        },
        
        _initilizeGridEditor: function(){
            //Sample comparator to be used for sorting
            var data = this._data;
            var compare2ndLetter = function(inA, inB){
                // Sample Comparator for second alphabet
                // return <0, 0, >0
                return inA.charCodeAt(1) - inB.charCodeAt(1);
            }
            
            var model = new dojox.grid.data.Table([{}, {}, {}, {
                compare: compare2ndLetter
            }], data);
            
            // Layout
            var criteriaCell = {
                name: Constants.getMessage("labelDescriptionQualityGoal", Constants.getExtensionDisplayMessage(this.getTitle())),
                width: '280px',
                styles: 'text-align: left;'
            };
            var statusCell = {
                name: Constants.PROCESS_CURRENT_STATUS_QUALITYGOAL,
                width: '143px',
                styles: 'text-align: center;'
            };
            var metStateCell = {
                name: Constants.PROCESS_MET_STATUS_QUALITYGOAL,
                width: '160px'
            };
            var commentCell = {
                name: Constants.PROCESS_COMMENT_QUALITYGOAL,
                width: '143px'
            };
            if (!this._isReadOnly) {
                criteriaCell.editor = statusCell.editor = commentCell.editor = dojox.grid.editors.Input;
                metStateCell.editor = dojox.grid.editors.Select;
                metStateCell.options = [Constants.PROCESS_MET_QUALITYGOAL, Constants.PROCESS_MET_EXCEP_QUALITYGOAL, Constants.PROCESS_NOT_MET_QUALITYGOAL, Constants.PROCESS_NOT_STARTED_QUALITYGOAL, Constants.PROCESS_UNRESOLVED_QUALITYGOAL];
            }
            var gridLayout = [{
                defaultCell: {
                    styles: 'text-align: left;'
                },
                rows: [[criteriaCell, statusCell, metStateCell, commentCell]]
            }];
            dojox.data.dom.removeChildren(this._gridEditorNode);
            if (!this._grid) {
                this._grid = new com.ibm.rqm.planning.web.ui.internal.view.common.GridContainer({
                    model: model,
                    structure: gridLayout,
                    rowCount: "10",
                    elasticView: "2",
                    defaultStaticRow: [Constants.getMessage("labelDescriptionQualityGoal", Constants.getExtensionDisplayMessage(this.getTitle())), "", Constants.PROCESS_NOT_STARTED_QUALITYGOAL, ""],
                    onchange: dojo.hitch(this, "setDirty"),
					singleClickEdit: true
                });
            }
			var dojoGrid = this._grid.domNode.lastChild;
			
			dojoGrid.childNodes[0].firstChild.style.maxWidth = "100%";
			dojoGrid.childNodes[1].firstChild.style.maxWidth = "100%";
			dojoGrid.childNodes[1].firstChild.childNodes[2].style.maxHeight = "140px";
			dojoGrid.childNodes[1].firstChild.childNodes[2].firstChild.style.minHeight = "1px";
 
            this._gridEditorNode.appendChild(this._grid.domNode);
        },
        
		_initializeTableViewer: function(){
			dojox.data.dom.removeChildren(this._gridEditorNode);
 
            var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
                showPagingControl: false,
                showPagingRangeControl: false,
                showGroupFilterControls: false,
                showPagingActionsControls: false,    
                rowsCheckable: false,
                rowSelectIsSingle: false,
                pageSize: -1
            });
 
            var content = {
	            numOfColumns: 4,
	            columns: [{name: Constants.getMessage("labelDescriptionQualityGoal", Constants.getExtensionDisplayMessage(this.getTitle()))},
				           {name: Constants.PROCESS_CURRENT_STATUS_QUALITYGOAL},
						   {name: Constants.PROCESS_MET_STATUS_QUALITYGOAL}, 
						   {name: Constants.PROCESS_COMMENT_QUALITYGOAL}],  
	            records:[/*{
	                cells: [{
	                    value: ""   
	                }] 
	            }*/]
	        };
            var row = 0;
			var col = 0;
			
			if (this._data) {
				for (; row < this._data.length; row++) {
					var datarow = {
						cells: []
					};
					
					for (col = 0; col < 4; col++) {
						datarow.cells.push({
							value: this._data[row][col]
						});
					}
					
					content.records.push(datarow);
				}
			}
            tableViewer.setContentProvider(new com.ibm.rqm.process.web.ui.internal.view.extension.QualityGoalTableProvider());

			
			
            tableViewer.setContent(content);
            tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getExtensionDisplayMessage(this.getTitle())));
 
            tableViewer.show();
 
            this.tableViewer = tableViewer;
			
            this._gridEditorNode.appendChild(this.tableViewer.domNode);
		},
		
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            this.addWorkItemSupport();
        },
        
        //returns the persistent section DTO
        //@Overridden from AbstractArtifactEditorSection
        getSectionInfo: function(){
            this.inherited("getSectionInfo", arguments);
            if (this._grid || this.getSection().content) {
                this.getSection().attributes = [];
                this.getSection().attributes.push({
                    type: "richtext",
                    name: "content",
                    value: this._grid ? dojo.toJson(this._grid.getAllRows()) : this.getSection().content
                });
            }
            //TODO get grid data and marshell into section content
            return this.getSection();
        },
        
        //@Overriden from AbstractArtifactEditorSection
        isSummaryEditable: function(isEditable){
            return isEditable;
        },
        
        //@Overriden from AbstractArtifactEditorSection
        getSummaryChangeHandler: /* Function */ function(){
            return dojo.hitch(this, "_summaryChanged");
        },
        
        _summaryChanged: function(oldValue, newValue){
            this.description = newValue;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        getExpandHandler: /* Function */ function(){
            return dojo.hitch(this, "_showEditor");
        },
        
        _showEditor: function(){
            if (this._grid) {
                this._grid.render();
                if (!this._isInit && !this._isReadOnly) {
                    this._grid.initialize();
                    this._isInit = true;
                }
            }
        }
    });
})();
