/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.process.web.ui.internal.view.extension.ReviewSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant");

dojo.require("com.ibm.rqm.process.web.ui.internal.view.approval.MainReview");

(function(){ // module container
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    
    // global save context
    var SaveParticipant = com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant;
    
    dojo.declare("com.ibm.rqm.process.web.ui.internal.view.extension.ReviewSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: "<div dojoAttachPoint=\"_reviewSection\" class=\"com-ibm-rqm-approval-web-ui-internal-ReviewSection\"></div>",
        
        constructor: function(args){
        },
        
        postCreate: function(){
            this.inherited('postCreate', arguments);
            this.addWorkItemSupport();
        },
        
        //@Overriden from AbstractArtifactEditorSection - invoked on first expand
        initialize: function(isEditable){
            var artifactType = null;
            var parentEditor = this.getParentEditor();
            var self = this;
            var args = {
                itemId: self.getParentEditor().getVersionableItemId(),
				stateId: self.getParentEditor().getStateId(),
                parentController: self.parentController,
				parentSection: this,
                originator: (this.getArtifact() == null) ? null : this.getArtifact().originator,
                isEditable: isEditable,
                sectionContainer: this.container,
                section: this,
                sectionOwner: (this.getWorkItemInfo() == null) ? null : this.getWorkItemInfo().owner
            }
            
            switch (parentEditor.getArtifactType()) {
                case Constants.TYPE_CASE:
                    args.itemType = "VersionedTestCase";
                    break;
                case Constants.TYPE_PLAN:
                    args.itemType = "VersionedTestPlan";
                    break;
                default:
                    break;
            }
            this._widget = new com.ibm.rqm.process.web.ui.internal.view.approval.MainReview();
            this._widget.initialize(args);
            
            dojo.addClass(this._widget.minimizedForm, "hidden");
            dojo.removeClass(this._widget.maximizedForm, "hidden");
            dojox.data.dom.removeChildren(this._reviewSection);
            this._reviewSection.appendChild(this._widget.domNode);
            
			if(!this.saveParticipant)
            	this._addSaveParticipant();
        },
        
        _addSaveParticipant: function(){
            //create a SaveParticipant for the section
            this.saveParticipant = new SaveParticipant();
            this.saveParticipant.setDirty(false);
            this.saveParticipant.getItemType = function(){
                return "com.ibm.rqm.process.ApprovalRecord";
            };
			
			var self = this;
			this.saveParticipant.parent = this.getParentEditor();
			this.saveParticipant.saveContext = this.getSaveContext();
			
			this.saveParticipant.getItemId = function(){
                return self.getParentEditor().getVersionableItemId();
            };
			this.saveParticipant.getStateId = function(){
                return self.getParentEditor().getStateId();
            };
			this.saveParticipant.isDirty = function(){
				var param = self._getSaveParam();
				return (param.remove || param.add || param.update) ? true : false;
            };
			this.saveParticipant.getAttributes = function(){
				var param = self._getSaveParam();
				
				//create new referece if it is dirty
				if(param.remove || param.add || param.update) {
					this.saveContext.createReference(this, "artifactItemId", this.parent);
				}
                return {operationJson: dojo.toJson(param)};
            };
            this.getSaveContext().addParticipant(this.saveParticipant);
        },
		
		_getSaveParam: function() {
			return this._widget.approveWidget.getSavedParam();
		},
        
        //@Overriden from AbstractArtifactEditorSection
        //Summary: provide section level validation; this is a hack since any section
        //requiring validation should be using the global save's validation mechanism 
        //@Deprecated: WILL BE REMOVED IN M7B - See RQM WI 2562
        //@Abstract: Override to provide section level validation
        validate: function(){
            var validate = {};
            if (this._widget && this._widget.getApprovalWidget()) {
				var tableView = this._widget.getApprovalWidget();
				validate = tableView.validate();
				if (!validate.value) {
					this.getParentEditor().setHeaderStatus(Constants.VALIDATION_ERROR, Constants.HEADER_STATUS_ERROR, validate.msg);
					return false;
				}
			}
            return true;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            this.addWorkItemSupport();
			this.initialize(this.isEditable());
        },
        
        //only the artifact's owner or originator is the manager
        _isManager: function(){
            var artifact = this.getArtifact();
            var workItem = this.getWorkItem();
            var ownerItemId = (workItem == null) ? null : workItem.ownerItemId;
            var currentUser = EditorUtil.getCurrentContributor();
            var originator = (artifact == null) ? null : artifact.originator;
            if (this.isEditable() && (currentUser.name != "GUEST") &&
            (originator == null || ownerItemId == currentUser.itemId || originator.itemId == currentUser.itemId)) {
                return true;
            }
            return false;
        },
		
		validateStateTransition:function(initialState, action, endState, editable)
		{
			if (!this._widget)  //Is that the base way to know if the section has been initialized
				this.initialize(editable);

			return this._widget.validateStateTransition(initialState, action, endState);
		}	
    });
    
})();
