/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.extension.TestSuiteSummarySection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextControl");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");

(function(){ // module container

	var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var RichTextControl = com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextControl;
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
	var wrapText  = com.ibm.asq.common.web.ui.internal.util.wordWrapping.wrapText;
	
    dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.extension.TestSuiteSummarySection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: "<div dojoAttachPoint=\"_details\" class=\"com-ibm-rqm-test-planning-summary\">"+
							"<div class=\"editor\">"+
								"<div dojoAttachPoint=\"_categoriesNode\"></div>"+
								"<table width=\"100%\">"+
									"<tr>"+
										"<td width=\"125\" dojoAttachPoint=\"descriptionLabel\" valign=\"top\"></td>"+
										"<td colspan=\"3\">"+
							              "<div class=\"richText\" dojoAttachPoint=\"_richTextEditorNode\"></div>"+
										"</td>"+
									"</tr>"+
								"</table>"+
							"</div>"+
						"</div>",
                

        constructor: function(){
            this._workItem = null;
        },
        
        postCreate: function(args){
            this.inherited('postCreate', arguments);
            //set up workitem support for this section
            this.addWorkItemSupport(EditorUtil.getCurrentContributor().itemId);
			this.descriptionLabel.innerHTML = wrapText(
                        Constants.getMessage("labeldescription"), 
                        "", 
                        85, // hard code the max width since the dom node has not been added to the tree. 
                        true,
                        "<BR/>");
        },
        
		//@Overriden from AbstractArtifactEditorSection
        initialize: function(isEditable) {
            this._suite = this.getArtifact();
			
            //create the rich text control
			dojox.data.dom.removeChildren(this._richTextEditorNode);
			if (!this._richText) {
				this._richText = new RichTextControl({
                    editable: this.isEditable(),
					inEditMode: false,  //default to preview mode
					contentHTML: this.getSection().content ? this.getSection().content : ""
				});
			}			
			this._richTextEditorNode.appendChild(this._richText.domNode);
			dojo.connect(this._richText, "onChange", this, "setDirty");
			this._richText.show();
        },
               
        //@Overridden from AbstractArtifactEditorSection
        getAttributes: function(){
            var attributes = {};
            if (this._richText && this._richText.getContentHTML() != null) {
                attributes.description = this._richText.getContentHTML();
            }
            if (this.getWorkItemEditor()) {            
                attributes.ownerId = this.getWorkItemEditor().getOwnerId();                
            }
            return attributes;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
			this.initialized = false;
			this.addWorkItemSupport(true);
        },
        
        //@Overridden from AbstractArtifactEditorSection --> disable 'remove' capability for Summary section          
        getHeaderContent: /* String || DOM Node */ function(isEditable) {
            return null;
        },
        
		//@Overridden from AbstractArtifactEditorSection: 
		validate: function () {
			return true;
		},
		
		//@Overridden from AbstractArtifactEditorSection:		
        getSectionInfo: function() {      
            this.inherited("getSectionInfo", arguments);
            if (this._richText || this.getSection().content) {
                var strValue = this._richText ? this._richText.getContentHTML() : this.getSection().content;
                this.getSection().attributes = [];
                this.getSection().attributes.push({
                    type: "richtext",
                    name: "content",
                    value: strValue ? dojo.trim(strValue) : null
                });
            }
			return this.getSection();
        },
		
		_error: function(errorObj){
			console.error("Error: (message: '" + errorObj.message + "') from TestSuiteSummarySection#_error");
		}
		
    });
    
})();
