/*HomePageView***************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.itemtemplates.TemplatesView");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.template.TemplateClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TemplatesTableView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.itemtemplates.TemplateEditor");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

(function()
{   
	var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;	    
	var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
	
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var TemplateClient = com.ibm.rqm.planning.web.client.internal.template.TemplateClient;
	var TemplatesTableView = com.ibm.rqm.planning.web.ui.internal.view.table.TemplatesTableView;    
	var TemplateEditor = com.ibm.rqm.planning.web.ui.internal.view.itemtemplates.TemplateEditor;	
	var ViewHeaderLayout= com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
	
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.itemtemplates.TemplatesView",  [dijit._Widget, dijit._Templated], 
	{        		           
		templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/itemtemplates/TemplatesView.html"),        
			
		constructor: function() {
			this._view = null;
			this._templates = null;
		},
		
		// initialization      
		postCreate: function() {
			//load the reports
			this.initialize();		
		},
		
        initialize: function(){
			this._templates = null;			
            var templatesHandler = new ServiceResponseHandler(this, "_showTemplateTable", "_error");
            TemplateClient.getTemplates(templatesHandler, {});                      		  
		}, 
		
	    _userIsAdmin: function() {
	       var permissionInitializer = InitializationData.get("com.ibm.team.repository.service.canGrantPermissionInitializer");
	       return (permissionInitializer != null && permissionInitializer.canGrantPermission);
	    },		
				
		/*
		 * Create or Refresh the ReportsTableView widget 
		 */
		_showTemplateTable: function(templates) {
			console.debug(this.declaredClass + ":_showTemplateTable --> entry.");
			this._templates = templates;
            if (!this._templatesTable) {
				dojox.data.dom.removeChildren(this._templatesTableNode);
				var tArgs = {
					queryName: Constants.getMessage("labelAllTemplates"),
					templates: this._templates,
					actions: this._getTemplateTableActions(),
					refreshCallback: dojo.hitch(this, "initialize")					
				}
				this._templatesTable = new TemplatesTableView(tArgs);				
                this._templatesTableNode.appendChild(this._templatesTable.domNode);			     
			} else {
				this._templatesTable.refresh(this._templates);
			}			
		},
		
		_getTemplateTableActions: function() {
            var actions = [];            
            actions.push({
				id: "tp_create", //$NON-NLS$
                label: Constants.getMessage("labelCreateTemplate", Constants.TYPE_PLAN),
                actionHandler: dojo.hitch(this, "_handleNewTemplate", Constants.EDITOR_ID_PLAN),
                iconClass: "newplantemplate-action-icon-16"
            });
            actions.push({
				id: "tc_create", //$NON-NLS$
                label: Constants.getMessage("labelCreateTemplate", Constants.TYPE_CASE),
                actionHandler: dojo.hitch(this, "_handleNewTemplate", Constants.EDITOR_ID_CASE),
                iconClass: "newcasetemplate-action-icon-16"
            });  
            actions.push({
				id: "copy", //$NON-NLS$
                label: Constants.getMessage("labelCopyTemplate"),
                actionHandler: dojo.hitch(this, "_handleCopyTemplate"),
                iconClass: "template-icon-16",
				requiresSelection: true
            });  			
            actions.push({
				id: "setDefault", //$NON-NLS$
                label: Constants.getMessage("labelSetDefaultTemplate"),
                actionHandler: dojo.hitch(this, "_handleSetDefault"),
                iconClass: "template-default-icon-16",
				requiresSelection: true
            });
            actions.push({
				id: "archive", //$NON-NLS$
                label: Constants.getMessage("labelArchive"),
                actionHandler: dojo.hitch(this, "_handleArchive"),
                iconClass: "delete-icon-image",
                requiresSelection: true
            });
            							
			return actions;
		},
		
		_handleNewTemplate: function(editorId) {
	        var editor = new TemplateEditor({
	            editorId: editorId,
				onOK: dojo.hitch(this, "initialize")
	        });			
		},
		
        _handleCopyTemplate: function(editorId) {
			var selected = this._templatesTable.getSelected();
            if (!dojo.isArray(selected) || selected.length == 0) {
                alert(Constants.getMessage("msgTemplateMissing", Constants.getMessage("labelCopy")));
				return;
            } else if (selected.length > 1) {
                alert(Constants.getMessage("msgTemplateSingle", Constants.getMessage("labelCopy")));
				return;
            }
			
			var copy = selected[0];
			delete copy.itemId;
			delete copy.id;
			copy.modifiable = true;
            var editor = new TemplateEditor({                
				template: copy,
				isCopy: true,
                onOK: dojo.hitch(this, "initialize")
            });         
        },	
		
        _handleSetDefault: function(editorId) {
            var selected = this._templatesTable.getSelected();
            if (!dojo.isArray(selected) || selected.length == 0) {
                alert(Constants.getMessage("msgTemplateMissing", Constants.getMessage("labelSetDefaultTemplate")));
                return;
			} else if (selected.length > 1) {
                alert(Constants.getMessage("msgTemplateSingle", Constants.getMessage("labelSetDefaultTemplate")));
				return;
            }
            
            var template = selected[0];
			var srh = new ServiceResponseHandler(this, "initialize", "_error");
            TemplateClient.postSetDefaultTemplate(srh, template.editorId, template.itemId);
        },		
		
        _handleArchive: function(editorId) {
            var selected = this._templatesTable.getSelected();
            if (!dojo.isArray(selected) || selected.length == 0) {
                alert(Constants.getMessage("msgTemplateMissing", Constants.getMessage("labelArchive")));
                return;
			} else if (selected.length > 1) {
                alert(Constants.getMessage("msgTemplateSingle", Constants.getMessage("labelArchive")));
                return;
			}

            var self = this;
            var template = selected[0];
            var args = {
                title: Constants.getMessage("labelConfirmation"),
                message: Constants.getMessage("msgConfirmArchiveTemplate", template.name),
                yesCallback: function() {   
                    //archive the template		            
		            template.archive = true;
		            var args = {
		                itemId: template.itemId,
		                id: template.id,
		                editorId: template.editorId,
		                name: template.name,
		                description: template.description?template.description:"",
		                archive: true
		            }
		            var srh = new ServiceResponseHandler(self, "initialize", "_error");
		            TemplateClient.postTemplate(srh, args);
                    return true;          
                },
                noCallback: function() {
                    return false;
                }
            }
            var dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
            dialog.setSecondaryTitle(args.title);
            dialog.open();			        
        },			
		
		_error: function(e) {
			console.error("TemplatesView: Error: " + e.message);
		}	

	});
})();
