/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.reports.web.ui.internal.view.preferences.ReportsSystemPreferences");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.reports.web.client.internal.ReportClient");

(function(){
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var ReportClient = com.ibm.rqm.reports.web.client.internal.ReportClient;
	
	var FILE_UPLOAD_RESPONSE_PREFIX = "File Upload Response:"; // Must match response prefix in AttachmentRestService.java
	var ATTACHMENT_SERVICE_URI = "com.ibm.rqm.planning.service.internal.rest.IAttachmentRestService";
	
    dojo.declare("com.ibm.rqm.reports.web.ui.internal.view.preferences.ReportsSystemPreferences", [dijit._Widget, dijit._Templated], {
        
		templatePath: dojo.moduleUrl("com.ibm.rqm.reports.web", "ui/internal/view/templates/preferences/ReportsSystemPreferences.html"),						
		
		labelLoading: Constants.getMessage("labelLoading"),				
		labelUpload: Constants.getMessage("labelUpload"),
		labelUploading: Constants.getMessage("labelUploading"),
		labelSaving: Constants.getMessage("labelSaving"),
		labelDefault: Constants.getMessage("labelDefault"),
		labelRevert: "Use Default",
		labelReportHeader: Constants.getMessage("labelReportHeader"),
		labelCurrentHeader: Constants.getMessage("labelCurrentHeader"),
		labelReportFooter: Constants.getMessage("labelReportFooter"),
		labelCurrentFooter: Constants.getMessage("labelCurrentFooter"),
		
		_headerSaving: false,
		_footerSaving: false,
		editable: true,
        
		postCreate: function(){
			this.inherited("postCreate", arguments);
            if(!this.editable) {
                this.headerFileInputNode.disabled = true;
                this.footerFileInputNode.disabled = true;
            }
            
            //get the preferred header image information
            var headerHandler = new ServiceResponseHandler(this, "_setHeader", "_error");
            ReportClient.getHeaderAttachment(headerHandler);

            //get the preferred footer image information
            var footerHandler = new ServiceResponseHandler(this, "_setFooter", "_error");
            ReportClient.getFooterAttachment(footerHandler);			
		},
		
		_setHeader: function(attachment){
            this._headerAttachment = attachment != null ? attachment : null;
			this._headerDefaultButton.disabled = attachment != null ? false : true;
            //this.headerFileInputNode.textContent = attachment != null ? attachment.name : this.labelDefault;
			dojox.data.dom.removeChildren(this._currentHeaderNode);
			this._currentHeaderNode.style.width = "200px";
            this._currentHeaderNode.innerHTML = attachment != null ? attachment.name : this.labelDefault;
            this._headerDefaultButton.style.display = attachment != null ? "inline" : "none";
		},
		
		_setFooter: function(attachment) {
            this._footerAttachment = attachment != null ? attachment : null;
            this._footerDefaultButton.disabled = attachment != null ? false : true;
            //this.footerFileInputNode.textContent = attachment != null ? attachment.name : this.labelDefault;
            dojox.data.dom.removeChildren(this._currentFooterNode);
            this._currentFooterNode.innerHTML = attachment != null ? attachment.name : this.labelDefault;
			this._footerDefaultButton.style.display = attachment != null ? "inline" : "none";
		},
		
		_uploadHeader: function(){
			this.updateHeaderInputState(true);
			this.uploadImportFile(this.importHeaderFormNode, this.headerFileInputNode, "headerFileInput", ReportClient.ATTACHMENT_TYPE_HEADER);
		},
		
		_uploadFooter: function(){
			this.updateFooterInputState(true);
			this.uploadImportFile(this.importFooterFormNode, this.footerFileInputNode, "footerFileInput", ReportClient.ATTACHMENT_TYPE_FOOTER);
		},
		
		updateHeaderInputState: function(isSaving){
			this._headerSaving = (isSaving === true);
			this._headerProgressNode.style.display = this._headerSaving == true ? "inline" : "none";
			this.headerUploadButton.disabled = this._headerSaving || (this.headerFileInputNode.value == null);
		},
		
		updateFooterInputState: function(isSaving){
			this._footerSaving = (isSaving === true);
			this._footerProgressNode.style.display = this._footerSaving == true ? "inline" : "none";
			this.footerUploadButton.disabled = this._footerSaving || (this.footerFileInputNode.value == null);
		},
		
		uploadImportFile: function(formElement, inputArea, fileFields, attachmentType) {
            var formNode = dojo.byId(formElement);
            var self = this;
            this._currentBindRequest = dojo.io.iframe.send({
                url: TeamServerClient._getServicePrefix()  + ATTACHMENT_SERVICE_URI + "/",
                method: "POST",
                content: {
						mimetype: "text/html",
						fileFields: fileFields
					},
				handleAs:  "html",
				timeout: 300000, // 5 mins
                form: formNode, // the dom node of the form containing the file reference. 
                load: function(importId, msg) {
                    console.debug('Registered import with ID ' + importId);
                    return importId;
                },
                error: function(type, msg) {
                    console.debug('Encountered upload error while importing.');
                    return type;
                },
				handle:function(response, ioArgs){
					var responseText = response.body.textContent;
					var uuidOrMessage = responseText.substring(FILE_UPLOAD_RESPONSE_PREFIX.length, responseText.length).split(',');
					if(dojo.isArray(uuidOrMessage) && uuidOrMessage.length == 3){
						self._doPostAttachment(attachmentType, uuidOrMessage[0]);
					}
				}
            });
        },
		
		_doPostAttachment: function(type, attachmentUUID){
			if (type != ReportClient.ATTACHMENT_TYPE_HEADER && type != ReportClient.ATTACHMENT_TYPE_FOOTER) {
				console.error("Invalid type specified!")
			}
			
            var responseHandler = new ServiceResponseHandler(this, (type == ReportClient.ATTACHMENT_TYPE_HEADER ? "_headerSaved" : "_footerSaved"), "_error");
            ReportClient.setAttachment(responseHandler, type, attachmentUUID);
		},
		
		_headerSaved: function(attachment) {
			this._setHeader(attachment);
			this.updateHeaderInputState(false);
		},
		
        _footerSaved: function(attachment) {
            this._setFooter(attachment);
            this.updateFooterInputState(false);
        },		
		
		_revertHeader: function() {
			var responseHandler = new ServiceResponseHandler(this, "_setHeader", "_error");
			ReportClient.resetHeaderAttachment(responseHandler);
		},
		
		_revertFooter: function() {
            var responseHandler = new ServiceResponseHandler(this, "_setFooter", "_error");
            ReportClient.resetFooterAttachment(responseHandler);			
		},
		
		_error: function(errorObj) {
			console.error("ReportsSystemPreferences: An error occurred: " + errorObj);
		}
		
    });
})();
