/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.configuration.web.ui.internal.view.table.ConfigurationFilterTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");


dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var Constants  	   = com.ibm.rqm.planning.web.client.internal.Constants;

dojo.declare("ConfigurationFilterTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
	constructor: function(parentView) {
		this.parentView = parentView;
    },
	
	setRowCheckListener: function(listener) {
		this.rowCheckListener = listener;
	},
	
    // Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called.
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
		var selectionCount = tableViewer.getCheckedRows().length;
	        if (this.parentView.removeActionNode)
	            this.parentView.removeActionNode.setDisabled(selectionCount == 0);
    },
	
	// Summary:
    //    If the client have registered table actions, this message will be fired when an action (button)
    //    is invoked by the user.
    //
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback)
    {
		if (callback != null && dojo.isFunction(callback)) {
		        callback();
		   	}  
    }
});

dojo.declare("ConfigurationFilterTableView.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    cellValueChanged: function(oldValue, newValue, rowObject, columnIndex)
    {       
		rowObject.values[columnIndex] = newValue; 
    },
    
    cellActionInvoked: function(rowObject, columnIndex)
    {
		return;
    }
});

dojo.declare("ConfigurationFilterTableView.ListRowRenderer",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer,
{    
    // Summary:
    //   Override the base list row renderer's create list content method
    //   to supply more custom contents.
    //
    _createListContents: function(viewer, rowObject, listWidget)
    {
        this.inherited("_createListContents", arguments);   
    }
});

dojo.declare("ConfigurationFilterTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
	environments: [],
	
	constructor: function(args) {
		this.environments = args.environments ? args.environments : [];
		this.parent = args.parent ? args.parent : null;
    },
	
    // Summary:
    //
    getNumColumns: function(content)
    {
        return this.environments.length;
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
        return this.environments[index].key;
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {
        return null;
    },
	 
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return this.getCellAlignment(null, atColumnIndex);
    },	 
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
        return false;     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return false;   
    },
    
    getColumnWidth: function(content, atColumnIndex)
    {
		var width = 100 / this.getNumColumns();
        return width;
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        // our content is already an array of objects
        return content;
    },
    
    getRowDataId: function(rowData)
    {
        return rowData.itemId;
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        return TableConstants.RENDER_TYPE_ENUM;
    },

    getCellAlignment: function(rowData, atColumnIndex)
    {
        return TableConstants.ALIGN_LEFT;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {
		return rowData.values[atColumnIndex] ? rowData.values[atColumnIndex] : "";
    },

    
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
		var val = {};
		val.select = rowData.values[atColumnIndex] ? rowData.values[atColumnIndex] : "";
		var choices = [];
		if(this.parent && this.parent.tableType){
			choices.push({
				'text' : (this.parent.tableType == "exclusions") ?  Constants.getMessage("labelAll") : Constants.getMessage("labelAny"),
				'value' : "DC"
			});
		}
		for(var i = 0; i < this.environments[atColumnIndex].values.length; i++){
			choices.push({
				'text': this.environments[atColumnIndex].values[i], 
				'value': this.environments[atColumnIndex].values[i] 
				});
		}
		val.choices = choices;
		return val;
    }
    
    
});

dojo.declare("com.ibm.rqm.configuration.web.ui.internal.view.table.ConfigurationFilterTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div dojoAttachPoint="_tableViewerNode" tabindex="-1"></div>',

	environments: [], 
	newItemId: 1, // The itemId of content item, increase when new item is add to the table
	tableType: null,

    postCreate: function() 
    {
        this.inherited("postCreate", arguments);
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer();        
        this._tableViewerNode.appendChild(tableViewer.domNode);
		          
        var content = this.content ? this.content : [];
		  
        tableViewer.setContentPageSize(10);
        tableViewer.setContent(content);
        tableViewer.setContentProvider(new ConfigurationFilterTableView.TableProvider({environments: this.environments, parent: this}));		  
		tableViewer.setTableListener(new ConfigurationFilterTableView.TableListener(this));
        tableViewer.setCellModifier(new ConfigurationFilterTableView.CellModifier());
		tableViewer.setTableRowsCheckable(true, /* isSingleSelect */ false);

		//add actions to the table viewer
        this.addActionNode = tableViewer.addAction("add", Constants.getMessage("labelAddExclusion"), null, "add-icon-image", dojo.hitch(this, "_handleAddNewClick"));
    	this.removeActionNode = tableViewer.addAction("remove", Constants.getMessage("labelRemoveExclusion"), null, "delete-icon-image", dojo.hitch(this, "_handleDeleteClick"));
		this.removeActionNode.setDisabled(true);
		       
        tableViewer.show();
		  
		this.tableViewer = tableViewer;
    },
	
	_handleDeleteClick: function(){
		var selectedItems = this.getSelectedItems();
		for(var j = 0; j < selectedItems.length; j++){
			this.tableViewer.deleteItem(selectedItems[j]);
		}
		this.refresh();
	},
	
	_handleAddNewClick: function(){
		var content = this.tableViewer.getContent();
		var values = [];
		for( var i = 0; i < this.environments.length; i++){
			values.push(this.environments[i].values[0] ? this.environments[i].values[0] : "");
		}
		var newItem = {
			itemId: this.newItemId++,
			values: values
			};
	    content.push(newItem);  
	    this.refresh(content);
	},
	
	//API
	getTableViewer: function(){
		return this.tableViewer;
	},
	 
	// API
	getSelectedItems: function() {
	  	return this.tableViewer.getCheckedRows();
	},
	
	// API
    refresh: function(content) {
		if(content){
		  	this.tableViewer.setContent(content);
		}
	    this.tableViewer.refresh();
    }	 
});

})();
