/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.datapool.web.ui.internal.view.table.DataTableHeaderRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableHeaderRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.GridTableHeaderRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var SelectUtility  = com.ibm.asq.common.web.ui.internal.util.SelectUtility;

dojo.declare("com.ibm.rqm.datapool.web.ui.internal.view.table.DataTableHeaderRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.GridTableHeaderRenderer,
{
	columnTypeEditors: [],
	
    createTableHeader: function(table)
    {
        
        // not show table heading. 
        if(!this.viewer.getIsShowingTableHeading())
            return;
            
        var provider = this.viewer.getContentProvider();
        var content = this.viewer.getContent();
        var nColumns = provider.getNumColumns(content);
        var thead = table.createTHead();
        var headerTopTr, headerBotTr;
        var headerContentTr;
        var groupBy = this.viewer.getGroupByColumn();
        var atCol = 0;
        var slackWidth = 0;
        var nVisible = nColumns;
        var reduceWidth = 0;
        var tableHeaderStyle = "table-header-content";

        // set table header style for CJK pictographic based locales
        if (dojo.locale.indexOf("zh") == 0 ||
            dojo.locale.indexOf("ja") == 0 ||
            dojo.locale.indexOf("ko") == 0) {
            tableHeaderStyle = "table-header-content-cjk";
        }
        
        this.tableHeaders = {};
        this.sortByColumn = this.viewer.getSortByColumn();
        this.sortDir = this.viewer.getSortDirection();
        
        for (var i = 0; i < nColumns; i++) {
            if(!this.viewer.isColumnVisible(i) || (groupBy == i)) {
                nVisible--;
            }
        }
        if(nVisible == 0)
            return;

        // we're going to remove the group column,
        // distribute its width amongst all columns
        if(groupBy > -1 && this.viewer.isColumnVisible(groupBy)) {
            slackWidth = Math.floor(this.viewer.getColumnWidth(groupBy) / nVisible);
        }
        
        headerTopTr = table.insertRow(table.rows.length); 
        headerContentTr = table.insertRow(table.rows.length); 
        headerBotTr = table.insertRow(table.rows.length); 
        
        // The rows in the table are checkable, insert
        // a column to provide a check box for the rows
        if(this.viewer.getTableRowsCheckable()) {
            this.createHeaderBorderLine(table, headerTopTr, true);
            
            var checkbox_th = document.createElement("th");
            dojo.addClass(checkbox_th, tableHeaderStyle);
            
            // create a check box to select/deselect all rows in the table.
            if (!this.viewer.getIsSingleSelectionMode() && this.viewer.getIsAllowSelectAll()) {
                dojo.addClass(checkbox_th, "align-center");
                checkbox_th.vAlign = "center";
                checkbox_th.title = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_selectAll");

                var checkState = (this.checkAllState) ? true : false;
                var button = new dijit.form.CheckBox({
                    'checked': checkState
                });
                button.headerRenderer = this;
                button.onChange = function() {
                    this.headerRenderer.checkAllState = this.checked;
                    this.headerRenderer.viewer.performSelectAll(this.checked);
                };
                checkbox_th.appendChild(button.domNode);
            }
            else {
                checkbox_th.width = 20;
            }
            headerContentTr.appendChild(checkbox_th);
            this.createHeaderBorderLine(table, headerBotTr);
        }
        
        // if there's a group by, insert a filler column for the
        // indentation.
        if(groupBy > -1) {
            this.createHeaderBorderLine(table, headerTopTr, true);
            var gr_th = document.createElement("th");
            dojo.addClass(gr_th, tableHeaderStyle);
            headerContentTr.appendChild(gr_th);            
            gr_th.width = 8;
            this.createHeaderBorderLine(table, headerBotTr);
        }
        
		this.columnTypeEditors = [];
        for(var i = 0; i < nColumns; i++) {
            // skip displaying the group by column and non-visible columns.
            if(groupBy == i || !this.viewer.isColumnVisible(i)) {
                continue;
            }
            this.createHeaderBorderLine(table, headerTopTr, true);
            
            var title = provider.getColumnHeading(content, i);
            var th = document.createElement("th");
             
            dojo.addClass(th, tableHeaderStyle);
            th.width = (this.viewer.getColumnWidth(i) + slackWidth - reduceWidth) + "%";
            th.vAlign = "center";
            var align = provider.getColumnHeadingAlignment(i);

            if(align == TableConstants.ALIGN_CENTER) {
               dojo.addClass(th, "align-center");
            } 
            else if(align == TableConstants.ALIGN_RIGHT) {
               dojo.addClass(th, "align-right");   
            } 
            else {
                dojo.addClass(th, "align-left")
            }
                            
            if (provider.getIsColumnSortable(content, i)) {
                var anchor = document.createElement("a");
                var titleText = (title && title.length > 0) ? title : "\u00a0";
                anchor.appendChild(document.createTextNode(titleText));
                anchor.colIndex = i;
                anchor.name = "anchor";
                anchor.className = "table-link";
                th.appendChild(anchor);
                dojo.connect(anchor, "onclick", this, "_handleTableHeadSortBy");
                anchor.title = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_nosort");
                // Create the sort indicator.
                var img = this._locateHeaderSortDirImageNode(th, i, true);
                var isAscend = (this.sortDir == TableConstants.SORT_DIR_ASCEND);
                img.src = (isAscend) ? ImageCache.SORT_ASC : ImageCache.SORT_DSC;
                dojo.style(img, "display", ((this.sortByColumn == i) ? "inline" : "none"));
                if (this.sortByColumn == i) {
                    anchor.title = (isAscend) ? MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_sortAscend") : MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_sortDescend");
                }
            }
            else if (title != null && dojo.trim(title).length > 0){
                th.appendChild(document.createTextNode(title));
            }
            else {
                var spacer = document.createElement('span');
                spacer.style.width = '8px';
                th.appendChild(spacer);
            }
            // if the column header has a image, create it.
            var imgRef = provider.getColumnHeadingImage(content, i);
            if(imgRef) {
                var src = (dojo.isObject(imgRef) && imgRef.src) ? imgRef.src : imgRef;
                var isAction = (dojo.isObject(imgRef) && imgRef.actionId) ? true : false;
                // create a margin node if we already have
                // something created before the image.
                if(th.childNodes && th.childNodes.length > 0) {
                    var spacer = document.createElement("span");
                    spacer.className = "table-head-spacer";
                    th.appendChild(spacer);
                }
                
                var img = document.createElement("img");
                img.className = "table-head-image";
                img.src = src;
                th.appendChild(img);
            }
            
            // create the column type editor
			
			var editor = document.createElement("select");
			SelectUtility.add(editor, new Option("STRING", "STRING"));
			SelectUtility.add(editor, new Option("NUMBER", "NUMBER"));
			SelectUtility.add(editor, new Option("BOOLEAN", "BOOLEAN"));
			SelectUtility.add(editor, new Option("ENUMERATION", "ENUMERATION"));
			editor.style.display = "block";
			th.appendChild(editor);
			this.columnTypeEditors.push(editor);
			editor.style.display = "none"; 
			
			
            // attach the sort indicator if we've a sort defined.
            if(this.sortByColumn == i) {
                var img = this._locateHeaderSortDirImageNode(th, i, true);
                var isAscend = (this.sortDir == TableConstants.SORT_DIR_ASCEND);
                img.src = (isAscend) ? ImageCache.SORT_ASC : ImageCache.SORT_DSC;
                dojo.style(img, "display", "inline");
                th.title = (isAscend) ? 
                    MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_sortAscend") : 
                    MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_sortDescend");            
            }
            
            headerContentTr.appendChild(th);
            this.createHeaderBorderLine(table, headerBotTr);
            
            // store the column header in our associated array indexed
            // by the column position string key.
            this.tableHeaders[this._getTableHeadKey(i)] = {'thead': th};
        }
        
        thead.appendChild(headerTopTr);
        thead.appendChild(headerContentTr);    
        thead.appendChild(headerBotTr);    
          
        table.appendChild(thead);    
        this.viewer.contentRowStart = table.rows.length;
    }
});


})();
