/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.rqm.planning.web.client.internal.UserClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.PreferenceClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.RegistryClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredTableView");

(function() {

var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var TableConstants 		= com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     		= com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Utility        		= com.ibm.rqm.planning.web.ui.internal.util.Utility;
var OptionsDialog    	= com.ibm.asq.common.web.ui.internal.widgets.OptionsDialog;
var TestScriptClient	= com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient;
var GlobalEvents   		= com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
var Constants      		= com.ibm.rqm.planning.web.client.internal.Constants;
var PermissionsClient   = com.ibm.asq.common.web.client.internal.PermissionsClient;
var ViewHeaderLayout    = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
var UserClient 			= com.ibm.rqm.planning.web.client.internal.UserClient;    
var RegistryClient      = com.ibm.rqm.planning.web.client.internal.RegistryClient;
var TableContentFilterProvider = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider;
var DEFAULT_SCRIPT_TYPE = "Manual";
var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
var AbstractFilteredTableView = com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredTableView;
var PreferenceClient = com.ibm.rqm.planning.web.client.internal.PreferenceClient;

dojo.declare("TestScriptTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
    constructor: function(parentView) {
        this.parentView = parentView;
    },
	// Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called.
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
        var selectionCount = tableViewer.getCheckedRows().length;
        if (this.parentView.deleteActionNode)
            this.parentView.deleteActionNode.setDisabled(selectionCount == 0 || !tableViewer.isEditable());
		
		if (this.parentView.bulkchangeActionNode)
            this.parentView.bulkchangeActionNode.setDisabled(selectionCount == 0);
			
        //if table view has a rowCheckListener, invoke it
        var p = this.parentView;
        if (p && p.rowCheckedListener && dojo.isFunction(p.rowCheckedListener)) {
            p.rowCheckedListener(rowObject, state);
        }			
    },
    
    // Summary:
    //    A group check box is shown when the tables rows are checkable and the in multi-select mode.
    //    This method is called when the group's check box is checked/unchecked. The affected rows and 
    //    their states are passed.
    //
    groupRowsCheckStateChanged: function(tableViewer, rowObjects, state) 
    {
        this.rowCheckStateChanged(tableViewer, rowObjects, state);
    },
	
    // Summary:
    //    If the client have registered table actions, this message will be fired when an action (button)
    //    is invoked by the user.
    //
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback)
    {                
        if (callback != null && dojo.isFunction(callback)) {
            if (callbackData != null) {
                callback(callbackData);
            } else {
                callback();
            }
        } 
    },

    tableContentFilterActionInvoked: function(tableViewer, actionId, filters, callback, callbackData)
    {
        if(callback) {
            callback(actionId, filters);
        }                     
    },
	
	tableHeaderColumnActionInvoked: function(tableViewer, columnIndex, /*optional*/actionId)
    {
		if (this.parentView.parent && this.parentView.parent._TestScriptHeaderColumnActionInvoked) {
			this.parentView.parent._TestScriptHeaderColumnActionInvoked(tableViewer, columnIndex, actionId);
		}         
    }            
});

dojo.declare("TestScriptTableView.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    cellValueChanged: function(oldValue, newValue, rowObject, columnIndex)
    {
        alert(Constants.getMessage("labelCellValueChanged",rowObject.col1,newValue));
        
        // store the new values in our internal data model
        if(columnIndex == 6) {
            if(dojo.isString(newValue)) {
                rowObject.selection = newValue;
            }
        }
    },
    
    cellActionInvoked: function(rowObject, columnIndex)
    {
		//handle delete action
		if (columnIndex == 5) {
		   alert(Constants.getMessage("labelNotYetSupported"));
		}
    }
});

dojo.declare("TestScriptTableView.ListRowRenderer",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer,
{    
    // Summary:
    //   Override the base list row renderer's create list content method
    //   to supply more custom contents.
    //
    _createListContents: function(viewer, rowObject, listWidget)
    {
        this.inherited("_createListContents", arguments);   
        
        var moreContents = document.createElement("div");
        var toggle = document.createElement("button");
        toggle.innerHTML = "Added Custom Control";
        moreContents.appendChild(toggle);
        listWidget.contentNode.appendChild(moreContents);
    }
});

dojo.declare("TestScriptTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    constructor: function(args) {
        dojo.mixin(this, args);		
        this.COLUMNS = {
            TYPE: this.toColumn("", 2, false, TableConstants.ALIGN_CENTER, TableConstants.RENDER_TYPE_IMAGE, null),
            ID: this.toColumn(Constants.getMessage("labelId"), 3, false, TableConstants.ALIGN_CENTER, TableConstants.RENDER_TYPE_LINK, 'id'),
            NAME: this.toColumn(Constants.getMessage("labelName"), 40, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_LINK, 'name'),
			STATE: this.toColumn(Constants.getMessage("labelState"), 10, true, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_TEXT, 'state'),
            SCRIPT_TYPE: this.toColumn(Constants.getMessage("labelScriptType"), 20, true, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_TEXT, null),
            MODIFIED: this.toColumn(Constants.getMessage("labelModified"), 25, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_DATE, 'modified')
        }
		
        this.COLUMNS.ALL = [this.COLUMNS.TYPE, this.COLUMNS.ID, this.COLUMNS.NAME, this.COLUMNS.STATE, this.COLUMNS.SCRIPT_TYPE, this.COLUMNS.MODIFIED];
		this.COLUMNS.MINIMAL = [this.COLUMNS.TYPE, this.COLUMNS.ID, this.COLUMNS.NAME, this.COLUMNS.SCRIPT_TYPE];
		
        this.columnProfile = null;
        this.columnProfileName = null;
        this.setProfile(this.profile ? this.profile : "ALL"); //default to 'ALL' (full) column profile 	
    },
    
    toColumn: function(name, width, groupable, alignment, type, rowName) {
        return {
            name: name,
            width: width,
            groupable: groupable,
            alignment: alignment,
            type: type,
            rowName: rowName
        }
    },  
	
    setProfile: function(/* string */ profile) {
        if (this.COLUMNS[profile]) {
            this.columnProfileName = profile;
            this.columnProfile = this.COLUMNS[profile];
        }
    },	

    getColumnIndex: function(name) {
        for (var i = 0; i < this.columnProfile.length; i++) {
            if (this.columnProfile[i].name == name) {
                return i;
            }
        }
        return -1;
    },	
			
    // Summary:
    //
    getNumColumns: function(content)
    {
        return this.columnProfile.length;
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
        return this.columnProfile[index].name;
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {  
        return null;
    },
	 
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return this.columnProfile[atColumnIndex].alignment;
    },   
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
        return this.columnProfile[atColumnIndex].groupable;     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return this.columnProfile[atColumnIndex].name != this.COLUMNS.TYPE.name;     
    },
    
    getColumnWidth: function(content, atColumnIndex)
    {
        var column = this.columnProfile[atColumnIndex];
        if (this.columnProfileName == "ALL") {
            return column.width;
        }
		//TODO - refactor into base class or create profile class
        //if using a smaller column profile, adjust widths
        var total = 0;
        for (var colName in this.columnProfile) {
            total += this.columnProfile[colName].width;
        }
        //split the difference between all columns proportionately
        var rowBuffer = 100 - total;   //percentage of width unused
        var columnBuffer = rowBuffer * (column.width / total);  //proportionately redistrubute
        return (column.width + Math.round(columnBuffer));  
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        // our content is already an array of objects
        return content;
    },
    
    getRowDataId: function(rowData)
    {
        return "_table_row_for_testScript_" + rowData.id;
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        var type = this.columnProfile[atColumnIndex].type;
        // link is disabled as text label if the test script is archived
        if (rowData.archived && (type == TableConstants.RENDER_TYPE_LINK)) {
            return TableConstants.RENDER_TYPE_TEXT;
        }
        return type;
    },

    getCellAlignment: function(rowData, atColumnIndex)
    {
        return this.columnProfile[atColumnIndex].alignment;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {
        var val = "data";
        var colName = this.columnProfile[atColumnIndex].name;
        switch(colName) {
            case this.COLUMNS.TYPE.name: 
                val = "";
                break;
            case this.COLUMNS.ID.name: 
                val = rowData.id;
                break;
            case this.COLUMNS.NAME.name: 
                val = rowData.name;
                break;
            case this.COLUMNS.STATE.name: 
                val = rowData.archived ? Constants.getMessage("labelDeleted") : EditorUtil.getStateNameFromID(rowData.state);             
                break;				
            case this.COLUMNS.SCRIPT_TYPE.name:
                if (rowData.scriptType) {
					val = Constants.getExtensionDisplayMessage(rowData.scriptType.name);
				} else if (rowData.scriptTypeName) {
					val = Constants.getExtensionDisplayMessage(rowData.scriptTypeName);
				} else {
					val = Constants.getMessage("labelUnknown");
				}                         
                break;
            case this.COLUMNS.MODIFIED.name: 
                val = new Date();
                val.setTime(rowData.modified);
                break;   
            default:
                val = "";
                break;            
        }
        return val;		
    },

    getCellRawData: function(rowData, atColumnIndex)
    {
        var val = "";
        var rowName = this.columnProfile[atColumnIndex].rowName;
        
        if(rowName) {
            return (rowData[rowName] || val);
        }
        
        var colName = this.columnProfile[atColumnIndex].name;
        if(colName == this.COLUMNS.SCRIPT_TYPE.name) {
            if (rowData.scriptType) {
                val = Constants.getExtensionDisplayMessage(rowData.scriptType.name);
            } else if (rowData.scriptTypeName) {
                val = Constants.getExtensionDisplayMessage(rowData.scriptTypeName);
            } else {
                val = Constants.getMessage("labelUnknown");
            }                         
        }
        return val;
    },
        
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = {};
        var colName = this.columnProfile[atColumnIndex].name;
        switch(colName) {
            case this.COLUMNS.TYPE.name: 
                val.image = ImageCache.TEST_SCRIPT;
				val.tooltip = rowData.archived ? Constants.getMessage("labelTSDeleted") : Constants.getMessage("labelClickToOpenTestScript");
                if (!rowData.archived) {
                    if (rowData.newItem && this.newItemLinkCallback) {
                        val.onclick = dojo.hitch(this, function(){
                            this.newItemLinkCallback(rowData.participantId);
                        });
                    } else {
                        val.href = Utility.getTestScriptURI(rowData.id);
                    }
                }
                break;
            case this.COLUMNS.ID.name:
                if (!rowData.archived) {
                    val.linkType = Constants.getMessage("TestScriptID");
                }
            case this.COLUMNS.NAME.name: 
                if (!rowData.archived) {
                    if (rowData.newItem && this.newItemLinkCallback) {
                        val.onclick = dojo.hitch(this, function(){
                            this.newItemLinkCallback(rowData.participantId);
                        });
                    } else {
                        val.href = Utility.getTestScriptURI(rowData.id);
                    }
                }
                break;
            default:
                break;			        
        }
        return val; 		
    }
    
    
});

dojo.declare("TestScriptTableView.TableFilterProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider,
{
    constructor: function(tableView, openSection) {
        this.tableView = tableView;
        this.initOpenFilter = openSection;

        // define the static filters
        this.filters = [
            {
                name: "myItemsOnly",  // must match key in the arg list of the API
                title: Constants.getMessage("labelOnlyMyTestScripts"),
                promptTitle: Constants.getMessage("labelOnlyMyTestScripts"),
                type: TableContentFilterProvider.FIELD_CHECKBOX,
                defaultValue: true, 
                params: null
            }, {
                name: "name",  // must match key in the arg list of the API
                title: Constants.getMessage("labelname"),
                promptTitle: Constants.getMessage("labelName"),
                type: TableContentFilterProvider.FIELD_TEXT,
                defaultValue: null, 
                params: null
            }
        ];
    },

    getTitle: function() {
        return Constants.getMessage("labelFilterTitle");    
    },
    
    getIsSectionOpenInitially: function() {
       return this.initOpenFilter; 
    },
    
    getSummaryText: function() {
        return Constants.getMessage("labelFilterDescriptionAllVisible");
    },
    
    getInitialVisibleFilterCount: function() {
        return 5;    
    },
    
    getFilterCount: function() {
        return this.filters.length;
    },
    
    getFilterName: function(index) {
        return this.filters[index].name;
    },
    
    getFilterTitle: function(index) {
        return this.filters[index].title;
    },
    
    getFilterPromptTitle: function(index) {
        return this.filters[index].promptTitle;        
    },
    
    getFilterType: function(index) {
        return this.filters[index].type;
    },
    
    getFilterDefaultValue: function(index) {
        return this.filters[index].defaultValue;
    },
    
    getFilterParams: function(index) {        
        return this.filters[index].params;
    },
    
    getCustomField: function(index) {
        return null;
    },
    
    getActionsInfo: function() {
        var actions = [
            {
                id: "cancel",
                title: Constants.getMessage("labelReset"),
                callback: dojo.hitch(this, "filterResetActionInvoked"),
                callbackData: null,
                isDefault: false
            }, {
                id: "run",
                title: Constants.getMessage("labelRun"),
                callback: dojo.hitch(this, "filterRunActionInvoked"),
                callbackData: null,
                isDefault: true
            }
        ];
        return actions;
    },
    
    filterResetActionInvoked: function(actionName, filters) {
        this.tableView.tableViewer.setContentFilterAttributeProvider(this.tableView.filterProvider);
    },

    filterRunActionInvoked: function(actionName, filters){
        var args = {};
        
        for (var nameKey in filters) {
            if (filters[nameKey] != null) {
                args[nameKey] = filters[nameKey] || "";
                if (args[nameKey] == 'Any@@') {
                    args[nameKey] = '';
                }
            }
        }
        this.tableView.runQuery(args);
    }
});


dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptTableView", 
    [dijit._Widget, dijit._Templated, AbstractFilteredTableView],
{    
    templateString: '<div class="com-ibm-rqm-test-artifact-table-view" tabindex="-1">' +
                    '  <div dojoType="com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout" "noline"=true dojoAttachPoint="viewHeaderNode">' +
                    '    <div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1"></div>' +
                    '  </div>' +
				    '</div>',

    editable: true,
	printable: false,
    widgetsInTemplate: true,

    constructor: function() {
	 	this.scripts = null;
		this.newItemLinkCallback = null;
		this.rowCheckedListener = null;
        this.showInitLoadProgress = false;
        this.showQueryFilters = false;
        this.filterArgs = null;
		this.initOpenFilter = true;
	 },

    postCreate: function() 
    {
        this.inherited("postCreate", arguments);

        // the view is in a query page if there is a query name otherwise it is within
        // some container that don't need the header
        this.isQueryPage = this.queryName;

        // apply the user preference whether to initially open the 
        // filter area
        if (this.showQueryFilters) {
            var context = {
                self: this,
                prefCallback: function(userPref) {
                    this.self.initOpenFilter = (userPref && typeof userPref['value'] != "undefined") ? userPref.value : true;
                }
            }
            if(this.isQueryPage)
				PreferenceClient.getCurrentUserPreferences(context, context.prefCallback, Constants.USER_UI_CUSTOMISE_QUERYLISTS_FILTER_INITOPEN_CATEGORY.key);
			this._initScriptTypes();	         
        }
        else {
           this._initialize();
        }
		this.setSearchFunction(dojo.hitch(TestScriptClient, "searchTestScripts"));
		this.setSearchCallback(dojo.hitch(this, "show"));
		this.setTypeLabel(Constants.getMessage("labelScript"));
        if (this.postQueryFilterCallback) {
            this.setSearchFilterCallback(this.postQueryFilterCallback);
        }		
    },
    
    _initialize: function() 
    {
        //create table viewer       
        var isMinimal = this.profile && this.profile != Constants.TABLE_PROFILE_ALL;
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            showPagingControl: !this.printable,
			showPagingRangeControl: !this.printable,
            showGroupFilterControls: !this.printable,
            showPagingActionsControls: (!isMinimal && !this.printable),		
            allowAdjustSettings: !isMinimal,
            autoSaveSettings: !isMinimal                               
        }); 		
		      
        this._tableViewerNode.appendChild(tableViewer.domNode);
		          
        if (this.showQueryFilters) {
            tableViewer.setContentFilterAttributeProvider(this.filterProvider);
            
            // we're not showing any content yet, disable the no item message
            tableViewer.noItemMessage = " ";
        }

        var content = this.scripts ? this.scripts : [];
        
        if (this.isQueryPage) {
            // it is a query page so set header title and add table viewer as the header content
            this.viewHeaderNode.setTitle(this.queryName);
            this.viewHeaderNode.setCSHelpId("rqm_view_test_scripts.html");        
            this._tableViewerNode.appendChild(tableViewer.domNode);
        } else {
            // it is not a query page i.e. within a section without header
            this.viewHeaderNode.domNode.style.display = "none";
            this.domNode.appendChild(tableViewer.domNode);
        }
        // setup the table's id for saving table settings
        if(!isMinimal) {
            tableViewer.tableId = (this.isQueryPage) ? "RQM-TS-QUERY-TABLEVIWER" : "RQM-TS-TABLEVIWER";
        }
        tableViewer.setContentPageSize(20);
        tableViewer.setContent(content);
		tableViewer.setTableRowsCheckable(this.editable && !this.printable);
        this.provider = new TestScriptTableView.TableProvider({
            profile: this.profile ? this.profile : Constants.TABLE_PROFILE_ALL,
			newItemLinkCallback: this.newItemLinkCallback      
        }); 		
	
        tableViewer.setContentProvider(this.provider);
        tableViewer.setTableListener(new TestScriptTableView.TableListener(this));
        tableViewer.setCellModifier(new TestScriptTableView.CellModifier());       
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("labelTestScripts"))); 
        if (this.enableRowSelection) {
            tableViewer.setTableRowsCheckable(this.editable && !this.printable, /* isSingleSelect */ false);
        }        
        if (this.refreshCallback && dojo.isFunction(this.refreshCallback)) {
            this.refreshAction = tableViewer.addAction("Refresh", Constants.getMessage("labelRefreshTestScripts"), this, "refresh-icon-image", this.refreshCallback);
        }
		
        for (var i = 0; i < (this.actions ? this.actions.length : 0); i++) {
            var action = this.actions[i];
            if (!(this.readOnly && action.disableOnReadOnly)) {
                if (!this.isQueryPage && action.label == Constants.getMessage("labelRemoveTestScripts")) {//If not querypage remove action will be sent
					this.deleteActionNode = tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
					this.deleteActionNode.setDisabled(true);
				}
				else if (action.label == Constants.getMessage("labelChangeTestScriptProperties")) {
					this.bulkchangeActionNode = tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
					this.bulkchangeActionNode.setDisabled(true);
				}
				else {
					tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
				}	
            }
        }  		
		
        var permission = PermissionsClient.getStoredPermission(PermissionsClient.OP_TEST_SCRIPT);
		if (this.isQueryPage) {
            if (!permission || permission[PermissionsClient.ACTION_ARCHIVE]) {
                this.deleteActionNode = tableViewer.addAction(
                    "delete",                                  // action id
                    Constants.getMessage("labelDelete"),       // label
                    null,                                      // callback data
                    "delete-icon-image",                       // icon class
                    dojo.hitch(this, "_onDeleteActionClick")); // callback
                this.deleteActionNode.setDisabled(true);
            }       
        }
              		  
		this.tableViewer = tableViewer;
        if(this.showInitLoadProgress) {
            tableViewer.showLoadingProgress();
        }
        this.tableViewer.show();
        this.tableViewer.setEditable(this.editable);	
			  
        // apply the user preference whether to run my query immediately.
        if (this.isQueryPage && this.showQueryFilters) {
            PreferenceClient.getCurrentUserPreferences(this, this.initRunQuery, Constants.USER_UI_CUSTOMISE_QUERYLISTS_AUTORUN_MYQUERY_CATEGORY.key);
        }   
    },

    // Constructs the query filter definitions and the content filter provider for the
    // table. The table will construct the filters section based on the definitions.
    // 
    _initializeFilters: function(scriptTypes) {
        // create the filter provider for the table
        this.filterProvider = new TestScriptTableView.TableFilterProvider(this, this.initOpenFilter);

        this.moreFilters = [];

        this.moreFilters.push({
            name: "stateIds", // must match key in the arg list of the API
            title: Constants.getMessage("labelStateLabel"),
            promptTitle: Constants.getMessage("labelState"),
            type: TableContentFilterProvider.FIELD_TOGGLELIST,
            defaultValue: [],
            params: {options: [{
                    name: Constants.getMessage("com.ibm.rqm.planning.common.new"),
                    value: Constants.STATETRANSITION_STATE_DRAFT,
                    checked: false
                }, {
                    name: Constants.getMessage("com.ibm.rqm.planning.common.underreview"),
                    value: Constants.STATETRANSITION_STATE_UNDERREVIEW,
                    checked: false
                }, {
                    name: Constants.getMessage("com.ibm.rqm.planning.common.approved"),
                    value: Constants.STATETRANSITION_STATE_APPROVED,
                    checked: false
                }, {
                    name: Constants.getMessage("com.ibm.rqm.planning.common.retired"),
                    value: Constants.STATETRANSITION_STATE_RETIRED,
                    checked: false
                }
            ]}
        });

        if (scriptTypes != null && scriptTypes.length > 0) {
            var types = [];
            
            for(var i = 0; i < scriptTypes.length; i++) {
                if (scriptTypes[i].name == "Unknown" || scriptTypes[i].name == "NoScript") { continue; }
                types.push({
                    name: scriptTypes[i].displayName,
                    value: scriptTypes[i].id,
                    checked: false
                });
            }

            this.moreFilters.push({
                name: "scriptTypeIds",  // must match key in the arg list of the API
                title: Constants.getMessage("labelScriptType") + Constants.getMessage("labelColen"),
                promptTitle: Constants.getMessage("labelScriptType"),
                type: TableContentFilterProvider.FIELD_TOGGLELIST,
                defaultValue: null,
                params: {'options': types}
            });
        }           

        // initlize the orginator filter with the list of contributors
        var self = this;
        var serviceObj = {
            success: function(contributors) {
                for (var i = 0; i < contributors? contributors.length : 0; i++) {
                    if (contributors[i].name == "Unassigned") {
                        contributors[i].name = Constants.getMessage("labelUnassigned");
                    }
                    contributors[i].value = contributors[i].itemId;
                    contributors[i].checked = false;
                }
                
                var originators = {
                    name: "originatorUUIDs",  // must match key in the arg list of the API
                    title: Constants.getMessage("labeloriginatorLabel"),
                    promptTitle: Constants.getMessage("labelOriginator"),
                    type: TableContentFilterProvider.FIELD_TOGGLELIST,
                    defaultValue: [],
                    params: (contributors == null) ? null : {options: contributors}
                };
                self.filterProvider.filters.push(originators);
                // add the category filters to the end if there is any
                for (var j = 0; j < self.moreFilters.length; j++) {
                    self.filterProvider.filters.push(self.moreFilters[j]);
                }
                self._initialize();
            },
            
            failure: function(e) {
                self._error(e);
                // add the category filters to the end if there is any even when
                // it failed to initialize the originator filter
                for (var j = 0; j < self.moreFilters.length; j++) {
                    self.filterProvider.filters.push(self.moreFilters[j]);
                }
                self._initialize();
            }
        };
        var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");
        UserClient.getContributors(responseHandler);
    },
    
    _initScriptTypes: function() 
    {
        var self = this;
        var serviceObj = {
            success: function(scriptTypes) {
                self._sortScriptTypes(scriptTypes);
                self._initializeFilters(scriptTypes);
            },
            failure: function(e) {
                self._error(e);
                self._initializeFilters();
            }
        };
        var handler = new ServiceResponseHandler(serviceObj, "success", "failure");
        RegistryClient.getAllExecutionScriptTypes(handler);         
    },
    
    _sortScriptTypes: function(types) 
    {
        for (var i = 0; i < types.length; i++) {
            if (types[i].name == "Unknown" || types[i].name == "NoScript") { continue; }
            types[i].displayName = (types[i].name == DEFAULT_SCRIPT_TYPE) ? Constants.getMessage("labelManual") : Constants.getExtensionDisplayMessage(types[i].name);
        }
        
        for(var ix = 0; ix < types.length; ix++ ) {
            for(var iy = ix + 1; iy < types.length; iy++ ) {
                if(types[iy].displayName < types[ix].displayName) {
                    var swap = types[ix];
                    types[ix] = types[iy];
                    types[iy] = swap;
                }
            }
        }
    },

	_onDeleteActionClick: function() {
        // check if any selected test cases are current displayed in another tabs
        var openedArtifacts = [];
        this.selectedScripts = [];
        if (this.artifactsDisplayContainer) {
            var userSelectedScripts = this.tableViewer.getCheckedRows();
            
            if (userSelectedScripts.length > 0) {
                for (var i = 0; i < userSelectedScripts.length; i++) {
                    var scriptURI = Utility.getTestScriptURI(userSelectedScripts[i].id);
                    if (this.artifactsDisplayContainer.hasPage(scriptURI)) {
                        openedArtifacts.push(userSelectedScripts[i]);
                    } else {
                        this.selectedScripts.push(userSelectedScripts[i]);
                    }
                }
            }
        } else {
            this.selectedScripts = this.tableViewer.getCheckedRows();
        }
        
        var prompt = "";
        if (this.selectedScripts.length > 1) {
            prompt = Constants.getMessage("msgRemoveItem2", this.selectedScripts.length, Constants.getMessage("labelScript"));
        } else if (this.selectedScripts.length == 1) {
            prompt = Constants.getMessage("msgRemoveItem", Constants.getMessage("labelScript"), this.selectedScripts[0].name);
        }
        if (openedArtifacts.length > 0) {
            if (this.selectedScripts.length > 0) {
                // some selected artifacts are not opened in tabs
                prompt = Constants.getMessage("msgDelSomeOpenTestScriptConfirm");
            } else {
                // all selected artifacts are opened in tabs
                prompt = Constants.getMessage("msgDelAllOpenTestScriptConfirm");
            }
        }

        var args = {
            'message': prompt,
            'options': openedArtifacts,
            'okCallback': 
                function(view) {
                    view._deleteAction();
                },
            'okCallbackData': this,
            'cancelCallback': null,
            'cancelCallbackData': null
        };
        this.confirmDialog = new OptionsDialog(args);
        this.confirmDialog.setTitle(Constants.getMessage("labelConfirmation"));
        this.confirmDialog.open();
    },

    _deleteAction: function() {         
        this.selectedScripts = this.selectedScripts.concat(this.confirmDialog.getSelection());
        this.selectedScriptCount = this.selectedScripts.length;

        if (this.selectedScriptCount > 0) {
            this.deleteActionNode.setDisabled(true);
            this.errorScriptCount = 0;
            this.deletedScriptCount = 0;

            for (var i = 0; i < this.selectedScriptCount; i++) {
				//TODO: here we need to check if manual script or remote script
                var responseHandler = new ServiceResponseHandler(this, "_deleteSuccess", "_deleteFailure");
                var args = {
                            uuid_script: this.selectedScripts[i].itemId,
                            archive: true,
							stateId: this.selectedScripts[i].stateId
                        }
				var scriptTypeId = (this.selectedScripts[i].scriptTypeId) ? this.selectedScripts[i].scriptTypeId :
				    (this.selectedScripts[i].scriptType) ? this.selectedScripts[i].scriptType.id : null;
				if (scriptTypeId == "com.ibm.rqm.planning.common.scripttype.manual")
                    TestScriptClient.archiveTestScript(responseHandler, args);
                else
                    com.ibm.rqm.execution.web.client.internal.execution.RemoteExecutionScriptClient.archiveTestScript(responseHandler, args);
                
                if (this.isQueryPage)
                    this.viewHeaderNode.setProgressStart(Constants.getMessage("msgDeletingTestScript", this.selectedScripts[i].name));
            }
        }
    },
	
    _deleteSuccess: function(response) {
        if (this.isQueryPage)
            this.viewHeaderNode.setProgressEnd();
        this.deletedScriptCount++;
        this._setDeleteStatus();

    },
    
    _deleteFailure: function(err) {
        if (this.isQueryPage)
            this.viewHeaderNode.setProgressEnd();
        this.errorScriptCount++;
        this._setDeleteStatus();
    },
    
    _setDeleteStatus: function() {
        if ((this.deletedScriptCount + this.errorScriptCount) == this.selectedScriptCount) {
            if (this.isQueryPage)
                this.viewHeaderNode.setStatus(
                    Constants.getMessage("msgDeletedTestScript", this.deletedScriptCount, this.selectedScriptCount),
                    this.errorScriptCount == 0 ? ViewHeaderLayout.SUCCESS : this.selectedScriptCount == 1 ? ViewHeaderLayout.ERROR : ViewHeaderLayout.INFO
                );
            if (this.deletedScriptCount > 0) {
                // notify all pages (including itself) test cases are deleted
                dojo.publish(GlobalEvents.ArtifactsDeleted, [Constants.TYPE_SCRIPT, this.selectedScripts]);
            }
        }
    },
	
     // API
     getSelectedItems: function() {
         return this.tableViewer.getCheckedRows();
     },
     
     refresh: function(content, skipClearStatus) {
         this.scripts = content;
         var cancelProgress = this.showInitLoadProgress;
        
         // we don't create the table until we fetched the categories.
         // The table may not have existed at this point. The real data
         // has arrived. Always reset the init load progress flag. 
         if(this.showInitLoadProgress) {
             this.showInitLoadProgress = false;
         }
         if (this.tableViewer) {
            if(cancelProgress) {
                this.tableViewer.endLoadingProgress();
            }
		 	this.tableViewer.setContent(content);
		 	this.tableViewer.refresh();
		 }
         if (this.viewHeaderNode && !skipClearStatus) {
             this.viewHeaderNode.clearStatus();
         }
     },
	 
	 show: function(scripts) {
        this.refresh(scripts || []);
        this.stopInProgress();
        if (self.queryFiltersCallback) {
            self.queryFiltersCallback();
        }	 	
	 },
	 
	 // handler on artifacts changed
     onArtifactChanged: function(changeType, artifactType, artifactArray) {
        // only care about test scripts
        if (artifactType == Constants.TYPE_SCRIPT) {
            if (changeType == GlobalEvents.ArtifactsAdded) {
                // add test scripts to the table
                if (!this.scripts) {
                    this.scripts = artifactArray;
                } else {
                    this.scripts = this.scripts.concat(artifactArray);
                }
                this.refresh(this.scripts, true);
            } else {
                // iterate through the array of artifacts
                for (var i = 0; i < artifactArray.length; i++) {
                    // iterate through the array of test scripts in the table
                    for (var j = 0; j < this.scripts.length; j++) {
                        // handle the matching test script
                        if (artifactArray[i].id == this.scripts[j].id) {
                            if (changeType == GlobalEvents.ArtifactsUpdated) {
                                // swap test script on update
                                this.scripts.splice(j, 1, artifactArray[i]);
                                this.refresh(this.scripts, true);
                            } else if (changeType == GlobalEvents.ArtifactsDeleted) {
                                // remove test script from table on delete
                                this.scripts.splice(j, 1);
                                this.refresh(this.scripts, true);
                            }
                            break;
                        }
                    }
                }
            }
        }
    },
	
	setTableRowsCheckable: function(checkable, isSingleSelect){
		this.tableViewer.setTableRowsCheckable(this.editable && checkable, isSingleSelect);
	},
    
    _error: function(e) {
        console.error(this.declaredClass + ": Error: " + e.message);
    }
          
});

})();
