/*****************************CollectionsQueryView***************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//
//
// Author: P. Chu
//
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.collections.CollectionsQueryView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.SimpleClearCollapsibleArea");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");

dojo.require("com.ibm.rational.test.lm.web.client.internal.collections");
dojo.require("com.ibm.rational.test.lm.web.client.internal.inventory");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationsUriUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.configurations.AssetDescriptionSelectorDialog");

(function() {

var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var AssetUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil;
var ConfigurationUriUtil = com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationsUriUtil;
var LOADING = RtlmUiUtil.getMessage("appWorking");
var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.collections.CollectionsQueryView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div class="collections-query-view" dojoAttachPoint="domNode" tabindex="-1">' +
                    '  <div dojoType="com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout" dojoAttachPoint="viewHeaderNode" noline="true">' +
                    '    <div class="collectionss-area-container">' +
                    '      <div dojoAttachPoint="collectionsArea" class="collectionss-area"></div>' +
                    '    </div>' +
                    '  </div>' +
                    '</div>',

    widgetsInTemplate: true,
    
    refreshActionNode: null,

    collectionsService: null,
    inventoryService: null,
    deployActionNode: null,
    undeployActionNode: null,
    assignConfigActionNode: null,
    
    // the virtual image collections displayed in this view
    vmiCollections: null,
    
    // the virtual machine collections displayed in this view
    vmCollections: null,
    
    // flag to track which type of collection is displayed, default is virtual image
    showVMI: true,

    // the table to display the collections
    tableViewer: null,
    
    postCreate: function() {
        // set title and subtitle
        this.inherited("postCreate", arguments);
        this.viewHeaderNode.setTitle(RtlmUiUtil.getMessage("labelAllVirtualCollections"));
        this.viewHeaderNode.setCSHelpId("rtlm_allvirtcoll.html", com.ibm.rational.test.lm.web.CSHelpID);		
		this.viewHeaderNode.setCSHelpButtonLabel(RtlmUiUtil.getMessage("labelVirtualCollectionsHelp"));

        this.collectionsService = new com.ibm.rational.test.lm.web.client.internal.collections();
        this.inventoryService = new com.ibm.rational.test.lm.web.client.internal.inventory();
        
        // create the table to display the collections
        this.tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            allowAdjustSettings: true,
            autoSaveSettings: true,
            tableId: "RQM-LM-RES-COLLECTION-TABLEVIEWER"                                
        });
        this.tableViewer.setContentProvider(new Collections.TableProvider());
        this.tableViewer.setContentPageSize(15);
        this.tableViewer.setTableRowsCheckable(true);
        this.tableViewer.setTableListener(new Collections.TableListener(this));
		this.tableViewer.setTableSummary(RtlmUiUtil.getMessage("labelTableSummary", RtlmUiUtil.getMessage("labelAllVirtualCollections")));

        // set the custom row renderer to render as list
        this.tableViewer.setCustomRowRenderer(new Collections.ListRowRenderer());
        this.tableViewer.rowRenderer.container = this;
        // setting to list mode will remove the column header
//      this.tableViewer.setTableMajorMode(TableConstants.MAJOR_MODE_LIST);

        this.tableViewer.setContent([]);
        this.tableViewer.show();
        
        var query = [
            {'name': RtlmUiUtil.getMessage("labelAllVirtualImages"), 'value': 0},
            {'name': RtlmUiUtil.getMessage("labelAllVirtualMachines"), 'value': 1}
        ];
        this.contentSelector = this.tableViewer.setTableContentSelectors(query);
        
        this.refreshActionNode = this.tableViewer.addAction(
                "refresh",                     // action id
                RtlmUiUtil.getMessage("labelRefreshButton"),  // label
                null,                          // callback data
                "refresh-icon-image",          // icon class
                dojo.hitch(this, "refresh"));  // callback

        this.deployActionNode = this.tableViewer.addAction(
                "deploy",                        // action id
                RtlmUiUtil.getMessage("Deploy"), // label
                null,                            // callback data
                "deploy-collection-icon-image", // icon class
                dojo.hitch(this, "deploy"));     // callback

        this.undeployActionNode = this.tableViewer.addAction(
                "undeploy",                        // action id
                RtlmUiUtil.getMessage("Undeploy"), // label
                null,                              // callback data
                "undeploy-collection-icon-image",  // icon class
                dojo.hitch(this, "undeploy"));     // callback

        this.assignConfigActionNode = this.tableViewer.addAction(
                "assignConfig",                        // action id
                RtlmUiUtil.getMessage("labelAssignConfig"), // label
                null,                                  // callback data
                "assign-collection-config-icon-image", // icon class
                dojo.hitch(this, "assignConfig"));     // callback

        this._updateActionsDisplay();
        this.collectionsArea.appendChild(this.tableViewer.domNode);
        
        // load collections into table
        this._loadCollections();
    },
    	
    _loadCollections: function() {
		var type = dojo.getObject( "com.ibm.rational.test.lm.web.collection-type" );
		if( type && type == "MachineCollection" )
		{
			this.contentSelector.selectedIndex = 1;
			this.showVMI = false;
		}					

        this.collections = null;
        var responseHandler = new ServiceResponseHandler(this, "_loadSuccess", "_loadFailure");
        this.viewHeaderNode.setProgressStart(LOADING);
        if (this.showVMI) {
            this.collectionsService.getVirtualImageCollections(responseHandler);
        } else {
            this.collectionsService.getVirtualMachineCollections(responseHandler);
        }
    },
    
    _loadSuccess: function(response) {
        this.viewHeaderNode.setProgressEnd();
        if (this.showVMI) {
            this.vmiCollections = response ? response : [];
        } else {
            this.vmCollections = response ? response : [];
        }
        this.tableViewer.setContent(response ? response : []);
        this.tableViewer.refresh();
        this._updateActionsDisplay();
        this.viewHeaderNode.clearStatus();
		var id = dojo.getObject( "com.ibm.rational.test.lm.web.collection-id" );

		if( id )
		{
			for( var iWidget = 0; this.listrowwidgets && ( iWidget < this.listrowwidgets.length ); iWidget++ )
			{
				if( this.listrowwidgets[ iWidget ][ "id" ] == id )
				{
					if( !this.listrowwidgets[ iWidget ][ "widget" ].open )
						this.listrowwidgets[ iWidget ][ "widget" ].toggle();
				}
			}
		}
		
		dojo.setObject( "com.ibm.rational.test.lm.web.collection-id", null );
		dojo.setObject( "com.ibm.rational.test.lm.web.collection-type", null );							
    },
    
    _loadFailure: function(response) {
        this.viewHeaderNode.setProgressEnd();
        this.viewHeaderNode.setStatus(RtlmUiUtil.getMessage("errLoadCollectionsInfo"), ViewHeaderLayout.ERROR);
		dojo.setObject( "com.ibm.rational.test.lm.web.collection-id", null );
		dojo.setObject( "com.ibm.rational.test.lm.web.collection-type", null );							
    },
    
    _displayCollections: function() {
        // the collection is default to null and set to an empty array if server return empty
        if (this.showVMI) {
            if (this.vmiCollections == null) {
                // virtual image collection is not retrieved yet, query the server
                this._loadCollections();
            } else {
                this.tableViewer.setContent(this.vmiCollections);
                this.tableViewer.refresh();
                this._updateActionsDisplay();
            }
        } else {
            if (this.vmCollections == null) {
                // virtual machine collection is not retrieved yet, query the server
                this._loadCollections();
            } else {
                this.tableViewer.setContent(this.vmCollections);
                this.tableViewer.refresh();
                this._updateActionsDisplay();
            }
        }
    },

    _hasDeployPermission: function()
    {
        var perm = PermissionsClient.getStoredPermission(PermissionsClient.OP_LM_VIRTUALIZATION);  
        return (!perm || perm[PermissionsClient.ACTION_DEPLOY]);   
    },

    _hasCollectionPermission: function()
    {
        var perm = PermissionsClient.getStoredPermission(PermissionsClient.OP_LM_COLLECTION);  
        return (!perm || perm[PermissionsClient.ACTION_EDIT]);   
    },

    _updateActionsDisplay: function() {
        var hasNoRows = (this.tableViewer.getCheckedRows().length == 0);
		var disableDeploy = (hasNoRows || !this._hasDeployPermission());
		var disableEdit = (hasNoRows || !this._hasCollectionPermission());
		
        this.deployActionNode.setDisabled(disableDeploy);
        this.deployActionNode.domNode.style.display = this.showVMI ? "" : "none";
        this.undeployActionNode.setDisabled(disableDeploy);
        this.undeployActionNode.domNode.style.display = this.showVMI ? "none" : "";
        this.assignConfigActionNode.setDisabled(this.tableViewer.getCheckedRows().length != 1 || disableEdit);
        this.assignConfigActionNode.domNode.style.display = this.showVMI ? "" : "none";
    },
    
    refresh: function() {
        this._loadCollections();
    },
    
    deploy: function() {
        var selectedCollections = this.tableViewer.getCheckedRows();
        this.deployActionNode.setDisabled(true);
        this.viewHeaderNode.setProgressStart(RtlmUiUtil.getMessage("labelDeployingVICollection"));
        
        this.selectedCollectionsCount = selectedCollections.length;
        this.successCollectionsCount = 0;
        this.errorCollectionsCount = 0;

        for (var i = 0; i < selectedCollections.length; i++) {
            var responseHandler = new ServiceResponseHandler(this, "_deployUndeploySuccess", "_deployUndeployFailure");
            this.inventoryService.deployVirtualCollection(responseHandler, selectedCollections[i]['id']);
        }
    },
    
    undeploy: function() {
        var selectedCollections = this.tableViewer.getCheckedRows();
        this.undeployActionNode.setDisabled(true);
        this.viewHeaderNode.setProgressStart(RtlmUiUtil.getMessage("labelUndeployingVMCollection"));
        
        this.selectedCollectionsCount = selectedCollections.length;
        this.successCollectionsCount = 0;
        this.errorCollectionsCount = 0;

        for (var i = 0; i < selectedCollections.length; i++) {
            var responseHandler = new ServiceResponseHandler(this, "_deployUndeploySuccess", "_deployUndeployFailure");
            this.inventoryService.undeployVirtualCollection(responseHandler, selectedCollections[i]['id']);
        }
    },

    _deployUndeploySuccess: function (successResponse) {
		if(successResponse && (successResponse.rc == "SPIRC_NO_ERROR")) {
			this.successCollectionsCount++;
		} else {
			this.errorCollectionsCount++;
		}
        this._setDeployUndeployStatus();
    },
    
    _deployUndeployFailure: function (failureResponse) {
        this.errorCollectionsCount++;
        console.debug(failureResponse['message']);
        this._setDeployUndeployStatus();
    },

    _setDeployUndeployStatus: function() {
        if ((this.successCollectionsCount + this.errorCollectionsCount) == this.selectedCollectionsCount) {
            var messageKey = this.showVMI ? "msgDeployedVICollections" : "msgUndeployedVMCollections";
            this.viewHeaderNode.setStatus(
                RtlmUiUtil.getMessage(messageKey, this.successCollectionsCount, this.selectedCollectionsCount),
                this.errorCollectionsCount == 0 ? ViewHeaderLayout.SUCCESS :
                    this.errorCollectionsCount == this.selectedCollectionsCount ? ViewHeaderLayout.ERROR : ViewHeaderLayout.INFO
            );
            this.viewHeaderNode.setProgressEnd();
            if (this.showVMI) {
                this.deployActionNode.setDisabled(false);
            } else {
                this.undeployActionNode.setDisabled(false);
            }
            
            // bring up the deployment page, seed it with deploy directive.
            dojo.setObject( "com.ibm.rational.test.lm.web.deployCollection", "true" );
            window.location.href = AssetUriUtil.getDeployViewURI();
        }
    },

    assignConfig: function() {
        var selectedCollections = this.tableViewer.getCheckedRows();
        this.assignConfigActionNode.setDisabled(true);
        if (selectedCollections.length == 1) {
            this._chooseLRD();
        }
    },

    _chooseLRD : function() {
        var configs = new com.ibm.rational.test.lm.web.client.internal.config();
        var responseHandler = new ServiceResponseHandler(this, "_loadAllConfigsSuccessHandler", "_loadAllConfigsErrorHandler");
    
        configs.getAll(responseHandler); 
    },
    
    _loadAllConfigsSuccessHandler :function(response) {
        this.selectDlg = new com.ibm.rational.test.lm.web.ui.internal.views.configurations.AssetDescriptionSelectorDialog({title:RtlmUiUtil.getMessage("srchSelectResouceDesc")});
        this.selectDlg.setOKAction( dojo.hitch( this, this._handleSelectOK ), RtlmUiUtil.getMessage("srchTestDataNotNeeded") );
        this.selectDlg.setCancelAction( dojo.hitch(this, this._handleSelectCancel));

        if (response) 
        {
            for (var i = 0; i < response.length; i++) 
            {
                var name = response[i].name;
                var id = response[i].id;
                
                this.selectDlg.addListItem(id, name);
            }
        }
        this.selectDlg.show();
    },
    
    _loadAllConfigsErrorHandler: function() {
        this.viewHeaderNode.setProgressEnd();
        this.viewHeaderNode.setStatus(RtlmUiUtil.getMessage("errRetrieveAllConfigs"), ViewHeaderLayout.ERROR);    
    },
    
   _handleSelectOK :function() {
        var selectedPart = this.selectDlg.getSelectedAssetDescriptor();
        this.selectedConfigId = selectedPart ? this.selectDlg.getSelectedConfigID() : null;
        this.selectedConfigName = selectedPart ? this.selectDlg.getSelectedConfigName() : null;

        var responseHandler = new ServiceResponseHandler(this, "_assignConfigSuccess", "_loadAllConfigsErrorHandler");
        this.viewHeaderNode.setProgressStart(RtlmUiUtil.getMessage("msgSettingConfig"));
        this.collectionsService.setConfiguration(responseHandler, this.tableViewer.getCheckedRows()[0].id, this.selectedConfigId);
    },
    
    _handleSelectCancel: function(event) {
        this.assignConfigActionNode.setDisabled(false);
    },

    _assignConfigSuccess: function() {
        this.viewHeaderNode.setProgressEnd();
        if (this.selectedConfigId && this.selectedConfigName) {
            this.tableViewer.getCheckedRows()[0]['configuration'] = {
                id: this.selectedConfigId,
                name: this.selectedConfigName
            };
        } else {
            this.tableViewer.getCheckedRows()[0]['configuration'] = null;
        }
        this.tableViewer.refresh();
        this.selectedConfigId = null;
        this.selectedConfigName = null;
        this.assignConfigActionNode.setDisabled(false);
    },
    
    _assignConfigFailure: function() {
        this.viewHeaderNode.setProgressEnd();
        this.viewHeaderNode.setStatus(RtlmUiUtil.getMessage("errSetCollectionConfig"), ViewHeaderLayout.ERROR);    
        this.selectedConfigId = null;
        this.selectedConfigName = null;
        this.assignConfigActionNode.setDisabled(false);
    },
    
    onArtifactChanged: function(changeType, artifactType, artifactArray){
        // only care about virtual collections, image or machine
        if (artifactType == RtlmUiUtil.TYPE_VMI_COLLECTION) {
            this.vmiCollections = null;
            if (this.showVMI) {
                // update the table display if currently showing the virtual image collections
                this._displayCollections();
            }
        } else if (artifactType == RtlmUiUtil.TYPE_VM_COLLECTION) {
            this.vmCollections = null;
            if (!this.showVMI) {
                // update the table display if currently showing the virtual machine collections
                this._displayCollections();
            }
        }
    }
});

dojo.declare("Collections.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    getNumColumns: function(content) {
        return 3;
    },
    
    getColumnHeading: function(content, atColumnIndex) {
        var headings = [
            RtlmUiUtil.getMessage("labelHeaderCollectionName"),
            RtlmUiUtil.getMessage("labelHeaderDescription"),
            RtlmUiUtil.getMessage("cfgCfgLabel")
        ];
        return headings[atColumnIndex];
    },

    getColumnHeadingImage: function(content, atColumnIndex) {
        return null;
    },
    
    getIsColumnGroupable: function(content, atColumnIndex) {
        return false;
    },

    getIsColumnSortable: function(content, atColumnIndex) {
        return true;
    },
    
    getColumnWidth: function(content, atColumnIndex) {
        var columnWidth = [
            25, // collection name
            40, // description
            35  // configuration
        ];
        return columnWidth[atColumnIndex];
    },
    
    getContentObjects: function(content) {
        // the collections data is already an array
        return content;
    },
    
    getRowDataId: function(rowData) {
        return rowData.id;
    },

    getCellDisplayType: function(rowData, atColumnIndex) {
        var types = [
            TableConstants.RENDER_TYPE_TEXT,   // collection name
            TableConstants.RENDER_TYPE_TEXT,   // description
            TableConstants.RENDER_TYPE_TEXT    // configuration
        ];
        return types[atColumnIndex];
    },
    
    getCellDisplayData: function(rowData, atColumnIndex) {
        switch(atColumnIndex) {
            case 0: // collection name
                return rowData['name'];
            case 1: // description
                return rowData['description'];
            case 2: // configuration
                return rowData['configuration'] ? rowData['configuration']['name'] : RtlmUiUtil.getMessage("cfgUnassigned");
        }
        return "";
    }
});

dojo.declare("Collections.ListRowRenderer",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer,
{    
    renderRow: function(viewer, rowObject, trNode)
    {
        var provider = viewer.getContentProvider();
        var td = trNode.insertCell(trNode.cells.length);
        var listWidget = this._createListWidget(viewer, rowObject);
        
        td.colSpan = provider.getNumColumns();
        td.width = "100%";
        td.appendChild(listWidget.domNode);
		if( this.container )
		{
			if( !this.container.listrowwidgets )
				this.container.listrowwidgets = [];
				
			this.container.listrowwidgets.push( { "id": provider.getRowDataId( rowObject ), "widget": listWidget } );
		}
    },
    
    _createListWidget: function(viewer, rowObject) {
        var provider = viewer.getContentProvider();
        var isOpen = this._getRowDisplayState(provider, rowObject);
        var node = new com.ibm.asq.common.web.ui.internal.view.common.SimpleClearCollapsibleArea();
        
        // create the contents in the list widget.
        this._createListContents(viewer, rowObject, node);
        
        node.data = rowObject;
        node.provider = provider;
        node.onToggle = dojo.hitch(this, "_rowWidgetToggled");
        return node;
    },

    _createListContents: function(viewer, rowObject, listWidget)
    {
        // There is about 50px in each row in the table viewer for the selection check box
        // and the toggle image of the expando icon.  The row renderer will need to offset
        // left in each cell to compensate the shorter in width for the row comparing to
        // the table viewer header.  For 3 columns it will be about 15px left offset
        // starting from column 1.
        //
        var provider = viewer.getContentProvider();
        var isOpen = this._getRowDisplayState(provider, rowObject);

        var headerTable = document.createElement("table");
        headerTable.className = "collection-resources-table";
        var row = headerTable.insertRow(0);

        var nameCol = row.insertCell(0);
        nameCol.width = "25%";
        var nameNode = document.createElement("div");
        nameCol.appendChild(nameNode);
        
        var descCol = row.insertCell(1);
        descCol.width = "40%";
        var descNode = document.createElement("div");
        descNode.style.marginLeft = "15px";
        descCol.appendChild(descNode);

        var configCol = row.insertCell(2);
        configCol.width = "35%";
        var configNode = document.createElement("div");
        configNode.style.marginLeft = "30px";
        configCol.appendChild(configNode);
        
        nameNode.innerHTML = rowObject['name'];
        descNode.innerHTML = rowObject['description'] ? rowObject['description'] : "";
        
        if (rowObject['configuration']) {
            var configLink = document.createElement("a");
            configLink.tabIndex = 0;
            
            configLink.onclick = function(evt) {
                // don't want to expand the row
                dojo.stopEvent(evt);
                window.location.href = ConfigurationUriUtil.getEditConfigurationURI(rowObject['configuration']['id']);
            };
            
            configLink.onkeypress = function(evt) {
                // continue the event on tab
                if (evt.keyCode == dojo.keys.TAB || evt.keyCode == dojo.keys.TAB) {
                    return;
                }
                
                // don't want to expand the row
                dojo.stopEvent(evt);
                
                if (evt.keyCode == dojo.keys.ENTER || evt.keyCode == dojo.keys.SPACE || evt.charCode == dojo.keys.ENTER || evt.charCode == dojo.keys.SPACE) {
                    window.location.href = ConfigurationUriUtil.getEditConfigurationURI(rowObject['configuration']['id']);
                }
            };
            
            configLink.innerHTML = rowObject['configuration']['name'];
            configNode.appendChild(configLink);
        } else {
            configNode.innerHTML = RtlmUiUtil.getMessage("cfgUnassigned");
        }

        listWidget.headerNode.appendChild(headerTable);

        var innerContent = document.createElement("div");
        var labResLabel = document.createElement("div");
        labResLabel.innerHTML = RtlmUiUtil.getMessage("labelListLabResources");
        innerContent.appendChild(labResLabel);
        
        if (rowObject['resources'] && rowObject['resources'].length > 0) {
            var resList = document.createElement("ul");
            resList.className = "collection-resources-table-resources-ul";
            innerContent.appendChild(resList);
            
            for (var i = 0; i < rowObject['resources'].length; i++) {
                var li = document.createElement("li");
                var res = rowObject['resources'][i];
                var resType = res['isVirtualImage'] ? "VirtualImage" : "Machine";

                var resLink = document.createElement("a");
                resLink.href = AssetUriUtil.getViewAssetDetailURI(resType, res['resourceId']);
                resLink.innerHTML = res['resourceName'];

                li.appendChild(resLink);
                resList.appendChild(li);
            }
        } else {
            var noResLabel = document.createElement("div");
            noResLabel.innerHTML = RtlmUiUtil.getMessage("labelNoResinCollection");
            innerContent.appendChild(noResLabel);
        }
        listWidget.containerNode.appendChild(innerContent);
    }
});

dojo.declare("Collections.TableListener",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{    
    _collectionsView: null,
    
    constructor: function(collectionsView) {
        this._collectionsView = collectionsView;
    },
    
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback) {
        callback(callbackData);
    },

    rowCheckStateChanged: function(tableViewer, rowObject, state) {
        this._collectionsView._updateActionsDisplay();
    },

    tableContentSelectionChanged: function(tableViewer, contentSelectionValue) {
        this._collectionsView.showVMI = parseInt(contentSelectionValue) == 0;
        this._collectionsView._displayCollections();
    }
});

})();
