//AssetSelectorDialog
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.machines.AssetSelectorDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.AssetSelectorDialogProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

( function() {
	
	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
	var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;		
	var MachineInfo = com.ibm.rational.test.lm.web.ui.internal.views.providers.AssetSelectorDialogSearchInfo;
	var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	
	
dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.machines.AssetSelectorDialog",
			 com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog,
{	
    width: "70%",
	postCreate: function()
	{
		this.inherited( "postCreate", arguments );

        if (this.title) {
            this.setTitle(this.title);
        }
        this.assetType = "Machine";
        this.asset = catalog.createConfiguration(this.assetType);
        this.configPart = this.asset.getRootPart(0);
        
        //Create the table that will hold the assets
        this.createTable();
        this.loadTable();
        this._checkButtonStatus();
		
		//Add the contents to the dialog
		this.contentContainer.appendChild( this.tableViewer.domNode );		
	},
	
	//allow the user to specify a callback when OK is clicked
	setOKAction :function(callback)
	{
		this.yesCallback = callback;
	},
	
    //allow the user to specify a callback when Cancel is clicked
    setCancelAction :function(callback)
    {
        this.noCallback = callback;
    },
	
	getSelectedAssetIDs: function()
	{
		var retVal = [];
		var rows = this.tableViewer.getCheckedRows();
		for( var iRow = 0; iRow < rows.length; iRow++ )
		{
			var id = this.tableContentProvider.getRowDataId( rows[ iRow ] ) ;
			if (id)
				retVal.push(id);
		}
		return(retVal);	
	},
	
	getAssetType: function()
	{
		return this.assetType;
	},
	
    show: function() {
        this.open();
    },
    
	createTable: function()
	{
		this.tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer();
		this.tableContentProvider = new com.ibm.rational.test.lm.web.ui.internal.views.providers.AssetSelectorDialogProvider( this.assetType );
		this.tableContentProvider.receiver = this;
		this.tableViewer.setTableSummary(RtlmUiUtil.getMessage("labelTableSummary", RtlmUiUtil.getMessage("labelAsset")));
				
		this.tableViewer.setTableRowsCheckable( true );

		this.tableViewer.setContentProvider( this.tableContentProvider );
		this.tableViewer.setTableListener( this );

		//Add a refresh button just in case the user wants to refresh the view.  Maybe remove this since it's a modal dialog.
		this.refreshButton = this.tableViewer.addAction( "Search", RtlmUiUtil.getMessage("Refresh"), null, "refresh-icon-image", dojo.hitch( this, this._refreshSelected ) );

        //Create the asset type selector
        this._constructAssetSelector();
        this._clearTable();        
	},
	
	loadTable: function()
	{
		this.refreshButton.setDisabled( true );
	   	var responseHandler = new ServiceResponseHandler(this, "_searchSuccess", "_searchError");
        
        this.tableViewer.showLoadingProgress();
        
		this._searchAttributeCriteria = MachineInfo.getSearchAttributeCriteria()[this.assetType];
		
		var searchParam = {
                    configuration: this.asset,
                    columns: ( this._searchAttributeCriteria )
                };
        catalog.search(responseHandler, searchParam);		
	},
	
	_searchSuccess: function( response )
	{
		if (!(response.Header)){
			var colnames = this._searchAttributeCriteria.replace(/, /g,',');
			colnames = colnames.split(',');
			response.Header = colnames;
		}

        this.tableViewer.noItemMessage = "";

		this.refreshButton.setDisabled( false );
        this.tableViewer.endLoadingProgress();
        this.tableViewer.setContent( response );
        this.tableViewer.show();
        this.updatePosition();
	},
	
	_searchError: function( message, error )
	{
    	this.refreshButton.setDisabled( false );
        this.tableViewer.endLoadingProgress();
        this.tableViewer.noItemMessage = RtlmUiUtil.getMessage("failedLookupAssets");
        this.tableViewer.setContent({Data:[]});
        this.tableViewer.show();
	},
	
	_clearTable: function()
	{
        // loads an empty content to the table to clear the content.
        var emptyContent = {Data:[]};
        
        // don't show the standard empty content message. This is intensional
        // to have an empty content in the table.
        this.tableViewer.noItemMessage = " ";
        this.tableViewer.setContent(emptyContent);
        this.tableViewer.show();
	},
	
	//Asset selector methods
	_constructAssetSelector: function()
	{
        var views = [];
        var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
        this.assetTypeNames = catalog.getAssetTemplateNames();
                
        for(var i in this.assetTypeNames) {
            var opts = {
                'name': RtlmUiUtil.getMessage( this.assetTypeNames[i] ),
                'value': this.assetTypeNames[i]
            };
            views.push(opts);
        }
		this.assetType = this.assetTypeNames[0];
        this.tableViewer.setTableContentSelectors(views);
	},
	
	//Load the asset type specified in this.assetType
	_loadAssetType: function(assetType)
	{
        this.assetType = assetType;

        this.tableContentProvider = new com.ibm.rational.test.lm.web.ui.internal.views.providers.AssetSelectorDialogProvider( this.assetType );
        this.tableContentProvider.receiver = this;
                
        this.tableViewer.setContentProvider( this.tableContentProvider );

		//A new asset type so create a new blank config based on the type
		this.asset = catalog.createConfiguration(this.assetType);
		this.configPart = this.asset.getRootPart(0);
		
		//And recreate and load the table
 		this._clearTable();
		this.loadTable();
	},
	
	// a row was checked so lets enable and disable actions appropriately    
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
    	this._checkButtonStatus();
    },

    tableContentSelectionChanged: function(tableViewer, contentSelectionValue)
    {
        this._loadAssetType(contentSelectionValue);
    },
	
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback)
    {
        callback(callbackData);
    },

	_refreshSelected: function()
	{
		this._clearTable();
		this.loadTable();
	},
	
	_checkButtonStatus: function()
	{
		var rows = this.tableViewer.getCheckedRows();
        this._yesButton.disabled = (!rows || rows.length == 0);
	}				
});

})();

