/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.machines.ImportMachineView");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.rational.test.lm.web.client.internal.machine");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ImportPageLayout");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.Tooltip");

(function() {
	
var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;	
var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.machines.ImportMachineView",
    [dijit._Widget, dijit._Templated],
{
	
	templatePath: dojo.moduleUrl("com.ibm.rational.test.lm.web", "ui/internal/views/machines/templates/ImportMachineView.html"),
	widgetsInTemplate: true,

	//default prompt for asset import
	machineImportPrompt: RtlmUiUtil.getMessage("impPrompt_asset"),
	machineImportTooltip: RtlmUiUtil.getMessage("impMachTooltip"),
	machineImportButtonLabel: RtlmUiUtil.getMessage("impMachImport"),
	
	_importType: null,
	_timerId: null,	
	_machine: new com.ibm.rational.test.lm.web.client.internal.machine(),
	_importId: null,
	_statusCheckResponseHandler: null,
	
	constructor: function(/* Required. String. one of the strings returned from machine.getAllowedImportOps(). e.g. "asset" or "schema"*/importType) {
		this._importType = importType;
	},
	
	postCreate: function() {
        //console.debug("Import machine postCreate called");
        com.ibm.rational.test.lm.web.ui.internal.views.machines.ImportMachineView.superclass.postCreate.apply(this, arguments);
		
        if(this._importType) {
			this.mainImportSection.setTitle(RtlmUiUtil.getMessage("impPrompt_" + this._importType));     
	        this.viewHeaderLayoutNode.setTitle(RtlmUiUtil.getMessage("impLabel_" + this._importType));
	        this.viewHeaderLayoutNode.showContentArea(false);
	        if (this._importType == "asset") {
	            this.viewHeaderLayoutNode.setCSHelpId("rtlm_import_machine.html", com.ibm.rational.test.lm.web.CSHelpID);
				this.viewHeaderLayoutNode.setCSHelpButtonLabel(RtlmUiUtil.getMessage("labelHelp", "", RtlmUiUtil.getMessage("impLabel_" + this._importType)));
            }
		}
        this.fileLabel.innerHTML = RtlmUiUtil.getMessage("labelFile");
        this.machineImportButtonNode = this.viewHeaderLayoutNode.getImportButton();
        this.viewHeaderLayoutNode.onImport = dojo.hitch(this, "importMachinesUpload");
	},
	
	updateImportMachinesState: function() {
		this.viewHeaderLayoutNode.clearStatus();
		//enable import button only when file input field is not empty
		this.machineImportButtonNode.disabled = (this.machineImportInputNode.value.length == 0);
	},
	
	importMachinesUpload: function() {
		  var extIsXML = false;
		  
		  // clear status
		  this.viewHeaderLayoutNode.clearStatus();

		  // check the selected file extension to see if it is xml file
		    var filepath = this.machineImportInputNode.value;
		    var filenameArray = filepath.split("/");
		    if (filenameArray.length == 1) {
		      // no forward slash... hmm... windows system
		      filenameArray = filepath.split("\\");
		    }
		    if (filenameArray.length > 0) {
		      var filename = filenameArray[filenameArray.length - 1];
		      var filenameParts = filename.split(".");
		      if (filenameParts.length > 0) {
		        var ext = filenameParts[filenameParts.length - 1];
		        extIsXML = ext.toLowerCase() == "xml";
		      }
		      if (!extIsXML) {
		        extIsXML = confirm(RtlmUiUtil.getMessage("impMachFileNotXML", filename));
		      }
		    }
		
		  if (extIsXML) {		    
		    // bring up a working/busy dialog while we're fetching the results
		    this.viewHeaderLayoutNode.setProgressStart(RtlmUiUtil.getMessage("impMachUploading"));
		    var status = this._machine.uploadImportFile(this.machineImportFormNode.id, this._importType);
		    // disable buttons and file selection when uploading
		    this.machineImportInputNode.disabled = true;
		    this.machineImportButtonNode.disabled = true;
		  
		    status.addCallback(dojo.hitch(this, "_uploadMachineSuccess"));		
		    status.addErrback(dojo.hitch(this, "_uploadMachineError"));
		  }
	},
	
	_uploadMachineSuccess: function(importId) {
			
		this._importId = importId;
		this._statusCheckResponseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_importStatusCheckSuccess", "_importStatusCheckError");
		this._timedImportStatusCheck();	
	},
	
	_uploadMachineError: function(response) {
		  // bring down the working dialog
	      this.viewHeaderLayoutNode.setProgressEnd();
	
	      // enable buttons but don't clear selection on error
	      this.machineImportInputNode.disabled = false;
	      this.machineImportButtonNode.disabled = false;

	        if (response.dojoType == "cancel") {					
			  	this.viewHeaderLayoutNode.setStatus(RtlmUiUtil.getMessage("impMachCancel"), ViewHeaderLayout.ERROR);
	        }
	        else if (response.dojoType == "timeout") {					
			  	this.viewHeaderLayoutNode.setStatus(RtlmUiUtil.getMessage("impMachTimeout"), ViewHeaderLayout.ERROR);
	        } else {
	            //Some other error happened.
                if(response.message && response.message.toLowerCase().indexOf('license') > -1) {
                    RtlmUiUtil.showServerErrorMessage(MessageUtil.getMessage(MessageBundle.COMMON, "serverLicenseErrMsg"));                  
                }
			  	this.viewHeaderLayoutNode.setStatus(RtlmUiUtil.getMessage("impMachFailed"), ViewHeaderLayout.ERROR);
	        }
	},
	
	_timedImportStatusCheck: function() {
		this._timerId = window.setTimeout(dojo.hitch(this, '_timedImportStatusCheck'), 2000);
		this._machine.getUploadStatus(this._statusCheckResponseHandler, this._importId);
	},
	
	_stopImportStatusCheck: function() {
		window.clearTimeout(this._timerId);
	    this.machineImportFormNode.reset();
	    this.machineImportInputNode.disabled = false;
	    this.machineImportButtonNode.disabled = true;
	},
	
	_importStatusCheckSuccess: function(response) {
        // Response will be null if the status cannot be
        // determined.  This will happen if there is a race
        // condition or if the task status has been purged.
        if(response == null) 
            return;
		var status = response.importStatus;
            // status code 0 - 2 are info, import is not done yet
		if(status < 2){
			this.viewHeaderLayoutNode.setStatus(RtlmUiUtil.getMessage("impStatus-" + status), ViewHeaderLayout.INFO);
		}
		else if(status == 2){
			this.viewHeaderLayoutNode.setStatus(RtlmUiUtil.getMessage("impStatus-" + status, response.numImportedElems, response.numImportableElems), ViewHeaderLayout.INFO);
			
		}
		else if(status >= 3 && status <= 5){
            // status code 3 - 5 are actually error condition
			this.viewHeaderLayoutNode.setStatus(RtlmUiUtil.getMessage("impStatus-" + status), ViewHeaderLayout.ERROR);
			this._stopImportStatusCheck();
            this.viewHeaderLayoutNode.setProgressEnd();
		}
		else if(status == 6){
			this.viewHeaderLayoutNode.setStatus(RtlmUiUtil.getMessage("impStatus-" + status, response.numImportedElems, response.numImportableElems), ViewHeaderLayout.SUCCESS);
			this._stopImportStatusCheck();
            this.viewHeaderLayoutNode.setProgressEnd();
			this._checkCPULimit();
		}
	},
	
	_importStatusCheckError: function(err) {
		console.debug(err);		
		this._stopImportStatusCheck();
		
	    this.viewHeaderLayoutNode.setProgressEnd();
	    this.machineImportInputNode.disabled = false;
	    this.machineImportButtonNode.disabled = false;
		this.viewHeaderLayoutNode.setStatus(RtlmUiUtil.getMessage("impMachFailed"), ViewHeaderLayout.ERROR);
	},
	
	//Checks to see if the number of CPUs in the system exeeds the licensed limit.
	//If it does, it displays a warning.
	_checkCPULimit: function()
	{
        var adminClient = new com.ibm.rational.test.lm.web.client.internal.admin();
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(
            {
                parent: this,
                _success: function(response) 
                {
					var cpuLimit = response.globalCpuSoftLimit;
					var numCPUs = response.numGlobalCpus;
					if (numCPUs > cpuLimit)
						alert(RtlmUiUtil.getMessage("msgCPULimit", numCPUs, cpuLimit));
                },
                _failure: function(errObj) 
                {
                    //Ignore failure
                }
            },
            "_success",
            "_failure"
        );
        adminClient.getSystemOptions(responseHandler);
	}
	
});

})();
