/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.ActionConfigSummary");
 
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.ConfigSummary");

(function() 
{
	dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.ActionConfigSummary", 
		com.ibm.rational.test.lm.web.ui.internal.widgets.ConfigSummary,
{    
    templateString: '<div class="config-summary" dojoAttachPoint="domNode" tabindex="-1">' +
                    '    <table dojoAttachPoint="SummaryTable" class="config-table">' +
                    '     <tr>' +
                    '       <td width="100%" align="left">' +
                    '         <div dojoAttachPoint="configArea"></div>' +
                    '       </td>' +
                    '     </tr>' +
                    '     <tr>' +
                    '       <td width="100%" align="right">' +
                    '         <div dojoAttachPoint="configActionsNode"></div>' +
                    '       </td>' +
                    '     </tr>' +
                    '   </table>' +
					'</div>',
					
	_partSectionsMap: null,
					
	_constructWidget :function() {
		this._partSectionsMap = [];
		this.inherited("_constructWidget", arguments);	
	},
	
	/**
	 * Add an action to the bottom right corner of this config summary widget.
	 * @param {Object} label
	 * @param {Object} handler
	 * @param {Object} callbackData
	 */
	addAction: function(label, handler, callbackData, buttonClass) {
		var buttonNode = document.createElement("button");

		//Give clients access to the button in case they want to hide it, change the label, etc.
		if (callbackData != null)
			callbackData.buttonNode = buttonNode;
			
		if (buttonClass) 
			dojo.addClass(buttonNode, buttonClass)
		else 
		{
			dojo.addClass(buttonNode, "formActionLink");
			buttonNode.innerHTML = label;
		}
					
		buttonNode.onclick = function() {
            this.clientCallback(this.callbackData);
        };
        
        // store the client callback and data into the button
        buttonNode.parent = this;
        buttonNode.clientCallback = handler;
        buttonNode.callbackData = callbackData;
        
        this.configActionsNode.appendChild(buttonNode);
        return buttonNode;
    },
	
	addActionDeliminator: function(label) {
		var span = document.createElement("span");
		span.innerHTML = label;
		this.configActionsNode.appendChild(span);
	},
	
	/**
	 * When there is multiple parts in this config, this function is called to create a section for each part to shown in.
	 * @param {Object} sectionTitle
	 */
	_createPartSection: function(partIndex) {
		
		var sectionTitle = this.asset.getLabel(partIndex);
		var tr = this.configTable.insertRow(this.configTable.rows.length);
		var td = tr.insertCell(0);
		
		var isOpen = this.open ? true : false; 
        var section = new com.ibm.asq.common.web.ui.internal.view.common.MoreLessCollapsibleArea({title: sectionTitle, open: isOpen});
		this._partSectionsMap[partIndex] = section;
		td.appendChild(section.domNode);
		
		var contentTable = document.createElement("table");
		section.contentNode.appendChild(contentTable);
		
		return contentTable;
	},
	
	/**
	 * When there is multiple parts in this config, add an action on each part. No-op if there's only one part.
	 */
	addPartAction: function(label, handler, iconClass) {
		
		if(!this._partSectionsMap || this._partSectionsMap.length < 2)
			return;
			
		for (var i in this._partSectionsMap) {
			var callbackData = {'configID' : this.configID ? this.configID : null,
								'configPart': this.asset.getRootPart(i),
								'partIndex' : i};	
			this._partSectionsMap[i].addAction(label, handler, callbackData, iconClass);
		}
	}

});

})();
	
