/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 
 dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.CSHelp");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout.ContentPane");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.CSHelp", [dijit.layout.ContentPane, dijit._Templated],
{
    templateString: 
    '<div id="${id}" dojoAttachPoint="domNode">' +
    '<!-- CS help panel that can be close by mouse click, key press, or losing focus. -->' +
    '<div dojoAttachPoint="cshelp_panel" style="z-index: 999 !important; position:absolute; display:none;" class="csHelpPanel" align="center">' +
    '  <div align="right">' +
    '    <div class="csHelpCloseImage" tabindex="0"' +
    '         dojoAttachEvent="onclick: hideHelp"' +
    '         onKeyUp="if (this.buttonKeyPressed(event)) this.hideHelp(this.cshelp_panel);">&nbsp;' +
    '    </div>' +
    '    <div class="csHelpCloseText" tabindex="1"' +
    '         dojoAttachEvent="onclick: hideHelp"' +
    '         dojoAttachEvent="onkeyup: onkeyup"' +
    '    </div>' +
    '  </div>' +
    '  <div dojoAttachPoint="cshelp_content" class="csHelpContent" align="left"' +
    '       dojoAttachEvent="onkeypress:onKeyPressInContent"' +
    '  </div>' +
    '  <hr class="csHelpSeparator"></hr>' +
    '  <div align="left">' +
    '    <a dojoAttachPoint="csHelp_infoCtr" class="csHelpInfoLink" href=\'/help/index.jsp\' target=\'_blank\' ' +
    '       dojoAttachEvent="onkeypress:onKeyPressInContent"' +
    '       Link to Info Center</a>' +
    '  </div>' +
    '</div>'+
    '<!-- Use an iframe as blocker to workaround IE dont honor z-index -->'+
    '<!-- properly and popup can go behind combo box.  Also resolve the -->'+
    '<!-- popup being transparent in high contrast mode.                -->'+
    '<iframe dojoAttachPoint="csHelpBlocker" name="csHelpBlocker" frameborder="0" scrolling="no"'+
    '        style="z-index: 998 !important; position: absolute; left: -10000px; display: block;"></iframe>',
    

	onkeyup: function(event)
	{
		if (this.buttonKeyPressed(event))
			this.hideHelp();
	},
	
	onKeyPressInContent: function(event)
	{
		if (this.escapeKeyPressed(event))
			this.hideHelp();
	},
	    
 	escapeKeyPressed: function(event) 
	{
	    var charCode = (event.which) ? event.which : event.keyCode
	    return charCode == 27;
	},

	buttonKeyPressed: function (event) 
	{
	    var charCode = (event.which) ? event.which : event.keyCode
	    // return spacebar or enter
	    return (charCode == 32) || (charCode == 13);
	},

	callerButton: null,

	/**
	 * Hide the context sensitive help panel.
	 */
	hideHelp: function() 
	{
		var help_panel = this.cshelp_panel;
	    var helpBlocker = this.csHelpBlocker;
	    if (helpBlocker) 
	    {
	        helpBlocker.style.left = '-10000px';
	        helpBlocker.style.display = 'none';
	    }
	    if (help_panel) 
	    {
	        help_panel.style.display = 'none';
	    }
	    if (this.callerButton) 
	    {
	        // restore the focus on the button which bring up the help
	        this.callerButton.focus();
	        this.callerButton = null;
	    }
	},

	/**
	 * Show the context sensitive help content in a predefined help panel.
	 */
	showHelpContent: function(helpButton, contextId) 
	{
		var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;

	    // save the button which bring up the help so
	    // we can restore to focus when we are done
	    this.callerButton = helpButton;
		    
	    var help_panel = this.cshelp_panel;
	    var help_content = this.cshelp_content;
		    
	    if ((help_panel != null) && (help_content != null)) 
	    {
	        // clear the help panel.
			help_content.innerHTML = "";
			help_content.style.height = "";
		
			// Get the content from server
			dojo.xhrGet({
				url : "/help/context?id=com.ibm.rational.test.lm.web." + contextId,
		            handleAs :"text",
		            timeout :5000, // Time in milliseconds
		            parentCSHelp: this,
		
		            // Set the help content with the response.
		            load: function(response, ioArgs) {
		                this.parentCSHelp.showHelpPopup(response);
		                return response;
		            },
		
		            // The ERROR function will be called in an error case.
		            error: function(response, ioArgs) {
		                this.parentCSHelp.showHelpPopup(RtlmUiUtil.getMessage("MsgFailGetHelpContent"));
		                console.error("HTTP status code: ", ioArgs.xhr.status);
		                return response;
		            }
		        });
		}
	},

	showHelpPopup: function(cshelpContent) 
	{
	    var help_panel = this.cshelp_panel;
	    var help_content = this.cshelp_content;
	
	    // Get the position of the Help button
	    var buttonXY = dojo.coords(this.callerButton, true);
	    var helpbutton_x = buttonXY.x;
	    var helpbutton_y = buttonXY.y;
	
	    // Set the position of the help panel
	    help_panel.style.left = (helpbutton_x + 20) + 'px';
	    help_panel.style.top = (helpbutton_y + 20) + 'px';
	    
	    // Get the width and height of the visible document
	    var docWidth = document.body.clientWidth;
	    var docHeight = document.body.clientHeight;
	    
	    // width is default to 1/3 of the document width
	    help_panel.style.width = Math.round(docWidth / 3) + 'px';
	
	    // Panel height is driven by the content and the max is 2/3 of the document height
	    var maxHeight = Math.round(docHeight * 2 / 3);
	    
	    // hide the help panel before setting to display in order
	    // to gather size and position of the panel
	    help_panel.style.visibility = 'hidden';
	    help_panel.style.display = 'block';
	
	    help_content.innerHTML += cshelpContent;
	
	    // get the panel height after setting the content
	    var panelHeight = help_panel.clientHeight;
	    var panelWidth = help_panel.clientWidth;
	    
	    // resize if it is more than max...  pad 100 pixels for
	    // buffer of adding scroll bar 
	    if (panelHeight > (maxHeight + 100)) {
	        // set the content height so the scroll bar is within the content
	        // and not the outer panel
	        help_content.style.height = maxHeight + 'px';
	        // update the panel height
	        panelHeight = help_panel.clientHeight;
	    }
	        
	    // determine whether the help panel is out of the scope of the document
	    if ((helpbutton_x + panelWidth + 20) > docWidth) 
	    {
	        help_panel.style.left = (docWidth - panelWidth - 20) + 'px';
	    }
	    if ((helpbutton_y + panelHeight + 20) > docHeight) 
	    {
	        // ensure if the must be clipping, clipping is at the bottom to show the close button
	        help_panel.style.top = Math.max(0, (docHeight - panelHeight - 20)) + 'px';
	    }
	
	    // Use an iframe as blocker to workaround IE don't honor z-index properly
	    // and popup can go behind combo box.  Also resolve the popup being
	    // transparent in high contrast mode.
	    var helpBlocker = this.csHelpBlocker;
	    if (helpBlocker) 
	    {
	        helpBlocker.width = panelWidth + 2;  // 2 pixels for the border of info pop
	        helpBlocker.height = panelHeight + 2;
	        helpBlocker.style.width = panelWidth + 2 + 'px';
	        helpBlocker.style.height = panelHeight + 2 + 'px';
	        helpBlocker.style.top = help_panel.style.top;
	        helpBlocker.style.left = help_panel.style.left;
	        helpBlocker.style.display = 'block';
	    }
	
	    // now we can show the panel
	    help_panel.style.visibility = 'visible';
	    help_panel.focus();  // IE can't focus content div if there is no html element
	    help_content.focus();  // FF can't focus the outer panel, we want focus here
	}
        
});
    
