/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
CommonDialog = dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
CommonDialog.blue   = 0;
CommonDialog.gray   = 1;
CommonDialog.gold   = 2;
CommonDialog.green  = 3;
CommonDialog.teal   = 4;
CommonDialog.purple = 5;
CommonDialog.orange = 6;

dojo.require("dijit.Dialog");
dojo.require("net.jazz.ajax.ui.internal.Dialog");
dojo.require("dojo.dnd.Moveable");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets._CommonDialog",
net.jazz.ajax.ui.internal.Dialog,
{
    // Summary:
    //    An internal class that defines the content container area and the
    //    action buttons container area.
    //
    width: "60%",
    contentContainer: null,
    separator: null,
    actionsContainer: null,
	showActions: true,
	opened: false,
	movable: false,					//allows moving the dialog, turned off by default
	closing: false,
	afterUpdatePositionHandler: null, //handler to be called after updatePosition
    csHelpID: null, // if specified a CS Help Icon is shown in the title area.
    csHelpEngine: null,
    cshelp: null,
    csHelpProviderId: null,
	zIndex: 10001,
    forceZIndex: false,
    
    postCreate: function() {
        this.inherited('postCreate', arguments);
        dojo.addClass(this.domNode, "com-ibm-asq-common-web-dialog");
        
        var containerWrapper = document.createElement("div");
        containerWrapper.className = "content-container";
        this.contentContainer = document.createElement("span");
        containerWrapper.appendChild(this.contentContainer);

        this.separator = document.createElement("div");
        this.separator.className = "separator";

        this.actionsContainer = document.createElement("div");
        this.actionsContainer.className = "actions-container";

        this.contentNode.appendChild(containerWrapper);
        this.contentNode.appendChild(this.separator);
        this.contentNode.appendChild(this.actionsContainer);
         
        this.blockerDiv.style.display = "none";
        this.domNode.style.display = "none";
		
		// force front of zorder in FF3, unnecessary in FF2 and IE7
       // if (this.forceZIndex || (dojo.isFF >= 3) || (dojo.isChrome)) {
            this.blockerDiv.style.zIndex=this.zIndex;
            this.domNode.style.zIndex=this.zIndex + 1;
       // }
		if(this.movable){
			this._moveable = new dojo.dnd.Moveable(this.domNode, { handle: this._header });
		}
        if (!this.showActions) {
            this.separator.style.display = "none";
            this.actionsContainer.style.display = "none";
        }	
        
        this.truncateTitle = true;
        
        // setup the cs help button.
        if (this.csHelpID) {
            this.setCSHelp();
        }
		
		//for accessibility reasons 
		this._keyboardconnects = [];
    },
    
    setCSHelp: function()
    {
        if (this._headerText) {
            if (this.csHelpID) {   
                // disable the title truncation when we add
                // the cs help icon. Otherwise, it will trim 
                // the title for no reason other then the title
                // height got taller.
                this.truncateTitle = false;
                
                if(!this.helpButtonNode) {
                    this.helpButtonNode = document.createElement("span");
                    dojo.addClass(this.helpButtonNode, "help-button-node");
                    this._headerText.appendChild(this.helpButtonNode);
                }
                this.helpButtonNode.style.display = "inline";
                var args = {
                    'contextID': this.csHelpID,
                    'csHelpEngine': this.csHelpEngine,
                    'csHelpProviderId': this.csHelpProviderId
                };
                this.cshelp = new com.ibm.asq.common.web.ui.internal.util.CSHelp(args);
                this.helpButtonNode.appendChild(this.cshelp.domNode);
            } 
            else if(this.helpButtonNode){
                this.helpButtonNode.style.display = "none";
            }
        }
    },
    
    open: function(skipCompact) {
        if (skipCompact) {
            // skip compact will honor the width setting
            this.blockerDiv.style.display = "block";
            this.domNode.style.display = "block";
        } else {
            // compact the dialog width to the contentContainer's width
            this.domNode.style.visibility = "hidden";
            // enable the display after hiding it to make it invibsible
            // but yet able to compose the contents
            this.domNode.style.display = "block";
            var minWidth = this.contentContainer.style.minWidth ? parseInt(this.contentContainer.style.minWidth) : 0;
            var mb = dojo.marginBox(this.contentContainer);
            if (mb.w)
                this.setWidth((Math.max(mb.w, minWidth) + 50) + "px");      
            this.blockerDiv.style.display = "block";
            this.domNode.style.visibility = "visible";
            
            this.blockerDiv.style.zindex = "5000";
            this.domNode.style.zindex = "5000";
            
            if(!dojo.isFF && !dojo.isIE)
            	this.blockerDiv.style.position = "fixed";
            
            
        }
		
		this.opened = true;
		
		this._keyboardconnects.push(dojo.connect(dojo.doc.documentElement, "onkeypress", this, "_onKey"));
		
		//Call updatePosition once the dialog is made.  This will place it in the current view
		//correctly without scrolling the window to the top, which can be very annoying, if one
		//wants to stay in the same position after the dialog is dismissed.				
		this.updatePosition();
		
		if (this._bodyContainer) {
            this._bodyContainer.focus();
        }	
    },
	
	close: function()
	{
		this.closing = true;
		this.inherited( "close", arguments );
		this.opened = false;
		this.closing = false;
		
		dojo.forEach(this._keyboardconnects, dojo.disconnect);
			this._keyboardconnects = [];
	},

    setTitle: function(/* string */title){
        this.primaryTitle = title;
        this.inherited('setTitle', arguments);
    },
	
	setSecondaryTitleNode: function(/*domNode*/domNode) {
		if (domNode && this._headerSecondary) {
			this._headerSecondary.appendChild(domNode);
		}
	},
	
	/*
	 * Override net.jazz.ajax Dialog's '_truncate' method which seems to loop 
	 * infintely when primaryTitle is not set
	 */
	_truncate: function() {
		if (this.truncateTitle && this.primaryTitle && dojo.trim(this.primaryTitle).length > 0) {
			this.inherited('_truncate', arguments);
		}
	},
	
	/*updatePosition: function( event, blockerOnly )
	{
		//If the method is not overriden and the check made below before calling the subclassed method,
		//error can sometimes occur in IE because a call is trying to be made on the dialog when it is not visible.
		if( this.opened && !this.closing )
		{
			this.inherited( "updatePosition", arguments );
		}
	}*/
	
	/*
	 * Override net.jazz.ajax Dialog's 'updatePosition' method to support callback after creation
	 */
	updatePosition: function(event, blockerOnly) {
		if (this.opened && !this.closing) {
			var de = document.documentElement;
			var height = de.clientHeight;
			var width = de.clientWidth;
			var scrollTop = de.scrollTop;
			var scrollLeft = de.scrollLeft;
			if (this.blockerDiv) {
				this.blockerDiv.style.height = height + "px";
				this.blockerDiv.style.top = scrollTop + "px";
				this.blockerDiv.style.left = scrollLeft + "px";
			}
			
			
			if (blockerOnly) 
				return;
			
			this._bodyContainer.style.maxHeight = height - 40 + "px";
			this._dialog.style.maxWidth = width - 40 + "px";
			
			if (this.center) {
				var coords = dojo.coords(this._dialog);
				var top = height / 2 - coords.h / 2 + scrollTop;
				if (top < 10) 
					top = 10;
				var left = width / 2 - coords.w / 2 + scrollLeft;
				if (left < 10) 
					left = 10;
				dojo.marginBox(this._dialog, {
					l: left,
					t: top
				});
			}
			this.callAfterUpdatePosition();
		}
	},
	
	//callback after updatePosition
	callAfterUpdatePosition: function(){
		if(this.afterUpdatePositionHandler)
			this.afterUpdatePositionHandler();
	},
	
	_onKey: function(/*Event*/ evt){
		// summary: handles the keyboard events for accessibility reasons
		if (evt.keyCode) {
			var node = evt.target;
			// see if the key is for the dialog
			while (node) {
				if (node == this.domNode) {
					if (evt.keyCode == dojo.keys.ESCAPE) {
						this.close();
					}
					else {
						return; // just let it go
					}
				}
				node = node.parentNode;
			}
		}
	}
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog",
com.ibm.asq.common.web.ui.internal.widgets._CommonDialog,
{
    // Summary:
    //    A dialog with the "Yes" and "No" buttons with the option to
    //    show the "Cancel" button as well.  Callbacks with callback data
    //    can be specified and the "Cancel" will simply close the dialog.
    //
    isClosable: false,
    yesLabel: MessageUtil.getMessage(MessageBundle.COMMON, "actionLabel_yes"),
    noLabel: MessageUtil.getMessage(MessageBundle.COMMON, "actionLabel_no"),
    cancelLabel: MessageUtil.getMessage(MessageBundle.COMMON, "actionLabel_cancel"),
    _yesButton: null,
    _noButton: null,
    _cancelButton: null,
    yesCallback: null,
    yesCallbackData: null,
    noCallback: null,
    noCallbackData: null,
    showCancel: false,
	cancelCallback: null,
	message: null,
    _messageNode: null,
    
    postCreate: function() {
        this.inherited('postCreate', arguments);
		
		var yesButtonDiv = document.createElement("span");
        this._yesButton = document.createElement("button");
        this._yesButton.innerHTML = this.yesLabel;
		this._yesButton.tabIndex = "0";
        this._yesButton.onclick = dojo.hitch(this, "_yesAction");
		yesButtonDiv.appendChild(this._yesButton);
		yesButtonDiv.className = "button-div";
        
		var noButtonDiv = document.createElement("span");
        this._noButton = document.createElement("button");
        this._noButton.innerHTML = this.noLabel;
		this._noButton.tabIndex = "0";
        this._noButton.onclick = dojo.hitch(this, "_noAction");
		noButtonDiv.appendChild(this._noButton);
		noButtonDiv.className = "button-div";
        
        this.actionsContainer.appendChild(yesButtonDiv);
        this.actionsContainer.appendChild(noButtonDiv);

        if (this.showCancel) {
			var cancelButtonDiv = document.createElement("span");
            this._cancelButton = document.createElement("button");
            this._cancelButton.innerHTML = this.cancelLabel;
			this._cancelButton.tabIndex = "0";
            this._cancelButton.onclick = dojo.hitch(this, "_cancelAction");
			cancelButtonDiv.appendChild(this._cancelButton);
			cancelButtonDiv.className = "button-div";
			this.actionsContainer.appendChild(cancelButtonDiv);
        }
				
        if (this.message)
            this.setMessage(this.message);      
    },
	   
    setMessage: function(msg) {
        if (!this._messageNode) {
            this._messageNode = document.createElement("span");
            this.contentContainer.appendChild(this._messageNode);
        }
        this._messageNode.innerHTML = msg;
		this._messageNode.tabIndex = "0";
    },
	
    _yesAction: function() {
        // disable button to prevent invoking twice from double click on the button
        this._yesButton.disabled = true;
        this.close();
        if (dojo.isFunction(this.yesCallback)) {
			this.yesCallback(this.yesCallbackData);
		}
    },

    _noAction: function() {
        // disable button to prevent invoking twice from double click on the button
        this._noButton.disabled = true;
        this.close();
        if (dojo.isFunction(this.noCallback)) {
			this.noCallback(this.noCallbackData);
		}
    },
	
    _cancelAction: function() {
        // disable button to prevent invoking twice from double click on the button
        this._cancelButton.disabled = true;
        this.close();
        if (dojo.isFunction(this.cancelCallback)) {
			this.cancelCallback();
		}
    },	
	
	//override from _CommonDialog, focus to message node when open the dialog.
    open: function(skipCompact){
		this.inherited('open', arguments);
		 if (this._messageNode) {
		 	var self = this;
            this._messageNode.focus();
            setTimeout(function(){
                self._messageNode.blur();
            }, 100);
        }
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog",
com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog,
{
    // Summary:
    //    Basically the YesNoDialog with the "OK" and "Cancel" label instead.
    //
    okLabel: MessageUtil.getMessage(MessageBundle.COMMON, "actionLabel_ok"),
    cancelLabel: MessageUtil.getMessage(MessageBundle.COMMON, "actionLabel_cancel"),
    okCallback: null,
    okCallbackData: null,
    cancelCallback: null,
    cancelCallbackData: null,
    
    postCreate: function() {
        // the "No" button is labelled as the "Cancel" button to allow option
        // to specify callback
        this.showCancel = false;
        
        // map the appropriated named variables to the functional variables
        this.yesCallback = this.okCallback;
        this.yesCallbackData = this.okCallbackData;
        this.noCallback = this.cancelCallback;
        this.noCallbackData = this.cancelCallbackData;
        this.yesLabel = this.okLabel;
        this.noLabel = this.cancelLabel;
        
        this.inherited('postCreate', arguments);
    }
 
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.AlertDialog",
com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog,
{
    // Summary:
    //    Simple dialog with just the "OK" button.
    //
    postCreate: function() {
        this.inherited('postCreate', arguments);
        
        // simply hide the "Cancel" button
        this._noButton.style.display = "none";    
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog",
com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog,
{
    selectionHandler: null,
    
    // Summary:
    //    A confirmation dialog that expect a selection is made in the
    //    content area.  The consumer needs to supply the handler to
    //    get the selection and control the enablement of the "OK" button.
    //
    postCreate: function() {
        this.inherited('postCreate', arguments);

        // disable the "OK" button by default and let the consumer
        // control the enablement
        this._yesButton.disabled = true;
    },
    
    // Override the "_yesAction" handler to invoke the "selectionHandler" if it
    // is specified.  The selection is also passed to the "yesCallback" handler
    // for validation and the dialog won't close if the "yesCallback" handler
    // returns false.
    // 
    _yesAction: function() {
        // disable button to prevent invoking twice from double click on the button
        this._yesButton.disabled = true;
        var selection = null;
        if (dojo.isFunction(this.selectionHandler)) {
			selection = this.selectionHandler();
		}            
                     
        if (dojo.isFunction(this.yesCallback)) {
			//allow callback to cancel close (for example, if some validation fails)
			var callbackResponse = this.yesCallback(this.yesCallbackData, selection);
			if (callbackResponse != null && callbackResponse == false) {
                // enable the button if the dialog is not closing
                this._yesButton.disabled = false;
				//do not close
				return;
			}
		} 
		this.close();
    },

    getOkButton: function() {
        return this._yesButton;
    },
	
	getCancelButton: function() {
		return this._noButton;
	}
	
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.OptionsDialog",
com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog,
{
    options: [],     // array of options supplied by the consumer
    optionNodes: [], // corresponding array of checkboxes for the options
    selection: [],   // the options that the user selected
    
    // Summary:
    //    A confirmation dialog that presents a list of checkboxes for
    //    selection in the content area.  The entries in the array of
    //    specified options must have a "name" property which is used
    //    to present in the checkboxes.
    //
    //    Note that if options is not specified this dialog will behave
    //    the same as a confirmation dialog.
    //
    postCreate: function() {
        this.inherited('postCreate', arguments);

        if (this.options.length > 0) {
            var optionContainer = document.createElement("form");
            optionContainer.style.marginLeft = "10px";
            optionContainer.style.marginTop = "5px";
            optionContainer.style.marginBottom = "5px";
            this.optionNodes = [];
            var maxStringLength = 0;
            for (var i = 0 ; i < this.options.length; i++) {
                var container = document.createElement("div");
                var checkboxNode = document.createElement("input");
                var textNode = document.createElement("span");
                checkboxNode.type = "checkbox";
                checkboxNode.name = this.options[i].name;
                checkboxNode.data = this.options[i];
				checkboxNode.checked = this.options[i].checked;
                this.optionNodes.push(checkboxNode);
                textNode.innerHTML = this.options[i].name;
                textNode.style.paddingLeft = "5px";
                maxStringLength = Math.max(maxStringLength, this.options[i].name.length);
                container.appendChild(checkboxNode);
                container.appendChild(textNode);
                optionContainer.appendChild(container);
            }
            // roughly set a min width that can contain the longest name string
            this.contentContainer.style.minWidth = (maxStringLength * 6) + "px";
            this.contentContainer.appendChild(optionContainer);
        }
    },

    // Override the "_yesAction" handler to initialize the user selection.
    // 
    _yesAction: function() {
        if (this.options.length > 0) {
            this.selection = [];
            for (var i = 0 ; i < this.optionNodes.length; i++) {
				this.optionNodes[i].data.checked = this.optionNodes[i].checked;
                if (this.optionNodes[i].checked) {
                    this.selection.push(this.optionNodes[i].data);
                }
            }
        }
        
        this.inherited('_yesAction', arguments);
    },

    getSelection: function() {
        return this.selection;
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.ProgressDialog",
             [dijit.Dialog],
{
    // Summary:
    //    Simple dialog to show a modal progress panel.
    //
    postCreate: function() {
        this.inherited('postCreate', arguments);
        
        if(this.titleBar)
            this.titleBar.style.display = "none"; 
        
        this.containerNode.style.border = "none";
        this.containerNode.style.padding = "0px";
        
        var content = document.createElement('div');
        content.className = "progress-panel";
        
        var image = document.createElement('div');
        image.className = "progress-image";
        
        this.messageNode = document.createElement('span');
        this.messageNode.className = "progress-label";
        image.appendChild(this.messageNode);
        
        content.appendChild(image);
        this.containerNode.appendChild(content);
    },
    
    setMessage: function(message)
    {
        this.messageNode.innerHTML = message;
    }
});

})();
