/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// Summary:
//    The base class wrapper for an attribute field in the SectionLayoutForm. All attribute fields
//    must be a subclass of this "abstract" class in the form. The base class provides a simple layout for an 
//    attribute: 
//
//    +-------------------+--------------------------+
//    | attribute Label   |  attribute field         |  
//    +-------------------+--------------------------+
//
//    The concrete subclass will provide a specific field implementation to display and 
//    allow the user to edit the attribute value.
//    
// 
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.AbstractFormField");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

(function() {
dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.AbstractFormField", 
    [dijit._Widget, dijit._Templated],
{
    templateString:'<table tabindex="-1" id="${id}" dojoAttachPoint="domNode" class="section-form-field" tabindex="-1" border="0" cellpadding="0" cellspacing="0" width="100%">' +
                    '  <tr tabindex="-1" dojoAttachPoint="fieldRowNode">' +
                    '    <td tabindex="-1" class="field-label-td">' +
                    '      <div class="field-label" dojoAttachPoint="fieldLabelNode" tabindex="-1">' +
                    '      </div>' +
                    '    </td>' +
                    '    <td align="left" tabindex="-1" class="field-td" dojoAttachPoint="fieldHolderNode">' +
                    '    </td>' +
                    '  </tr>' +
                    '</table>',

    // any data the that the application like to stuff into.
    userData: null,
    
    // the label to be displayed in the attribute Label node.
    fieldLabel: "",
    
    // the dom node for the field. Subclasses should set this baseclass member once
    // it has constructed the field.  
    formField: null,
    
    // field's data value states. 
    isEdited: false,
    isRequired: false,
    hasError: false,
    tooltipMessage: null,
	
	//Modified is a different state than edited.
	//When configs are used within requests, we want to essentially remember is an attribute has been modified since it was originally defined
	//The modified flag indicates that state.
	isModified: false,
    
    postCreate: function() {
        if(this.fieldLabel)
            this.setLabel(this.fieldLabel);
    },

    // Summary:
    //    Set the label string for the attribute field.
    // 
    setLabel: function(label) 
    {
        var required_ind = (this.isRequired) ? " *" : "";
        if (label.lastIndexOf(" :") == (label.length - 2)) {
            label = label.substring(0, label.length - 2) + "&nbsp;:";
        }
        this.fieldLabelNode.innerHTML = label + required_ind;
    },
    
    setLabelIndentLevel: function(indentLevel, factor)
    {
        var indent = indentLevel * factor;
        dojo.style(this.fieldLabelNode, "paddingLeft", indent + "em");
    },
    
    // Summary:
    //    Loads the value into the field. Subclass must override.
    //
    setValue: function(value)
    {
    },
    
    // Summary:
    //    Returns the value from the field. Subclass must override. A null is returned if
    //    no value is specified.
    //
    getValue: function()
    {
        return null;
    },
    
    // Summary:
    //    This method is called when the field's value has been changed. Consumer of the widget 
    //    overrides this method to receive notification of the value changed event.
    //
    onValueChanged: function(fieldWidgetRef, oldValue, newValue)
    {
    },
    
    setHasError: function(hasError)
    {
        this.hasError = hasError;
        var baseName = "field-label";
        var errName = baseName + "-attention"
        
        if (hasError) {
            // we are edited, check if we already have the editled class
            if (this.fieldLabelNode.className.search(errName) == -1) {
                // if no then change from not edited to edited
                this.fieldLabelNode.className = this.fieldLabelNode.className.replace(baseName, errName);
            }
        } 
        else {
            this.fieldLabelNode.className = this.fieldLabelNode.className.replace(errName, baseName);
        }
    },
    
    setErrorMessage: function(message)
    {
    },
    
    getHasError: function()
    {
        return this.hasError;
    },
    
    setEdited: function(isEdited)
    {
        this.isEdited = isEdited;

/*
 *  The dojo form text widget updates the className after all events.  Manipulating
 *  the className would not work anymore.
 *
        var baseName = this._getBaseClassName();
        var editedName = baseName + "-edited";
        
        // This call may be trigger from already edited to still edited
        // which the style for edited is already set.  If blindly replace
        // <baseName> to <baseName>"-edited" you might end up with
        // <baseName>"-edited-edited"
        if (isEdited) {
            // we are edited, check if we already have the editled class
            if (this.formField.className.search(editedName) == -1) {
                // if no then change from not edited to edited
                this.formField.className = this.formField.className.replace(baseName, editedName);
            }
        } 
        else {
            this.formField.className = this.formField.className.replace(editedName, baseName);
        }
*/
    },
    
    getIsEdited: function()
    {
        return this.isEdited;
    },

    setModified: function(isModified)
    {
        this.isModified = isModified;

/*
 *  The dojo form text widget updates the className after all events.  Manipulating
 *  the className would not work anymore.
 *
        var baseName = this._getBaseClassName();
        var editedName = baseName + "-modified";
        
        // This call may be trigger from already edited to still edited
        // which the style for edited is already set.  If blindly replace
        // <baseName> to <baseName>"-edited" you might end up with
        // <baseName>"-edited-edited"
        if (isModified) {
            // we are Modified, check if we already have the editled class
            if (this.formField.className.search(editedName) == -1) {
                // if no then change from not edited to edited
                this.formField.className = this.formField.className.replace(baseName, editedName);
            }
        } 
        else {
            this.formField.className = this.formField.className.replace(editedName, baseName);
        }
*/
    },
    
    getIsModified: function()
    {
        return this.isEdited;
    },
    
    setIsRequired: function(isRequired)
    {
        this.isRequired = isRequired;
        this.setLabel(this.fieldLabel);
    },
    
    getIsRequired: function()
    {
        return this.isRequired;    
    },
    
    // Summary:
    //    Returns the field's set class name. This abstract class doesn't define one.
    //    All concrete form field classes needs to override this method to provide the
    //    appropriate class name string for the field's dom node.
    //
    _getBaseClassName: function()
    {
        return "";
    }
});

})();
