/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    AttributePicker is a simple widget that displays a list of names in a popup
 //    panel that allows the user to make selection. The user can single/multi select 
 //    the list items. The user also can add the selected items without closing down
 //    popup. The picker presents those options in two action buttons.
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.layout.AttributePicker");

dojo.require("dijit.Tooltip");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.AttributePicker",
             [dijit._Widget, dijit._Templated],
{    
    templateString: '<div id="${id}" dojoAttachPoint="domNode" class="com-ibm-asq-common-widget-attribute-picker">' +
                    '  <div dojoAttachPoint="titleBar" class="dijitDialogTitleBar" tabindex="0" waiRole="dialog">' +
                    '    <span dojoAttachPoint="titleNode" class="dijitDialogTitle">${title}</span>' +
                    '    <span dojoAttachPoint="closeButtonNode" class="dijitDialogCloseIcon" dojoAttachEvent="onclick: onClose"></span>' + 
                    '  </div>' +
                    '  <div class="body" dojoAttachPoint="bodyNode">' +
                    '    <div dojoAttachPoint="promptNode" class="prompt">' +
                    '    </div>' +
                    '    <label>' +
					'     <select dojoAttachPoint="attrSelectNode" class="optionSelect" multiple="true" size="14" dojoAttachEvent="onchange: _onSelection">' +
                    '     </select>' +
					'    </label>' +
                    '  </div>' +
                    '  <div class="footer" dojoAttachPoint="footerNode">' +
                    '    <button dojoAttachPoint="addCloseButtonNode" name="add_and_close" disabled="true" dojoAttachEvent="onclick:_addCloseClicked" >${addAndCloseLabel} ' +
                    '    </button>' +
                    '    <button dojoAttachPoint="addButtonNode" name="add" disabled="true" dojoAttachEvent="onclick:_addClicked" >${addLabel} ' +
                    '    </button>' +
                    '  </div>' +
                    '</div>',
    
    widgetsInTemplate: true,
    addAndCloseLabel: MessageUtil.getMessage(MessageBundle.COMMON, "add_close_button"),
    addLabel: MessageUtil.getMessage(MessageBundle.COMMON, "add_button"),
    closeLabel: MessageUtil.getMessage(MessageBundle.COMMON, "close_button"),
    title: MessageUtil.getMessage(MessageBundle.COMMON, "select_attribute"),
    promptText: "",

    // specifies the list of names to be shown in the option list. This widget supports
    // both a single array of names, or an array of object with name/value pair attributes.
    attributeItems: null,
    
    postCreate: function() 
    {
        this.promptNode.innerHTML = this.promptText;
        this.refreshList();
    },
    
    // Summary:
    //    Notification of the add button being clicked. Client can override this method
    //    to receive add button clicked events. The list of selected items are passed in
    //    the argument.
    //
    onAdd: function(selectedItems)
    {    
    },
    
    // Summary:
    //    Notification of the close dialog event. Client can override this method
    //    to receive close events. 
    //
    onClose: function()
    {
    },
        
    getSelectedItems: function()
    {
        var selected = [];
        for(var i = 0; i < this.attrSelectNode.options.length; i++) {
            if(this.attrSelectNode.options[i].selected)
                selected.push(this.attrSelectNode.options[i].value);
        }
        return selected;
    },

    // Summary:
    //    Regenerate the selection list of the picker. 
    //
    refreshList: function()
    {
        this.attrSelectNode.length = 0;

        for(var i = 0; i < this.attributeItems.length; i++) {
            var attr = this.attributeItems[i];
            var name, value;
            
            if(dojo.isObject(attr)) {
                name = attr.name;
                value = attr.value;
            }
            else {
                name = value = attr;
            }
            this.attrSelectNode.options[i] = new Option(name, value);
        }
        this.addButtonNode.disabled = true;
        this.addCloseButtonNode.disabled = true;
    },
    
    // Summary:
    //    Internal method to handle the add button clicked.
    //
    _addClicked: function(e)
    {
        var selectedItems = this.getSelectedItems();
        this.onAdd(selectedItems);
        if (this.attrSelectNode.options.length == 0)
            this.onClose();
    },

    // Summary:
    //    Internal method to handle the add and close button clicked.
    //
    _addCloseClicked: function(e)
    {
        this._addClicked(e);
        this.onClose();
    },
    
    // Summary:
    //    Internal method to list selection events.
    //
    _onSelection: function()
    {
        this.addButtonNode.disabled = false;
        this.addCloseButtonNode.disabled = false;
    }
});

})();
