/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.layout.FormToggleListField");

dojo.require("dijit.Menu");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.CheckBox");

dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField");

(function() {

var WordWrappingUtil = com.ibm.asq.common.web.ui.internal.util.wordWrapping;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.FormToggleListField", 
    com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField,
{
    // Summary:
    //   A form field with toggle list dropdown menu.  The array of options is in the form of
    //   { name: <display name>, value: <unique value>, checked: <boolean> }
    //
    options: [],
    value: null,
    displayValue: MessageUtil.getMessage(MessageBundle.COMMON, "labelAny"),
    
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.toggleField = document.createElement("div");
        this.fieldHolderNode.appendChild(this.toggleField);
        this.formField = this.toggleField;
        
        // we're going to use the options object to store our toggle
        // states. This is client data, make a copy of it first.
        if(this.options) {
            this.options = dojo.clone(this.options);
        }
        if (this.options.length > 0) {
            var menu = new dijit.Menu({
                'class': "asq-section-form-field-toggle-menu"
            });
            menu.domNode.style.display="none";
            
            for (var i = 0; i < this.options.length; i++) {
                // create toggle menu items
                var toggleMenuItem = new com.ibm.asq.common.web.ui.internal.widgets.layout.toggleMenuItem({
                    checked: this.options[i].checked,
                    label: this.options[i].name,
                    toggleClickCallback: dojo.hitch(this, "_menuItemToggle"),
                    toggleClickCallbackData: { index: i }
                });
                menu.addChild(toggleMenuItem);
            }

            this._updateDisplayValue();
            var params = {
                label: this.displayValue,
                dropDown: menu
            };
            this.toggleFilter = new com.ibm.asq.common.web.ui.internal.widgets.layout.DropDownButton(params);
            this.toggleField.appendChild(this.toggleFilter.domNode);
            this.toggleField.title = this.displayValue;
        } else {
            console.error("options array not specified");
        }
    },
    
    _menuItemToggle: function(args) {
        this.options[args.index].checked = args.checkbox ? args.checkbox.checked : false;
        this._updateDisplayValue();
        this.toggleFilter.setLabel(this.displayValue);
        this.toggleField.title = this.displayValue;
        this.changed = true;
    },
    
    setValue: function(value) {
        // The checked values are in the options and manipulated through
        // the dropdown menu, not set directly.
    },

    getValue: function() {
        if (this.changed) {
            // convert the array of checked associative array to array of value only
            if (this.options && (this.options.length > 0)) {
                this.value = [];
                for (var i = 0; i < this.options.length; i++) {
                    if (this.options[i].checked) {
                        this.value.push(this.options[i].value);
                    }
                }
            } else {
                this.value = null;
            }
            this.changed = false;
        }
        return this.value;
    },

    _updateDisplayValue: function() {
        this.displayValue = "";
        if (this.options) {
            this.value = [];
            for (var i = 0; i < this.options.length; i++) {
                if (this.options[i].checked) {
                    this.value.push(this.options[i].value);
                    if (this.displayValue == "") {
                        this.displayValue = this.options[i].name;
                    } else {
                        this.displayValue = MessageUtil.getMessage(MessageBundle.COMMON, "labelOrValues", this.displayValue, this.options[i].name);
                    }
                }
            }
            this.changed = false;
        }
        if (this.displayValue == "") {
            this.value = null;
            this.displayValue = MessageUtil.getMessage(MessageBundle.COMMON, "labelAny");
        }
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.DropDownButton", dijit.form.DropDownButton, {
    // Summary:
    //   Simplified dropdown button that conform to the abstract form field style.
    //
    templateString: '<div class="asq-section-form-field toggle-field" ' +
                    '     dojoAttachEvent="onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse,onclick:_onDropDownClick,' +
                    '     onkeydown:_onDropDownKeydown,onblur:_onDropDownBlur,onkeypress:_onKey">' +
                    '  <div class="toggle-button" dojoAttachPoint="focusNode,titleNode" waiRole="button" waiState="haspopup-true,labelledby-${id}_label">' +
                    '    <div class="toggle-edit-closed" dojoAttachPoint="iconNode">' +
                    '      <div class="dijitButtonText toggle-button-label" dojoAttachPoint="containerNode,popupStateNode"' +
                    '           id="${id}_label">${label}</div>' +
                    '    </div>' +
                    '  </div>' +
                    '</div>',

    postCreate: function() {
        this.inherited("postCreate", arguments);
        // setLabel() will trim the label to fit in the button but
        // it must be done after the widget is created and added to
        // the DOM tree in order to determine the button width
        setTimeout(dojo.hitch(this, "setLabel", this.label), 5);
    },
    
    _openDropDown: function() {
        this.inherited("_openDropDown", arguments);
        this.iconNode.className = "toggle-edit-opened";
        
        // the drop down of the button is a table which is a child of the popup DIV
        var popup = this.dropDown.domNode.parentNode;
        var popupMB = dojo.marginBox(popup);

        // determine the top of the popup that is relative to the visible window
        var popupWindowTop = popupMB.t - document.documentElement.scrollTop;

        // remove the borders on the table to use borders from the popup DIV instead
        dojo.addClass(popup, "dijitMenu");
        this.dropDown.domNode.style.border = "none";

        // check if the popup DIV is too long for the visible height that needs to be trimmed
        // and put up a scrollbar
        if((popupMB.h + popupWindowTop) > document.documentElement.clientHeight) {
            popup.style.height = (document.documentElement.clientHeight - popupWindowTop - 10) + "px";
            popup.style.overflowX = "hidden";
            popup.style.overflowY = "scroll";
        }
    },
    
    _closeDropDown: function(focus) {
        this.inherited("_closeDropDown", arguments);
        this.iconNode.className = "toggle-edit-closed";
    },

    setLabel: function(label) {
        label = this.trimLabel(label);
        com.ibm.asq.common.web.ui.internal.widgets.layout.DropDownButton.superclass.setLabel.apply(this, [ label ]);
    },
    
    trimLabel: function(label) {
        // In case clientWidth is not returned correctly (in IE) use scrollWidth which is
        // fine because overflow is hidden anyway.
        var buttonWidth = this.containerNode.clientWidth || this.containerNode.scrollWidth;
        if (buttonWidth > 30) {
            // there is about 30 pixels of margin and icon in the button
            label = WordWrappingUtil.wrapText(label, this.containerNode.className, buttonWidth - 30, true, "@@@");
            var labelBreak = label.indexOf("@@@");
            if (labelBreak > -1) {
                label = label.substring(0, labelBreak) + "...";
            }
        }
        return label;
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.toggleMenuItem", dijit.MenuItem, {
    // Summary:
    //   A check box menu item.
    //
    templateString: '<tr class="dijitReset dijitMenuItem" dojoAttachEvent="onmouseenter:_onHover,onmouseleave:_onUnhover">' +
                    '  <td dojoAttachPoint="containerNode" waiRole="menuitem">' +
                    '    <input type="checkbox" id="${id}_checkbox" dojoAttachPoint="checkboxNode" class="toggle-checkbox">' +
                    '    <label for="${id}_checkbox" class="toggle-checkbox-label">${label}</label>' +
                    '  </td>' +
                    '</tr>',

    postCreate: function() {
        this.checkboxNode.checked = this.checked;
        this.checkboxNode.onclick = dojo.hitch(this, "_onToggleClick");

        if (this.toggleClickCallbackData) {
            this.toggleClickCallbackData.checkbox = this.checkboxNode;
        } else {
            this.toggleClickCallbackData = { checkbox: this.checkboxNode };
        }
    },
    
    _onToggleClick: function() {
        if (dojo.isFunction(this.toggleClickCallback)) {
            this.toggleClickCallback(this.toggleClickCallbackData);
        }
    }
});

})();
