/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 //
 // Summary:
 //    RichTextContent provides a convenience wrapper widget around an instance
 //     of RichTextEditor.  It provides generic APIs for manipulating the editor's content, 
 //     and provides default style options for the editor's look & feel.
 //     
 //       
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextControl");

dojo.require("dojox.data.dom");
dojo.require("dijit.form.Button");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextEditor");

//must require extra plugins
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.RichTablePlugin");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.InsertImagePlugin");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.InsertAttachPlugin");

dojo.require("com.ibm.asq.common.web.client.internal.Constants");

( function() {

var Constants = com.ibm.asq.common.web.client.internal.Constants;
var RichTextEditor = com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextEditor;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextControl", [dijit._Widget, dijit._Templated], {
    templateString: '<div class="com-ibm-asq-common-richtext-content">' +
                    '   <div dojoAttachPoint="_actionsNode" class="actions">' +
                    '      <div dojoAttachPoint="_previewActionNode" class="action"></div>' +
                    '      <div dojoAttachPoint="_editActionNode" class="action"></div>' + 									
                    '   </div>' +    
					'   <div class="clear"></div>' +
                    '   <div dojoAttachPoint=\"_editorContainerNode\" class=\"editor-content hidden\"></div>' + 
					'   <div dojoAttachPoint=\"_viewerContainerNode\" class=\"content hidden\" dojoAttachEvent="onclick: _doEdit"></div>' + 
                    '   <div dojoAttachPoint=\"_emptyMessageNode\" class=\"content-empty hidden\" dojoAttachEvent="onclick: _doEdit">${labelAddContent}</div>' +                  
					'</div>',
	
	labelAddContent: Constants.getMessage("labelAddContent"),
	minHeight: "10em", 
	
    constructor: function(){      
		//optional parameters
		this.contentHTML = null;
		this.editable = true;
        this.inEditMode = false;  
		
		//extra plugins - by default, include image & table support 
		//but allow optional disabling of both via constructor arguments
		if (arguments[0] && !arguments[0].extraPlugins) {
			this.extraPlugins = [];
            if (arguments[0] && !arguments[0].disableTableSupport) {
                this.extraPlugins.push(InsertImagePlugin.PLUGIN_ID);
            }
			if (arguments[0] && !arguments[0].disableImageSupport) {
				this.extraPlugins.push(RichTablePlugin.PLUGIN_ID);
			}
			if (arguments[0] && !arguments[0].disableAttachSupport) {
				this.extraPlugins.push(InsertAttachPlugin.PLUGIN_ID);
			}
		}
        //internal
        this._editor = null;			     		
    },
	
    postCreate: function(args){
        //show/hide edit mode links
        //this._toggleLinkNode.innerHTML = this.inEditMode ? "Preview" : "";		
        if (this.editable) {
            dojo.removeClass(this._actionsNode, "hidden");
            this._createActions();
		} else {
            this.inEditMode = false;
            dojo.addClass(this._actionsNode, "hidden");
        }
		
        //default height, overridable on widget creation
        if (this.minHeight) {
            this._editorContainerNode.style.minHeight = this.minHeight;
            this._viewerContainerNode.style.minHeight = this.minHeight;
            this._emptyMessageNode.style.minHeight = this.minHeight;
        }		
		
		dojo.subscribe("RQM-Event-Back-From-Print-Preview", this, "_handleReturnFromPrintPreview");
    },
	
	_createActions: function() {
		this._previewButton = new dijit.form.Button({label: Constants.getMessage("labelPreview"), showLabel: true, onClick: dojo.hitch(this, "_doPreview")});			
		this._editButton = new dijit.form.Button({label: Constants.getMessage("labelEdit"), showLabel: true, onClick: dojo.hitch(this, "_doEdit")});		
		
		this._previewActionNode.appendChild(this._previewButton.domNode);
		this._editActionNode.appendChild(this._editButton.domNode);
	},
	
	_setActionStates: function() {
		if (this.inEditMode)
		{
			dojo.removeClass(this._previewActionNode, "hidden");
			dojo.addClass(this._editActionNode, "hidden");
		}
		else
		{
			dojo.addClass(this._previewActionNode, "hidden");
			dojo.removeClass(this._editActionNode, "hidden");			
		}
	},
    
    //////////////////////////////////////////////////////////////////////////////////////
    //////    API 
    //////////////////////////////////////////////////////////////////////////////////////  
    show: function(){    
        //create the editor if necessary   
        if (this.inEditMode) {    
            //create the editor
            //this._editor = this._editor ? this._editor : this._createEditor();
			this.contentHTML = this.getContentHTML();
			this._destroyEditor();				
			this._editor = this._createEditor();
			this._hideNode(this._viewerContainerNode);
			this._showNode(this._editorContainerNode);            
        } else {
			this._viewerContainerNode.innerHTML = this.contentHTML;
			this._hideNode(this._editorContainerNode);
			this._showNode(this._viewerContainerNode);			
		}        
        if (this.editable) {
            this._setActionStates();
        }
        this._showEmptyMessage();
    },
    
    hide: function() {
        //hide the domNodes
		this._hideNode(this._editorContainerNode);
		this._hideNode(this._viewerContainerNode);		      
    },
	
	_handleReturnFromPrintPreview: function(){
		if (this.inEditMode == true) {
			this.contentHTML = this.getContentHTML();
			//this._editor = this._editor ? this._editor : this._createEditor();
			this._destroyEditor();
			this._editor = this._createEditor();
			this._hideNode(this._viewerContainerNode);
			this._showNode(this._editorContainerNode);
		}	
	},
	
	_doEdit: function() {
        //begin editing the content
		if (this.editable) {
			this.contentHTML = this.getContentHTML();
			//this._editor = this._editor ? this._editor : this._createEditor();
			this._destroyEditor();
			this._editor = this._createEditor();
			this._hideNode(this._viewerContainerNode);
			this._showNode(this._editorContainerNode);
			//this._editLinkNode - grey out the link
		}	
		this.inEditMode = true;
		this._setActionStates();	
		this._showEmptyMessage();
		if (dojo.isFF == 2 && this.editable) {
			try{
				this._editor.setDisabled(false);
			}catch(e){
			}
		}
	},
	
	_doPreview: function() {
        //perview mode
		this.contentHTML = this.getContentHTML();
		this._hideNode(this._editorContainerNode);
        this._showNode(this._viewerContainerNode);
        this._viewerContainerNode.innerHTML  = this.contentHTML;
		this._filterAllAElement(this._viewerContainerNode);
		//this._previewLinkNode - grey out the link		
		this.inEditMode = false;
		this._setActionStates();
		this._showEmptyMessage();
	},
		
	toggleEditMode: function() {
        if (this.editable) {
            this.contentHTML = this.getContentHTML();
            if (this.inEditMode) {
                this._hideNode(this._editorContainerNode);
                this._showNode(this._viewerContainerNode);
                this._viewerContainerNode.innerHTML = this.contentHTML;
            }
            else {
                this._editor = this._editor ? this._editor : this._createEditor();
                this._hideNode(this._viewerContainerNode);
                this._showNode(this._editorContainerNode);
            }
            this.inEditMode = !this.inEditMode;
        } else {
            alert(Contants.getMessage("alertRichTextNotEditable"));
        }
		this._setActionStates();
        this._showEmptyMessage();
	},   
		
	getContentHTML: function() {
		if (this.inEditMode) {
            var tmpValue = this._filterUselessCSS(this._filterUselessATag(this._editor ? this._editor.getValue() : ""));
			return this._postContentFilter(tmpValue);
		} else {
            return this._viewerContainerNode.innerHTML;		  
		}		
	},
	
	setContentHTML: function(contentHTML) {
		this.contentHTML = contentHTML;
		if (this.inEditMode) {			
			//this._editor.setValue(this.contentHTML);  //TODO this doesn't work, figure out why!
			this._destroyEditor();
			this._editor = this._createEditor();
		} else {
			this._viewerContainerNode.innerHTML = this.contentHTML;
			//If in preview mode currently but the editor has been created (ie user toggled back and forth from editing to preview)
			//Set the editor content as well.  If we don't do this, the editor will display whatever content it previously had
			if (this._editor)
				this._editor.setValue(this.contentHTML);
		}
		this._showEmptyMessage();
	},

    //////////////////////////////////////////////////////////////////////////////////////
    //////    Internal 
    //////////////////////////////////////////////////////////////////////////////////////  		
    _hideNode: function(node) {
        if (!dojo.hasClass(node, "hidden")) {
            dojo.addClass(node, "hidden")
        }		
	},
	
    _showNode: function(node) {
        if (dojo.hasClass(node, "hidden")) {
            dojo.removeClass(node, "hidden")
        }       
    },

	//create the RichTextEditor
	_createEditor: function() {             
        //set up the dom node
        dojox.data.dom.removeChildren(this._editorContainerNode);   		
		this._editorNode = document.createElement("div");
		if (this.contentHTML) {
			this._editorNode.innerHTML = this.contentHTML;		
        }         
		this._editorContainerNode.appendChild(this._editorNode);	

        //create the editor
		var editorArgs = {
			extraPlugins: this.extraPlugins ? this.extraPlugins : null
		}
        var richTextEditor =  new RichTextEditor(editorArgs, dojo.byId(this._editorNode));
		dojo.connect(richTextEditor, "onChange", this, "onChange");
		return richTextEditor;
	},
    
    _showEmptyMessage: function() {
        if (this.inEditMode || (this.contentHTML != null && dojo.trim(this.contentHTML) != "")) {
            dojo.addClass(this._emptyMessageNode, "hidden");
			if (!this.inEditMode) { //If not in edit mode then show the content
                dojo.removeClass(this._viewerContainerNode, "hidden");
            }
        } else {
            dojo.removeClass(this._emptyMessageNode, "hidden");
            dojo.addClass(this._viewerContainerNode, "hidden");
            if (!this.editable) {
                this._emptyMessageNode.innerHTML = "";
            }
        }        
    },
	
    _destroyEditor: function() {
        if (this._editor) {
            this._editor.destroy();
            this._editor = null;
        }	   	
	},
	
	/*
     * The rich text editor does not properly clear its content when a user removes all content.  So,
     * useless HTML will remain when  a user clears the content manually.  To get around this, apply
     * this filter - will set the content to empty if:
     *           (1) it does not contain any text AND (2) it does not contain any img or table tags
     *
     */
     _postContentFilter: function(content){
		var upper = content.toUpperCase();
		var node = document.createElement("div");
        node.innerHTML = content;
		if (dojo.isIE) {
			//IE can't deal with dojox.data.dom.textContent perfectly if conteont is too large, 
			//won't use it in IE
			if(upper.lastIndexOf("IMG") != -1 ||
				upper.lastIndexOf("TABLE") != -1)
				return content;
			if(this._isEmptyNode(node)  &&
				upper.lastIndexOf("IMG") == -1 &&
				upper.lastIndexOf("TABLE") == -1){
				return "";
			}
			return content;
		}
		else {
			var text = dojox.data.dom.textContent(node);
			if ((text == null || dojo.trim(text).length == 0) &&
			upper.lastIndexOf("IMG") == -1 &&
			upper.lastIndexOf("TABLE") == -1) {
				return "";
			}
			return content;
		}
    },
	
	/*
	 * Implementation of the DOM Level 3 attribute; scan node for text description. 
	 * Reference dojox.data.dom.textContent. Fix wi 7343
	 */	
	_isEmptyNode: function(/*Node*/node) {
 		if(node == null){
			return true;
		}
		for(var i = 0; i < node.childNodes.length; i++){
			switch(node.childNodes[i].nodeType){
				case 1: // ELEMENT_NODE
				case 5: // ENTITY_REFERENCE_NODE
					if(!this._isEmptyNode(node.childNodes[i]))
						return false;
				case 3: // TEXT_NODE
				case 2: // ATTRIBUTE_NODE
				case 4: // CDATA_SECTION_NODE
					//HACK!! Doesn't work well in IE7
            		var text = node.childNodes[i].nodeValue;
					if (text == null || dojo.trim(text).length == 0)
			 			return true;
			 		return false;
				default:
					return true;
			}
		}
		return true;
	},	
	
	/*
	 * Remove useless a tag, such as: <a name="_Toc123"/>
	 */
	_filterUselessATag: function(oldValue) {
		var regrep = new RegExp("<a[^<]+?/>", "g");
		var newValue = oldValue.replace(regrep, "");
		return newValue;
	},
	
	/*
	 * Remove useless CSS, fix defect 11753
	 */
	_filterUselessCSS: function(oldValue) {
		var newValue = oldValue.replace(/a:link[^}]*}/, "").replace(/a:visited[^}]*}/, "");
		return newValue;
	},
	
	/*
	 * If <a> element has no href attribute, set the text color to black.
	 * Fix the wi 7341
	 */
	_filterAllAElement: function(node) {
		var children = node.getElementsByTagName("a");
		for(var i=0; i< (children ? children.length : 0); i++) {
			var child = children[i];
			if(!child.href){
				child.style.color = "#000";
			}
		}
		
	},
	
	onChange: function(){
		console.debug("editor dirty");
	}
	
});

})();
