/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008,2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//
// Summary:
//    RichTextEditor extends dojo's dijit.Editor widget to provide generic enhancements 
//     for rich text editing across the QM application.  If there are component-specific 
//     enhancements required, those components should subclass RichTextEditor.
//       
//
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextEditor");

dojo.require("dojox.data.dom");
dojo.require("dijit.Editor");

//must require the editor plugins to be used...
dojo.require("dijit._editor.plugins.AlwaysShowToolbar");
dojo.require("dijit._editor.plugins.TextColor");
dojo.require("dijit._editor.plugins.FontChoice");
dojo.require("dijit._editor.plugins.LinkDialog");

(function(){

    dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextEditor", [dijit.Editor], {
    
        //default list of Editor toolbar plugins
        //	defaultPlugins: ['foreColor', 'hiliteColor', 'fontSize', 'dijit._editor.plugins.AlwaysShowToolbar'],
        
        //@Overridden from dijit.Editor
        constructor: function(){
            this.defaultPlugins = ['fontSize', 'fontName', 'foreColor', 'hiliteColor', 'dijit._editor.plugins.AlwaysShowToolbar'];
            
            //provide default plugins if none specified
            if (!arguments[0].extraPlugins) {
                arguments[0].extraPlugins = this.defaultPlugins;
            }
            else {
                //prepend default plugins to extraPlugins param
                var plugins = this.defaultPlugins;
                for (var i = 0; i < arguments[0].extraPlugins.length; i++) {
                    if (!this._contains(plugins, arguments[0].extraPlugins[i])) {
                        plugins.push(arguments[0].extraPlugins[i]);
                    }
                }
                arguments[0].extraPlugins = plugins;
            }
            com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextEditor.superclass.constructor.apply(this, arguments);
        },
        
        //@Overridden from dijit.Editor
        postCreate: function(args){
            com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextEditor.superclass.postCreate.apply(this, arguments);
//            this.contentPostFilters.push(this._postContentFilter);
        },
        
        //@Overridden from dijit.Editor
        //reduce the width of Font Size plugin
        addPlugin: function(/*String||Object*/plugin, /*Integer?*/ index){
            com.ibm.asq.common.web.ui.internal.widgets.richtext.RichTextEditor.superclass.addPlugin.apply(this, arguments);
            if (plugin == "fontSize") {
                var currentPlugin = this._plugins[index];
                currentPlugin.button.domNode.style.width = "80px";
            }
            if (plugin == "fontName") {
                var currentPlugin = this._plugins[index];
                currentPlugin.button.domNode.style.width = "80px";
            }
			if(this._plugins[index].button){
				//FIX defect 4697. 
				//@Overridden: setDisabled function from dijit.form._FormWidget. 
				//When the domNode is disabled, the title should appear when the mouse is over the button. 
				this._plugins[index].button.setDisabled = function(/*Boolean*/ disabled){
					// summary:
					//		Set disabled state of widget.
	
					this.domNode.disabled = this.disabled = disabled;
					if(this.focusNode){
					//Fix: remove this line. 
			//			this.focusNode.disabled = disabled;
					}
					if(disabled){
						//reset those, because after the domNode is disabled, we can no longer receive
						//mouse related events, see #4200
						this._hovering = false;
						this._active = false;
					}
					dijit.setWaiState(this.focusNode || this.domNode, "disabled", disabled);
					this._setStateClass();
				};
			}
        },
        
        //@Overridden from dijit._editor.RichText
        //fix the bug of this.document.queryCommandValue("fontsize") in FF3
        queryCommandValue: function(command){
            // summary: check the value of a given command
            if (this.disabled) {
                return false;
            }
            command = this._normalizeCommand(command);
            if (dojo.isIE && command == "formatblock") {
                return this._local2NativeFormatNames[this.document.queryCommandValue(command)];
            }
            if (dojo.isFF == 3 && (command == "fontsize" || command == "fontname")) {
                try {
                    return this.document.queryCommandValue(command);
                } 
                catch (e) {
                    return "";
                }
                
            }
            return this.document.queryCommandValue(command);
        },
		
		//@Overridden from dijit._editor.RichText
		//FIX 6963: Remove the style "p{ margin: 1em 0 !important; }" to fix the truncation on the left margin area.
		_getIframeDocTxt: function(/* String */ html){
			var _cs = dojo.getComputedStyle(this.domNode);
			if(!this.height && !dojo.isMoz){
				html="<div>"+html+"</div>";
			}
			var font = [ _cs.fontWeight, _cs.fontSize, _cs.fontFamily ].join(" ");
	
			var lineHeight = _cs.lineHeight;
			if(lineHeight.indexOf("px") >= 0){
				lineHeight = parseFloat(lineHeight)/parseFloat(_cs.fontSize);
			}else if(lineHeight.indexOf("em")>=0){
				lineHeight = parseFloat(lineHeight);
			}else{
				lineHeight = "1.0";
			}
			return [
				this.isLeftToRight() ? "<html><head>" : "<html dir='rtl'><head>",
				(dojo.isMoz ? "<title>" + this._localizedIframeTitles.iframeEditTitle + "</title>" : ""),
				"<style>",
				"body,html {",
				"background:transparent;",
				"padding: 0;",
				"margin: 0;",
				"height: 100%;",
				"}",
				"body{",
				"top:0px; left:0px; right:0px;",
					((this.height||dojo.isOpera) ? "" : "position: fixed;"),
				"font:", font, ";",
				"min-height:", this.minHeight, ";",
				"line-height:", lineHeight,
				"}",
//				"p{ margin: 1em 0 !important; }",
				(this.height ?
					"" : "body,html{overflow-y:hidden;/*for IE*/} body > div {overflow-x:auto;/*for FF to show vertical scrollbar*/}"
				),
				"li > ul:-moz-first-node, li > ol:-moz-first-node{ padding-top: 1.2em; } ",
				"li{ min-height:1.2em; }",
				"</style>",
				this._applyEditingAreaStyleSheets(),
				"</head><body>"+html+"</body></html>"
			].join(""); // String
		},
        
		//@Overridden from dijit._editor.RichText
        //fix the bug of pasting formatted text from Word into rich text editor causes error:
		//"Warning: unresponsive script" 
        _postFilterContent: function(/*DomNode|DomNode[]|String?*/dom,/*Boolean?*/ nonDestructive){
            // summary:
            //		filter the output after getting the content of the editing area
            var ec;
            if (!dojo.isString(dom)) {
                dom = dom || this.editNode;
                if (this.contentDomPostFilters.length) {
                    if (nonDestructive && dom['cloneNode']) {
                        dom = dom.cloneNode(true);
                    }
                    dojo.forEach(this.contentDomPostFilters, function(ef){
                        dom = ef(dom);
                    });
                }
                ec = this.getChildrenHtml(dom);
            }
            else {
                ec = dom;
            }
			
			//separate \s|\xA0:  
			//hack: don't know why it doesn't work if using /^(?:\s|\xA0)+/g
            if (!ec.replace(/^(?:\s)+/g, "").replace(/(?:\s)+$/g, "").length) {
                ec = "";
            }
            if (!ec.replace(/^(?:\xA0)+/g, "").replace(/(?:\xA0)+$/g, "").length) {
                ec = "";
            }
            
            dojo.forEach(this.contentPostFilters, function(ef){
                ec = ef(ec);
            });
            
            return ec;
        },
        
        /*
         * The rich text editor does not properly clear its content when a user removes all content.  So,
         * useless HTML will remain when  a user clears the content manually.  To get around this, apply
         * this filter - will set the content to empty if:
         *           (1) it does not contain any text AND (2) it does not contain any img or table tags
         *
         */
        _postContentFilter: function(content){
            var node = document.createElement("div");
            node.innerHTML = content;
            
            var text = dojox.data.dom.textContent(node);
            var upper = content.toUpperCase();
            if ((text == null || dojo.trim(text).length == 0) &&
            upper.lastIndexOf("IMG") == -1 &&
            upper.lastIndexOf("TABLE") == -1) {
                return "";
            }
            return content;
        },
        
        onKeyUp: function(e){
            this.inherited("onKeyUp", arguments);
            
            //For defect 7851
			var k=e.keyCode,ks=dojo.keys;
			if(e.ctrlKey && !e.altKey){//undo and redo only if the special right Alt + z/y are not pressed IBM#79329
				if(k===90||k===122){ //z
					dojo.stopEvent(e);
					this.onChange();
					return;
				}else if(k===89||k===121){ //y
					dojo.stopEvent(e);
					this.onChange();
					return;
				}
			}

			switch(k){
				case ks.ENTER:
					this.onChange();
					break;
				case ks.BACKSPACE:
				case ks.DELETE:
					this.onChange();
					break;
				case 88: //x
				case 86: //v
					if(e.ctrlKey && !e.altKey && !e.metaKey){
						this.onChange();
						break;
					}
					//pass through
				default:
					if(!e.ctrlKey && !e.altKey && !e.metaKey && (e.keyCode<dojo.keys.F1 || e.keyCode>dojo.keys.F15)){
						this.onChange();
						break;
					}
					//pass through
				case ks.ALT:
					break;
				case ks.UP_ARROW:
				case ks.DOWN_ARROW:
				case ks.LEFT_ARROW:
				case ks.RIGHT_ARROW:
				case ks.HOME:
				case ks.END:
				case ks.PAGE_UP:
				case ks.PAGE_DOWN:
					break;
				//maybe ctrl+backspace/delete, so don't set dirty when ctrl is pressed
				case ks.CTRL:
				case ks.SHIFT:
				case ks.TAB:
					break;
			}
        },
        
        execCommand: function(/*String*/command, argument){
            this.inherited("execCommand", arguments);
			if((command.toLowerCase() == "hilitecolor")&&(dojo.isMoz)){
				this.document.execCommand("styleWithCSS", false, true);
			}
			this._onBlur();
        },

        _contains: function(array, value){
            for (var i = 0; i < (array ? array.length : 0); i++) {
                if (value == array[i]) {
                    return true;
                }
            }
            return false;
        },
        
		//copy from dijit._editor.html
        escapeXml: function(/*String*/str, /*Boolean*/ noSingleQuotes){
            //summary:
            //		Adds escape sequences for special characters in XML: &<>"'
            //		Optionally skips escapes for single quotes
            str = str.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;");
            if (!noSingleQuotes) {
                str = str.replace(/'/gm, "&#39;");
            }
            return str; // string
        },
        
		//copy from dijit._editor.html
        getNodeHtml: function(/* DomNode */node){
            var output;
            switch (node.nodeType) {
                case 1: //element node
                    output = '<' + node.nodeName.toLowerCase();
                    
                    //store the list of attributes and sort it to have the
                    //attributes appear in the dictionary order
                    var attrarray = [];
                    if (dojo.isIE && node.outerHTML) {
                        var s = node.outerHTML;
                        s = s.substr(0, s.indexOf('>'));
                        s = s.replace(/(['"])[^"']*\1/g, '');//to make the following regexp safe
                        var reg = /([^\s=]+)=/g;
                        var m, key;
                        while ((m = reg.exec(s))) {
                            key = m[1];
                            if (key.substr(0, 3) != '_dj') {
                                if (key == 'src' || key == 'href') {
                                    if (node.getAttribute('_djrealurl')) {
                                        attrarray.push([key, node.getAttribute('_djrealurl')]);
                                        continue;
                                    }
                                }
                                if (key == 'style') {
                                    attrarray.push([key, node.style.cssText.toLowerCase()]);
                                }
                                else {
                                    attrarray.push([key, key == 'class' ? node.className : node.getAttribute(key)]);
                                }
                            }
                        }
                    }
                    else {
                        var attr, i = 0, attrs = node.attributes;
                        while ((attr = attrs[i++])) {
                            //ignore all attributes starting with _dj which are
                            //internal temporary attributes used by the editor
                            var n = attr.name;
                            if (n.substr(0, 3) != '_dj'                        /*&&
                         (attr.specified == undefined || attr.specified)*/
                            ) {
                                var v = attr.value;
                                if (n == 'src' || n == 'href') {
                                    if (node.getAttribute('_djrealurl')) {
                                        v = node.getAttribute('_djrealurl');
                                    }
                                }
                                if (v != '') 
                                    attrarray.push([n, v]);
                            }
                        }
                    }
                    attrarray.sort(function(a, b){
                        return a[0] < b[0] ? -1 : (a[0] == b[0] ? 0 : 1);
                    });
                    i = 0;
                    while ((attr = attrarray[i++])) {
                        output += ' ' + attr[0] + '="' + this.escapeXml(attr[1], true) + '"';
                    }
                    if (node.childNodes.length) {
                        output += '>' + this.getChildrenHtml(node) + '</' + node.nodeName.toLowerCase() + '>';
                    }
                    else {
                        output += ' />';
                    }
                    break;
                case 3: //text
                    // FIXME:
                    output = this.escapeXml(node.nodeValue, true);
                    break;
                case 8: //comment
                    // FIXME:
                    output = '<!--' + this.escapeXml(node.nodeValue, true) + '-->';
                    break;
                default:
                    output = "Element not recognized - Type: " + node.nodeType + " Name: " + node.nodeName;
            }
            return output;
        },
        
		//copy from dijit._editor.html
        getChildrenHtml: function(/* DomNode */dom){
            // summary: Returns the html content of a DomNode and children
            var out = "";
            if (!dom) {
                return out;
            }
            var nodes = dom["childNodes"] || dom;
            var i = 0;
            var node;
            while ((node = nodes[i++])) {
                out += this.getNodeHtml(node);
            }
            return out; // String
        },
		
		//@Overridden from dijit._editor.RichText
		onChange: function(newContent){
			// summary:
			//		this is fired if and only if the editor loses focus and
			//		the content is changed
	
			//console.debug("Change in " + this.declaredClass);
		}
    });
    
})();
