/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.InsertImageDialog");
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.InsertImageDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("dojo.io.iframe");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");

dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

(function(){ // module container
    var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
    var Constants = com.ibm.asq.common.web.client.internal.Constants;
    var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
    var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
    var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
    var isGuest = com.ibm.team.repository.web.client.session.isGuest;
    var UPLOAD_SERVICE_URI = "com.ibm.rqm.planning.service.internal.rest.IAttachmentRestService/";
    var DOWNLOAD_SERVICE_URI = "com.ibm.rqm.planning.service.internal.rest.IAttachmentRestService/";
    var ACTION_PROP = "__jazzActionProp";
    var FILE_UPLOAD_RESPONSE_PREFIX = "File Upload Response:"; // Must match response prefix in AttachmentRestService.java 
    var SIZE_EQUALS = "size=";
    var MODIFIED_EQUALS = "modified=";
    var NEW_ATTACHMENT = "Now";
    var UNKNOWN = "Unknown";
    var _BYTES = " bytes";
    var _currentBindRequest;
    var _currentUploadFrameNumber = 1000;

    function displayBlock(node, isDisplayed){
        node.style.display = isDisplayed ? "block" : "none";
    }
    
    dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.InsertImageDialog", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/widgets/templates/richtext/plugins/InsertImageDialog.html"),
        
		labelToInsertAnImage: Constants.getMessage("labelToInsertAnImage"),
		labelAttachmentUpload: Constants.getMessage("labelAttachmentUpload"),
		labelCancelUpload: Constants.getMessage("labelCancelUpload"),
		actionLabel_cancel: Constants.getMessage("actionLabel_cancel"),
		actionLabel_ok: Constants.getMessage("actionLabel_ok"),
		
        postCreate: function(){
            this.id = "com.ibm.asq.common.web.ui.internal.widgets.richtext.plugins.InsertImageDialog";
            this.hideForm = false;
            if (this._uploadForm != null) {
                this._initializeUploadInput();
            }
            this.reset();
        },
        
        destroyWidget: function(){
            this.parentController.pageController.removeAllMouseListeners(this);
        },
        
        reset: function(){
            // initialize basic user interface based on browser and logged-in state
            // show the upload form if the user is logged in
            displayBlock(this._uploadForm, !isGuest());
            this._initializeUploadInput(); // reset the form
        },
        
		cancelUpload: function() {
			this._showFormHideMessage();
			this.parentWidget.editor.focus();
		},
		
        // IE does not support setting the value of an input type=file, so the only way to clear the form
        // is to destroy it and recreate it
        _initializeUploadInput: function(){
        
            var div = this._uploadInputDiv;
            dojox.data.dom.removeChildren(div);
			var inputLabel = document.createElement("label");
			var input1 = document.createElement("INPUT");
            input1.type = "file";
            input1.name = "uploadFileInput";
			input1.className = "UploadControl";
            this._uploadFileInput = input1;
            inputLabel.appendChild(input1);
			div.appendChild(inputLabel);
        },
        
        _getFileNameAndId: function(/* String id:filename*/fileName){
            var result = {
                id: " ",
                fileName: fileName
            };
            var fileNameColonIndex = fileName.indexOf(":");
            if (fileNameColonIndex > -1) {
                result.id = fileName.substring(0, fileNameColonIndex);
                result.fileName = fileName.substring(fileNameColonIndex + 1, fileName.length);
            }
            return result;
        },
        _getOriginalTarget: function(e){
            // normalize w3c (originalTarget) and srcElement (IE)
            return e.target || e.srcElement;
        },
        
        // hide progress message and show clearedupload form
        _showFormHideMessage: function(){
            this._isUploadInProgress = false;
            this._initializeUploadInput(); // reset the form
        },
        isUploadInProgress: function(){
            return this._isUploadInProgress;
        },
        cancelUploadInProgress: function(){
            if (this._currentBindRequest) {
                this._currentBindRequest.cancel();
                if (dojo.isIE) {
                    dojo.io.iframe.setSrc(dojo.io.iframe._frame, "", true);
                    this._currentBindRequest.isCancelled = true;
                }
                if (dojo.isMozilla) {
                    window.stop();
                }
            }
            
            this._showFormHideMessage();
        },
        _performUploadSubmit: function(e, isRestart){
            if (e) {
                e.preventDefault();
            }
            this._isUploadInProgress = true;
            // we always rebuild the input, so remove the change event
            dojo.disconnect(this._uploadFileInput_onchange_handle);
            
            var fullPath;
            // Check to see if the file is already loaded
            if (isRestart) {
                fullPath = this._priorFullPath;
            }
            else {
                fullPath = this._uploadFileInput.value;
            }
            if (fullPath.length === 0) {
                this._showFormHideMessage();
                return;
            }
            
            this._priorFullPath = fullPath;
            //TODO: this should change. It should handle the forward slash on Linux
            var parts = fullPath.split("\\");
            var fileName = parts[parts.length - 1];
			var fileNameParts = fileName.split(".");
			var ext = fileNameParts[fileNameParts.length-1];
			if(!ext)
			{
				alert(MessageUtil.getMessage(MessageBundle.COMMON, "msgInvalidImage"));
				this._showFormHideMessage();
				return;
			}
			ext = ext.toUpperCase();
			if(ext.match(/^(JPG|GIF|PNG|SVG|BMP)$/)===null)
			{
				alert(MessageUtil.getMessage(MessageBundle.COMMON, "msgInvalidImage"));
				this._showFormHideMessage();
				return;
			}
            var self = this;
            if (this._currentBindRequest != null) {
                this._currentBindRequest.cancel();
                this._currentBindRequest = null;
            }
            
            /*
             * Internet Explorer (with dojo) has a bug such that if you wait ten seconds after one file upload, the
             * next one breaks. Resetting the iframe fixes the problem
             * Unfortunately this code reaches into iframe private variables.
             */
            if (dojo.io.iframe._frame) {
                document.body.removeChild(dojo.io.iframe._frame);
            }
            var frameName = "dojoIoIframe" + _currentUploadFrameNumber++;
            dojo.io.iframe._iframeName = frameName;
            dojo.io.iframe._frame = null;
            
            var url = TeamServerClient._getServicePrefix() + UPLOAD_SERVICE_URI + "?projectId=" + getCurrentProjectArea().itemId;
            
            this._currentBindRequest = dojo.io.iframe.send({
                form: this._uploadForm,
                url: TeamServerClient._getServicePrefix() + UPLOAD_SERVICE_URI + "?projectId=" + getCurrentProjectArea().itemId,
                content: {
                    mimetype: "text/html",
                    fileFields: "uploadFileInput"
                },
                handleAs: "html",
                handle: function(response, ioArgs){
                    if (response instanceof Error) {
                        if (response.dojoType && response.dojoType == "cancel") {
                            return; // this is a cancelled request, ignore it
                        }
                        self._currentBindRequest = null; // no need to cancel it any more
                        alert(MessageUtil.getMessage(MessageBundle.COMMON,"msgImageUploadError"));
                        self._showFormHideMessage();
                        console.debug("Error uploading file: " + response.message);
                        return;
                    }
                    var data = response;
                    // If the user presses cancel on Firefox or Stop, then we get an extra call on the next upload. Skip it
                    // The data URI of the bad call is for the page and not the service
                    if (data && data.documentURI && data.documentURI.indexOf("IAttachmentRestService") == -1) {
                        self._wasUploadCancelled = true;
                        self._performUploadSubmit(null, true);
                        return;
                    }
                    if (self._currentBindRequest && self._currentBindRequest.isCancelled) {
                        self._currentBindRequest = null; // no need to cancel it any more
                        return;
                    }
                    self._currentBindRequest = null; // no need to cancel it any more
                    var uuidOrMessage = data.body.innerHTML; /* innerHTML OK */
                    if (uuidOrMessage.indexOf(FILE_UPLOAD_RESPONSE_PREFIX) != 0) {
                        alert(MessageUtil.getMessage(MessageBundle.COMMON,"msgImageUploadFailed"));
                        console.debug("Upload failed. Response is [" + uuidOrMessage + "]");
                        self._showFormHideMessage();
                        return;
                    }
                    uuidOrMessage = uuidOrMessage.substring(FILE_UPLOAD_RESPONSE_PREFIX.length, uuidOrMessage.length);
                    if (uuidOrMessage.substring(0, 1) != "_") {
                        // If an upload was cancelled we get one of these anomolous calls
                        if (self._wasUploadCancelled) {
                            self._wasUploadCancelled = false;
                            return;
                        }
                        alert(MessageUtil.getMessage(MessageBundle.COMMON,"msgImageUploadFailedUnknown",uuidOrMessage));
                        self._showFormHideMessage();
                        return;
                    }
                    
                    var message = uuidOrMessage.split(',');
                    var size = 0;
					var id;
                    if (message.length > 0) {
                        uuidOrMessage = message[0];
                        size = message[1];
                        id = message[2];
                    }
                    
                    if (size == "0") {
                        if (!confirm( MessageUtil.getMessage(MessageBundle.COMMON,"msgImageFileEmpty",fileName))) {
                            self._showFormHideMessage();
                            return;
                        }
                    }
                    var attachment = {
                        id: id,//fileNameAndId.id,
                        fileName: fileName,//filefilefileNameAndId.fileName,
                        size: size,
                        path: fullPath,
                        uuid: uuidOrMessage,
                        isNew: true
                    };
                    if (self.parentWidget) 
                        self.parentWidget.setValue(attachment);
                    self._showFormHideMessage();
                }
            });
        }
    });
})();
