/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//-*- tab-width: 4 -*-
//
//
package com.ibm.rational.test.lm.spi.bf;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import com.buildforge.services.client.api.APIClientConnection;
import com.buildforge.services.client.dbo.Manifest;
import com.buildforge.services.client.dbo.Selector;
import com.buildforge.services.client.dbo.Server;
import com.buildforge.services.client.dbo.ServerAuth;
import com.buildforge.services.common.ServiceException;
import com.buildforge.services.common.dbo.ServerDBO;
import com.buildforge.services.common.dbo.ServerDBO.Error;
import com.ibm.ccl.soa.deploy.operation.RunState;
import com.ibm.rational.test.lm.spi.bf.common.BFAPIClientConnectionException;
import com.ibm.rational.test.lm.spi.bf.common.DiscoveryUnitsMapKey;
import com.ibm.rational.test.lm.spi.bf.common.ServerAgent;
import com.ibm.rational.test.lm.spi.bf.common.StatusPollUnit;
import com.ibm.rational.test.lm.spi.bf.common.TopologyParser;
import com.ibm.rational.test.lm.spi.bf.internal.APIExceptionHandle;
import com.ibm.rational.test.lm.spi.bf.internal.BFAPIClientConnection;
import com.ibm.rational.test.lm.spi.bf.internal.BuildForgeArtifacts;
import com.ibm.rational.test.lm.spi.bf.internal.DiscoveryThread;
import com.ibm.rational.test.lm.spi.bf.internal.DiscoveryUnit;
import com.ibm.rational.test.lm.spi.bf.internal.StatusPollThread;
import com.ibm.rational.test.lm.spi.bf.utils.CreateTopology;
import com.ibm.rational.test.lm.spi.common.SPIException;

/**
 * The inventoryService class provide methods to get information from these
 * external system about lab assets they are aware of.
 * 
 */
public class InventoryService {

	// singleton
	public static final InventoryService INSTANCE = new InventoryService();

	private BFAPIClientConnection systemConn = null;

	private Object m_lock = new Object();
	
	/**
	 * 
	 * @param xml
	 * @param provider_server_name
	 * @param provider_server_port
	 * @return
	 * @throws SPIException
	 */
	public String addAsset(String xml, String provider_server_name,
			int provider_server_port) throws SPIException {
		// Parse xml string to get server agent
		TopologyParser topParser = new TopologyParser();
		ServerAgent serverAgent = topParser.parseToServerAgent(xml);
		String serverId = serverAgent.getServerId();
		String hostname = serverAgent.getHostname();
		String ipAddress = serverAgent.getIpAddress();
		String agentDirectory = serverAgent.getPath();
		String username = serverAgent.getUsername();
		String password = serverAgent.getPassword();
		String port = serverAgent.getPort();
		
		Server server = null;
		if (serverId != null) {
			server = this.getSeverAgentById(serverId, provider_server_name,
					provider_server_port);
		}
		
		if ((hostname != null && hostname.length() > 0)
				&& (agentDirectory != null && agentDirectory.length() > 0)
				&& (username != null && username.length() > 0)
				&& (password != null && password.length() > 0)) {
			if (server == null) {
				// selector name and agent name are specified to host name now.
				serverId = createServerAgent(provider_server_name,
						provider_server_port, hostname, ipAddress,
						agentDirectory, username, password, port);
				if (serverId != null) {
					createSelector(provider_server_name,
							provider_server_port, serverId);
				}
			} else {
				serverId = updateServerAgent(provider_server_name,
						provider_server_port, server, hostname, ipAddress,
						agentDirectory, username, password, port);
			}
			if (serverId != null) {
				return updateAssetById(serverId, provider_server_name,
						provider_server_port);
			}
			return null;
		} else {
			throw new SPIException("Required parameters for adding asset to Build Forge should not be null"); //$NON-NLS-N$
		}
	}

	/**
	 * Cancel status poll
	 * 
	 * @param serverId
	 * @param provider_server_name
	 * @param provider_server_port
	 * @return
	 * @throws SPIException
	 */
	public void cancelStatusPoll(String serverId, String provider_server_name,
			int provider_server_port) throws SPIException {

		StatusPollUnit unit = new StatusPollUnit(serverId,
				provider_server_name, provider_server_port);
		StatusPollThread thread = StatusPollThread.getInstance();
		thread.cancelStatusPollUnit(unit);

	}

	/**
	 * Find all assets
	 * 
	 * @param provider_server_name
	 * @param provider_server_port
	 * @return
	 * @throws SPIException
	 */

	public String updateAllAssets(String provider_server_name,
			int provider_server_port) throws SPIException {

		Manifest agentManifest = null;
		List<Server> servers = null;
		String XMLString = null;
		HashMap<String, Manifest> assetMapping = new HashMap<String, Manifest>();
		try {
			systemConn = BFAPIClientConnection.getConnection(provider_server_name,
					provider_server_port);
			servers = Server.findAll(systemConn.getConnection());
			Iterator<Server> server = servers.iterator();
			while (server.hasNext()) {
				Server serverAgent = server.next();
				String serverName = serverAgent.getName();

				agentManifest = serverAgent.getManifest();
				String serverInfo = serverName + ":" + getServerId(serverName, //$NON-NLS-1$
						provider_server_name, provider_server_port);
				assetMapping.put(serverInfo, agentManifest);
			}
			XMLString = manifestToXML(assetMapping);
		} catch (Exception e) {
			APIExceptionHandle.handleAPIExceptions(e);
		} finally {
			try {
				// Release the Build Forge connection resource to avoid the
				// connection leak
				if (systemConn != null)
					systemConn.close();
			} catch (BFAPIClientConnectionException e) {
				// throw new SPIException("BuildForge
				// ClientConnectionException"); //$NON-NLS-1$
			}
		}
		return XMLString;
	}

	/**
	 * List all assets's external Id
	 * 
	 * @param provider_server_name
	 * @param provider_server_port
	 * @return
	 * @throws SPIException
	 */
	public String listAllAssetIds(String provider_server_name,
			int provider_server_port) throws SPIException {

		List<Server> servers = null;
		String XMLString = null;
		try {
			systemConn = BFAPIClientConnection.getConnection(provider_server_name,
					provider_server_port);
			servers = Server.findAll(systemConn.getConnection());
			Iterator<Server> server = servers.iterator();
			List<String> serverIds = new ArrayList<String>();
			while (server.hasNext()) {
				Server serverAgent = server.next();
				String serverId = String.valueOf(serverAgent.getServerId());
				serverIds.add(serverId);
			}
			XMLString = serverIdToXML(serverIds);
		} catch (Exception e) {
			APIExceptionHandle.handleAPIExceptions(e);
		} finally {
			try {
				// Release the Build Forge connection resource to avoid the
				// connection leak
				if (systemConn != null)
					systemConn.close();
			} catch (BFAPIClientConnectionException e) {
				// throw new SPIException("BuildForge
				// ClientConnectionException"); //$NON-NLS-1$
			}
		}
		return XMLString;
	}


	
	/**
	 * 
	 * @param serverId
	 * @param provider_server_name
	 * @param provider_server_port
	 * @return
	 * @throws SPIException
	 */
	public 
	String updateAssetById(String serverId, String provider_server_name,
			int provider_server_port) throws SPIException {
		HashMap<String, Manifest> assetMapping = new HashMap<String, Manifest>();
		String xml = null;
		try {
			systemConn = BFAPIClientConnection.getConnection(provider_server_name,
					provider_server_port);
			Server serverAgent = Server.findById(systemConn.getConnection(),
					Integer.valueOf(serverId));

			String serverName = serverAgent.getName();
			// It must refresh manifest, because the fetched manifest at once
			// could be null if the server is newly created.
//			serverAgent.refreshManifest(Server.DEFAULT_TEST_TIMEOUT);
			Manifest agentManifest = serverAgent.getManifest();
			String serverInfo = serverName + ":" + getServerId(serverName, //$NON-NLS-1$
					provider_server_name, provider_server_port);
			assetMapping.put(serverInfo, agentManifest);

			xml = manifestToXML(assetMapping);
		} catch (Exception e) {
			APIExceptionHandle.handleAPIExceptions(e);
		} finally {
			try {
				// Release the Build Forge connection resource to avoid the
				// connection leak
				if (systemConn != null)
					systemConn.close();
			} catch (BFAPIClientConnectionException e) {
				// throw new SPIException("BuildForge
				// ClientConnectionException"); //$NON-NLS-1$
			}
		}
		return xml;
	}

	/**
	 * Create build forge agent Server, the name and hostname of server agent
	 * are always equivalent.
	 * 
	 * @param provider_server_name
	 * @param provider_server_port
	 * @param hostName
	 * @param rlmAgentDirectory
	 * @param userName
	 * @param password
	 * @throws IOException
	 * @throws ServiceException
	 * @throws SPIException
	 */
	private String createServerAgent(String provider_server_name,
			int provider_server_port, String hostName, String ipAddress,
			String rlmAgentDirectory, String userName, String password, String port)
			throws SPIException {
		try {
			// Connect to the Build Forge
			systemConn = BFAPIClientConnection.getConnection(provider_server_name,
					provider_server_port);
			BuildForgeArtifacts bfArtifact = new BuildForgeArtifacts();
			return bfArtifact.createRtlmAgent(systemConn.getConnection(),
					hostName, ipAddress, rlmAgentDirectory, userName, password, port);
		} catch (Exception e) {
			APIExceptionHandle.handleAPIExceptions(e);
		} finally {
			try {
				// Release the Build Forge connection resource to avoid the
				// connection leak
				if (systemConn != null)
					systemConn.close();
			} catch (BFAPIClientConnectionException e) {
				// TODO Log
			}
		}
		return null;
	}
	
	/*
	 * Update server agent including server auth, the name and hostname of
	 * server agent are always equivalent.
	 */
	private String updateServerAgent(String provider_server_name,
			int provider_server_port, Server serverAgent, String hostName,
			String ipAddress, String agentDirectory, String userName,
			String password, String port) throws SPIException {
		try {
			// Connect to the Build Forge
			systemConn = BFAPIClientConnection.getConnection(provider_server_name,
					provider_server_port);
			BuildForgeArtifacts bfArtifact = new BuildForgeArtifacts();
			return bfArtifact.updateRtlmAgent(systemConn.getConnection(),
					serverAgent, hostName, ipAddress, agentDirectory, userName, password, port);
		} catch (Exception e) {
			APIExceptionHandle.handleAPIExceptions(e);
		} finally {
			try {
				// Release the Build Forge connection resource to avoid the
				// connection leak
				if (systemConn != null)
					systemConn.close();
			} catch (BFAPIClientConnectionException e) {
				// TODO Log
			}
}
		return null;
	}

	/**
	 * Delete a server agent on the provider.
	 * There are 3 artifacts associated with a server:
	 * the Server itself, the ServerAuth, and the Selector for the server.
	 * Build Forge throws exceptions if any artifact is removed that is still
	 * referenced by other artifacts (i.e. a Selector being used by a Project).
	 * We remove the Server first since it references the Selector and Auth,
	 * then we try to remove the secondary artifacts. We do not fail on 
	 * deletion of the Auth or Selector seeing how those are not functionally
	 * significant for a Server delete (the intent of this method); it's just
	 * proper cleanup.
	 * 
	 * @param hostname
	 * @param port
	 * @param serverId
	 */
	public void deleteAsset(String hostname, int port,  String serverId)throws SPIException {
		try {
			systemConn = BFAPIClientConnection.getConnection(hostname, port);
			APIClientConnection conn = systemConn.getConnection();
			Server server = Server.findById(conn, Integer.valueOf(serverId));
			if (server != null) {
	             // Delete the server
                server.delete();
				// Delete Server Auth
				String authId = server.getAuthId();
				if (authId != null) {
				    try {
				        ServerAuth.deleteById(conn, authId);
				    } catch (Exception e) {
				        // Swallow the Auth deletion exception - just
				        // a secondary artifact
				    }
				}
				String selectorId = server.getName();
				// Delete server's Selector
				Selector selector;
				if (selectorId != null) {
					selector = Selector.findById(conn, selectorId);
					if (selector != null) {
					    try {
					        selector.delete();
					    } catch (Exception e) {
		                    // Swallow the Selector deletion exception - just
	                        // a secondary artifact
					    }
					}
				}
			}
		} catch (Exception e) {
			APIExceptionHandle.handleAPIExceptions(e);
		}
	}
	
	/**
	 * Send an asynchronous request for discovering all server agents on the
	 * provider
	 * 
	 * @param hostname
	 *            provider's host name or IP address
	 * @param port
	 *            provider's port
	 * @return TDM XML: OperationUnit topology, in which operation id is
	 *         generated in random
	 * @throws SPIException
	 */
	public String discoveryAll(String hostname, int port) throws SPIException {
		// Generate operation id
		String operationId = generateOperationId();
		List<Server> servers = getAllServerAgents(hostname, port);
		DiscoveryUnitsMapKey key = new DiscoveryUnitsMapKey(operationId,
				hostname, port);
		// Create a DiscoveryUnit object
		DiscoveryUnit unit = new DiscoveryUnit(servers);
		DiscoveryThread thread = DiscoveryThread.getInstance();
		thread.addDiscoveryUnit(key, unit);
		synchronized (m_lock) {
			if (!thread.isAlive()) {
				thread.start();
			}
		}
		return CreateTopology.getInstance().createDiscoveryOperationTopology(
				operationId);
	}

	/**
	 * Send an asynchronous request for discovering the server agent by the
	 * given id on the provider
	 * 
	 * @param hostname
	 * @param port
	 * @param serverId
	 * @return
	 * @throws SPIException
	 */
	public String discoveryById(String hostname, int port, String serverId)
			throws SPIException {
		Server server = getSeverAgentById(serverId, hostname, port);
		if (server != null) {
			// Generate operation id
			String operationId = generateOperationId();
			DiscoveryUnitsMapKey key = new DiscoveryUnitsMapKey(operationId,
					hostname, port);
			// Create a DiscoveryUnit object
			List<Server> servers = new ArrayList<Server>(1);
			servers.add(server);
			DiscoveryUnit unit = new DiscoveryUnit(servers);
			DiscoveryThread thread = DiscoveryThread.getInstance();
			thread.addDiscoveryUnit(key, unit);
			synchronized (m_lock) {
				if (!thread.isAlive()) {
					thread.start();
				}
			}
			return CreateTopology.getInstance()
					.createDiscoveryOperationTopology(operationId);
		} else {
			throw new SPIException(
					"Can't get server agent on Build Forge by its external id:"
							+ serverId);
		}
	}

	public String getDiscoveryRunStatus(String hostname, int port,
			String operationId) throws SPIException {
		DiscoveryUnitsMapKey key = new DiscoveryUnitsMapKey(operationId,
				hostname, port);
		DiscoveryThread thread = DiscoveryThread.getInstance();
		Integer runState = thread.getDiscoveryUnitStatus(key);
		String[] externalIds = null;
		if (runState == RunState.SUSPENDED) {
			externalIds = thread.getDiscoveredServerExIdsOnCanceled(key);
		}
		return CreateTopology.getInstance().createDiscoveryOperationWithRunState(
				runState, externalIds);
	}

	public void cancelDiscovery(String hostname, int port, String operationId)
			throws SPIException {
		DiscoveryUnitsMapKey key = new DiscoveryUnitsMapKey(operationId,
				hostname, port);
		DiscoveryThread thread = DiscoveryThread.getInstance();
		thread.cancelDiscoveryUnit(key);
	}

	/**
	 * start status poll to get the server's status
	 * 
	 * @param serverId
	 * @param providre_server_name
	 * @param providre_server_port
	 * @return TDM xml string
	 * @throws SPIException
	 */
	public String startStatusPollById(String serverId,
			String provider_server_name, int provider_port) throws SPIException {

		StatusPollUnit unit = new StatusPollUnit(serverId,
				provider_server_name, provider_port);
		StatusPollThread thread = StatusPollThread.getInstance();
		thread.addStatusPollUnit(unit);
		synchronized (m_lock) {
			if (!thread.isAlive()) {
				thread.start();
			}
		}

		return serverIdToXML(Arrays.asList(new String[] { serverId }));
	}

	/**
	 * Get status poll result
	 * 
	 * @param serverId
	 * @param provider_server_name
	 * @param provider_server_port
	 * @return
	 * @throws SPIException
	 */
	public String getStatusPollById(String serverId,
			String provider_server_name, int provider_server_port)
			throws SPIException {
		Server server = this.getSeverAgentById(serverId, provider_server_name,
				provider_server_port);

		Error err = server.getError();
		serverStatusToXML(serverId, err);

		return null;

	}

	/**
	 * Get server agent id form server agent name
	 * 
	 * @param serverName
	 * @param provider_server_name
	 * @param provider_server_port
	 * @return server Id
	 * @throws SPIException
	 */
	private String getServerId(String serverName, String provider_server_name,
			int provider_server_port) throws SPIException {
		String serverId = null;
		Server server = this.getSeverAgent(serverName, provider_server_name,
				provider_server_port);
		if (null == server) {
			return null;
		}
		serverId = String.valueOf(server.getServerId());

		return serverId;
	}

	/**
	 * Get the server Agent.
	 * 
	 * @param the
	 *            name of the Server.
	 * @return the specified Server object.
	 * @throws SPIException
	 */
	private Server getSeverAgent(String serverName,
			String provider_server_name, int provider_server_port)
			throws SPIException {
		Server server = null;
		try {
			systemConn = BFAPIClientConnection.getConnection(provider_server_name,
					provider_server_port);
			server = Server.findByName(systemConn.getConnection(), serverName);
		} catch (Exception e) {
			APIExceptionHandle.handleAPIExceptions(e);
		}
		return server;

	}

	/**
	 * Get the server Agent from server Id.
	 * 
	 * @param the
	 *            id of the Server.
	 * @return the specified Server object.
	 * @throws SPIException
	 */
	private Server getSeverAgentById(String serverId,
			String provider_server_name, int provider_server_port)
			throws SPIException {
		Server server = null;
		try {
			systemConn = BFAPIClientConnection.getConnection(provider_server_name,
					provider_server_port);
			server = Server.findById(systemConn.getConnection(), Integer
					.valueOf(serverId));
		} catch (Exception e) {
			APIExceptionHandle.handleAPIExceptions(e);
		}
		return server;

	}

	private String createSelector(String provider_server_name,
			int provider_server_port, String serverId) throws SPIException {
		Selector selector = null;
		try {
			systemConn = BFAPIClientConnection.getConnection(provider_server_name,
					provider_server_port);
			Server server = this.getSeverAgentById(serverId,
					provider_server_name, provider_server_port);
			String agentName = server.getName();
			// if we need prefix now.
			String selectorId = agentName;
			BuildForgeArtifacts bfArtifact = new BuildForgeArtifacts();
			selector = bfArtifact.getRlmSelector(systemConn.getConnection(),
					selectorId);
		} catch (Exception e) {
			APIExceptionHandle.handleAPIExceptions(e);
		}
		return selector.getSelectorId();

	}

	/**
	 * Convert manifest to XML
	 * 
	 * @param manifest
	 * @return a XML string of machine's detail information
	 */
	private String manifestToXML(HashMap<String, Manifest> assetMapping) {

		CreateTopology createTop = CreateTopology.getInstance();
		return createTop.createServerTopology(assetMapping);

	}

	/**
	 * Add serverId to xml
	 * 
	 * @param serverId
	 * @return
	 */
	private String serverIdToXML(List<String> serverIds) {

		CreateTopology createTop = CreateTopology.getInstance();
		return createTop.createServerTopology(serverIds);

	}

	private String serverStatusToXML(String serverId, ServerDBO.Error err) {

		CreateTopology createTop = CreateTopology.getInstance();
		return createTop.createServerStatusTopology(serverId, err);

	}

	/*
	 * Use Random to generate the operationId for discovery, the scope is 0 to
	 * (Integer.MAX_VALUE * 3/4).
	 */
	private String generateOperationId() {
		int scope = Integer.MAX_VALUE / 4 * 3;
		Random random = new Random();
		return String.valueOf(random.nextInt(scope));
	}

	private List<Server> getAllServerAgents(String hostname, int port)
			throws SPIException {
		List<Server> servers = new ArrayList<Server>();
		try {
			systemConn = BFAPIClientConnection.getConnection(hostname, port);
			servers = Server.findAll(systemConn.getConnection());
		} catch (Exception e) {
			APIExceptionHandle.handleAPIExceptions(e);
		}finally {
			try {
				// Release the Build Forge connection resource to avoid the
				// connection leak
				if (systemConn != null) {
					systemConn.close();
				}
			} catch (BFAPIClientConnectionException e) {
				// Do not need to throw exception for it is not a big issue.
			}
		}
		return servers;

	}
}
