/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2008,2009. All
 * Rights Reserved.
 * 
 * U.S. Government Users Restricted Rights: Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
// -*- tab-width: 4 -*-
//
//
package com.ibm.rational.test.lm.spi.bf.internal;

import java.io.IOException;

import com.buildforge.services.client.api.APIClientConnection;
import com.buildforge.services.client.dbo.AccessGroup;
import com.buildforge.services.client.dbo.BuildClass;
import com.buildforge.services.client.dbo.Collector;
import com.buildforge.services.client.dbo.CollectorProperty;
import com.buildforge.services.client.dbo.Selector;
import com.buildforge.services.client.dbo.SelectorProperty;
import com.buildforge.services.client.dbo.Server;
import com.buildforge.services.client.dbo.ServerAuth;
import com.buildforge.services.common.ServiceException;
import com.buildforge.services.common.api.APIException;
import com.buildforge.services.common.api.Permission;
import com.buildforge.services.common.dbo.SelectorPropertyDBO;
import com.buildforge.services.common.dbo.ServerDBO;
import com.buildforge.services.common.dbo.BuildClassDBO.PurgeRule;
import com.buildforge.services.common.dbo.BuildClassDBO.PurgeType;
import com.buildforge.services.common.text.TextUtils;
import com.ibm.rational.test.lm.spi.bf.common.BFAPIException;
import com.ibm.rational.test.lm.spi.bf.common.BFConstants;

public class BuildForgeArtifacts {
	private static final Class<BuildForgeArtifacts> CLASS = BuildForgeArtifacts.class;

	// -----------------------------------------------------------------------------------
	// -----------------------------------------------------------------------------------
	// Roles
	// -----------------------------------------------------------------------------------
	// -----------------------------------------------------------------------------------

	// private static final String rlmUserRoleName = "RlmUserRole";
	// //$NON-NLS-1$
	// private static final String rlmAdminRoleName = "RlmAdminRole";
	// //$NON-NLS-1$

	// Operator level
	private static int rlmUserRoleLevel = 1;
	// Security level
	private static int rlmAdminRoleLevel = 5;

	public static int getRlmUserRoleLevel() {
		return rlmUserRoleLevel;
	}

	public static int getRlmAdminRoleLevel() {
		return rlmAdminRoleLevel;
	}

	public static boolean rolesAreInitialized = false;

	public static void initRoles(APIClientConnection apiConn)
			throws BFAPIException {
		// rlmUserRoleLevel = initRole(apiConn, rlmUserRoleName, true, false,
		// false);
		// rlmAdminRoleLevel = initRole(apiConn, rlmAdminRoleName, false, true,
		// false);
		rolesAreInitialized = true;
	}

	static final Permission[] defaultPermissions = {
			Permission.ActivateProject, Permission.ActivateSched,
			Permission.ActivateServer,

			Permission.AddEnvGroup, Permission.AddEnvGroupValue,
			Permission.AddFilterSet, Permission.AddFilterSetEntry,
			Permission.AddNote, Permission.AddProject,
			Permission.AddProjectReg, Permission.AddProjectStep,
			Permission.AddProjectTagVar, Permission.AddSchedule,
			Permission.AddTemplate, Permission.AddView,
			Permission.AddViewEntry, Permission.ChangeBuildClass,
			Permission.CancelRunning, Permission.CancelStep,
			Permission.ClobberProject, Permission.CloneBuild,
			Permission.DeleteBuildWaiting, Permission.DeleteEnvGroup,
			Permission.DeleteEnvGroupValue, Permission.DeleteFilterSet,
			Permission.DeleteFilterSetEntry, Permission.DeleteNote,
			Permission.DelProjectReg, Permission.DeleteProject,
			Permission.DeleteProjectStep, Permission.DeleteProjectTagVar,
			Permission.DeleteSchedule, Permission.DeleteTemplate,
			Permission.DeselectSteps, Permission.EditEnvValue,
			Permission.EditFilterSetEntry, Permission.EditProjectReg,
			Permission.EditProjectTagVar, Permission.EditStartTagVars,
			Permission.EditSchedule, Permission.ExecStepPerm,
			Permission.ExportProject, Permission.ImportAsUser,
			Permission.ModifyEnvGroup,
			Permission.MoveEnvValue,
			Permission.MoveProjectStep,
			Permission.PauseStep,
			Permission.PurgeBuild,
			// Permission.
			Permission.RestartBuild, Permission.RunBuild, Permission.RunStep,
			Permission.SetState, Permission.ToggleLink, Permission.UnlockBuild,
			Permission.UpdateProject, Permission.UpdateProjectStep,
			Permission.ViewLog

	};
	static final Permission[] adminPermissions = {
			Permission.AddAccess,
			Permission.AddAccessMember,
			Permission.AddCheckpoint,
			Permission.AddClass,
			Permission.AddCollector,
			Permission.AddInterface,
			Permission.AddLevel,
			Permission.AddLink,
			Permission.AddSelector,
			Permission.AddServer,
			// Permission.AddSourceModule,
			// Permission.AddSourceServer,
			// Permission.AddTimeZones,
			Permission.AddUser,
			Permission.DeleteAccess,
			Permission.DeleteAccessMember,
			Permission.DeleteChange,
			Permission.DeleteCheckpoint,
			Permission.DeleteClass,
			Permission.DeleteCollector,
			Permission.DeleteInterface,
			Permission.DeleteLink,
			Permission.DeleteSelector,
			Permission.DeleteServer,
			// Permission.DeleteSourceModule,
			// Permission.DeleteSourceServer,
			// Permission.DeleteTimeZones,
			Permission.DeleteUser, Permission.DeleteView,
			Permission.DeleteViewEntry, Permission.EditCollector,
			Permission.EditHiddenTags, Permission.EditInterface,
			Permission.EditLevel, Permission.EditSecurity,
			Permission.EditSelector, Permission.EditServer,
			Permission.EditSourceServer, Permission.EditTimeZones,
			Permission.ExpireUserPasswords, Permission.ImportAsAdmin,
			Permission.ImportProject, Permission.LockBuild,
			Permission.MakeRelease, Permission.ResetServerUse,
			Permission.RestoreCheckpoint, Permission.ToggleDefGroup,
			Permission.UpdateAccessMember, Permission.UpdateClass,
			Permission.UpdatePermMember, Permission.UpdateServerAccount,
			Permission.UpdateSysconfig, Permission.UpdateTemplate,
			Permission.UpdateUser, };

	private static void grantDefaultPermissions(AccessGroup group)
			throws Exception {
		for (int i = 0; i < defaultPermissions.length; ++i)
			group.grantPermission(defaultPermissions[i]);

	}

	private static void grantAdminPermissions(AccessGroup group)
			throws Exception {
		for (int i = 0; i < adminPermissions.length; ++i)
			group.grantPermission(adminPermissions[i]);
	}

	private static int initRole(APIClientConnection apiConn, String roleName,
			boolean isDefaultRole, boolean isAdmin, boolean isDeactivated)
			throws BFAPIException {
		final String methName = "initRole"; //$NON-NLS-1$
		try {
			AccessGroup group = AccessGroup.findByName(apiConn, roleName);
			if (group != null)
				return group.getLevel();
		} catch (Exception e) { /* TODO: handle this exception */
		}

		AccessGroup group = new AccessGroup(apiConn);
		group.setName(roleName);
		group.setDefault(isDefaultRole);
		try {
			group.create();
			if (!isDeactivated)
				grantDefaultPermissions(group);
			if (isAdmin)
				grantAdminPermissions(group);
			return group.getLevel();
		} catch (Exception e) {
			throw ConnectionController.bfApiException(CLASS, methName,
					"Can't create Rlm Role: " + roleName, e); //$NON-NLS-1$
		}

	}

	// -----------------------------------------------------------------------------------
	// -----------------------------------------------------------------------------------
	// build classes
	// -----------------------------------------------------------------------------------
	// -----------------------------------------------------------------------------------

	public static void initBuildClass(APIClientConnection apiConn, String name,
			int purgeDays) throws BFAPIException {
		final String methName = "initBuildClass"; //$NON-NLS-1$
		try {
			BuildClass bc = BuildClass.findByName(apiConn, name);
			if (bc == null) {
				bc = new BuildClass(apiConn);
				bc.setName(name);
				bc.setPurgeDays(purgeDays);
				bc.setLevel(rlmUserRoleLevel);
				bc.setPurgeRule(PurgeRule.ANY_BUILD);
				bc.setPurgeType(PurgeType.CONSOLE_DATA);
				bc.create();
			}
		} catch (Exception e) {
			throw ConnectionController.bfApiException(CLASS, methName,
					"JobDefinition can't create BuildClass " + name, e); //$NON-NLS-1$
		}
	}

	public static String initRlmServerAuth(APIClientConnection conn,
			String hostName) throws APIException {
		ServerAuth sa = new ServerAuth(conn);
		sa.setAuthId(BFConstants.serverAuthIdPrefix + hostName);
		sa.setLevel(BuildForgeArtifacts.getRlmAdminRoleLevel());
		sa.setUsername(BFConstants.defaultAgentUsername);
		sa.setPassword(TextUtils.bf_encrypt(BFConstants.defaultAgentPwd));
		try {
			sa.create();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return sa.getAuthId();
	}

	/**
	 * Create RTLM server agent, the name and hostname of server agent are
	 * always equivalent. If failed to test using given hostName, then try again
	 * by ip address if its value isn't null. If failed again, delete the
	 * created one.
	 * 
	 * @param conn
	 * @param hostName
	 * @param ipAddress
	 * @param agentDirectory
	 * @throws ServiceException
	 * @throws IOException
	 */
	public String createRtlmAgent(APIClientConnection conn, String hostName,
			String ipAddress, String agentDirectory, String userName,
			String password, String port) throws IOException, ServiceException {
		// Agent ServerAuth
		// Do we still need prefix now?

		String name = hostName;

		String authId = BFConstants.serverAuthIdPrefix + name;
		ServerAuth serverAuth = ServerAuth.findById(conn, authId);
		if (serverAuth == null) {
			if ((hostName != null && hostName.length() > 0)
					&& (password != null && password.length() > 0))
				serverAuth = createRtlmAgentAuth(conn, hostName, userName,
						password);
			else
				// add throws
				return null;
		} else {
			serverAuth = updateRtlmAgentAuth(userName, password, serverAuth);
		}
		authId = serverAuth.getAuthId();
		boolean isNew = false;
		Server rlmAgent = Server.findByName(conn, name);
		if (rlmAgent == null) {
			rlmAgent = new Server(conn);
			if (port != null && !"".equals(port)){
				hostName = hostName + ":" + port;
			}
			isNew = true;
		} 
		rlmAgent.setHost(hostName);
		rlmAgent.setName(name);
		rlmAgent.setPath(agentDirectory);
		rlmAgent.setLevel(BuildForgeArtifacts.getRlmAdminRoleLevel());
		// set ServerAuth id
		rlmAgent.setAuthId(authId);
		// set Collector id
		rlmAgent.setCollectorId(BFConstants.agentCollectorId);
		// Set the server can put and get
		rlmAgent.setPublish(ServerDBO.Publish.ALL);
		if (isNew){
			rlmAgent.create();	
		} else {
			rlmAgent.update();
		}
		if (isServerAlive(rlmAgent, conn)) {
			return String.valueOf(rlmAgent.getServerId());
		} else if (ipAddress != null) {
			
			if (port != null && !"".equals(port)){
				ipAddress = ipAddress + ":" + port;
			}
			rlmAgent.setHost(ipAddress);
			rlmAgent.update();
			if (isServerAlive(rlmAgent, conn)) {
				return String.valueOf(rlmAgent.getServerId());
			}
		}
		// Delete if failed to try again
		rlmAgent.delete();
		serverAuth.delete();
		return null;
	}

	/**
	 * Update server agent including server auth, the name and hostname of
	 * server agent are always equivalent. If failed to test using given
	 * hostName, then try again by ip address if its value isn't null. If failed
	 * again, resume old name and host to the server agent.
	 * 
	 * @param conn
	 * @param serverAgent
	 * @param hostName
	 * @param ipAddress
	 * @param agentDirectory
	 * @param userName
	 * @param password
	 * @return id of the updated server agent, null if the agent is not
	 *         available.
	 * @throws IOException
	 * @throws ServiceException
	 */
	public String updateRtlmAgent(APIClientConnection conn, Server serverAgent,
			String hostName, String ipAddress, String agentDirectory,
			String userName, String password, String port) throws IOException,
			ServiceException {
		//String newName = hostName;

		String oldName = serverAgent.getName();
		String oldHost = serverAgent.getHost();
		// Update server agent name, hostname and path
	//	serverAgent.setName(newName);
		if (port != null && !"".equals(port)){
			hostName = hostName + ":" + port;
		}
		serverAgent.setHost(hostName);
		serverAgent.setPath(agentDirectory);
		serverAgent.update();
		// Update server auth
		String authId = serverAgent.getAuthId();
		ServerAuth serverAuth = ServerAuth.findById(conn, authId);
		if (serverAuth == null) {
			serverAuth = createRtlmAgentAuth(conn, hostName, userName, password);
		} else {
			serverAuth = updateRtlmAgentAuth(userName, password, serverAuth);
		}
		if (isServerAlive(serverAgent, conn)) {
			return String.valueOf(serverAgent.getServerId());
		} else if (ipAddress != null) {
			// Try again if ipAddress is not null
			if (port != null && !"".equals(port)){
				ipAddress = ipAddress + ":" + port;
			}
			serverAgent.setHost(ipAddress);
			serverAgent.update();
			if (isServerAlive(serverAgent, conn)) {
				return String.valueOf(serverAgent.getServerId());
			}
		}
		// Resume old name and host to the server agent if failed to try again
		serverAgent.setName(oldName);
		serverAgent.setHost(oldHost);
		serverAgent.update();
		// Don't resume path and auth for server agent to keep consistent
		// between RQM and Build Forge
		return null;
	}

	/**
	 * Create collector
	 * 
	 * @param conn
	 * @throws ServiceException
	 * @throws IOException
	 */
	public static String createCollector(APIClientConnection conn,
			String collectorId) throws BFAPIException {
		final String methName = "createCollector"; //$NON-NLS-1$
		try {
			Collector coll = Collector.findById(conn, collectorId);//$NON-NLS-1$
			if (coll != null) {
				return coll.getCollectorName();
			} else {
				coll = new Collector(conn);
				coll.setCollectorName(collectorId); //$NON-NLS-1$
				String[] properties = Collector.getBuiltIns();
				for (String rtlmProp : properties) {
					CollectorProperty collPro = new CollectorProperty(conn,
							coll);
					collPro.setUserDefined(false);
					collPro.setPropertyName(rtlmProp);
					collPro.setPropertyValue(""); //$NON-NLS-1$
					coll.addCollectorProperty(collPro);
				}

				coll.create();
				return coll.getCollectorName();
			}
		} catch (Exception e) {
			throw ConnectionController.bfApiException(CLASS, methName,
					"RTLM can't create collector " + collectorId, e); //$NON-NLS-1$
		}

	}

	// max time in milliseconds that we will wait for a BuildForge
	// test() API response. 60 second timeout.
	private static final int MAX_PING_TIME = 60 * 1000;
	private static final int PING_INCREMENT = 100;

	/**
	 * Test if the server alive and can be connected
	 * 
	 * @param serverAgent
	 * @return
	 */
	public boolean isServerAlive(Server serverAgent,
			APIClientConnection connection) {
		boolean isAlive = false;
		try {
			ServerDBO.Error error = serverAgent
					.test(Server.DEFAULT_TEST_TIMEOUT * 2); // Need to more than
			// 30s
			if ((ServerDBO.Error.NO).equals(error)) {
				isAlive = true;
				return isAlive;
			}
			serverAgent = Server
					.findById(connection, serverAgent.getServerId());
			// workitem 6180
			// Some times BuildForge will seem to "hang" on a response from an
			// agent. It will indefintely stay in the PROCESSING/TESTING state.
			// Keep a timer and break after 60 seconds if we don't get a
			// response.
			int currentPingTime = 0;
			while ((serverAgent.getError().equals(ServerDBO.Error.PROCESSING) || serverAgent
					.getError().equals(ServerDBO.Error.TESTING))
					&& currentPingTime < MAX_PING_TIME) {
				try {
					Thread.sleep(PING_INCREMENT);
					currentPingTime += PING_INCREMENT;
				} catch (InterruptedException e) {
				}
				serverAgent = Server.findById(connection, serverAgent
						.getServerId());
			}
			if (serverAgent.getError().equals(ServerDBO.Error.NO)) {
				isAlive = true;
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
		} catch (ServiceException e) {
			// TODO Auto-generated catch block
		}
		return isAlive;
	}

	/**
	 * Create RTLM server agent's Auth
	 * 
	 * @param conn
	 * @param userName
	 * @param password
	 * @return
	 * @throws IOException
	 * @throws ServiceException
	 * @throws
	 */
	private ServerAuth createRtlmAgentAuth(APIClientConnection conn,
			String hostName, String userName, String password)
			throws IOException, ServiceException {

		String name = hostName;

		// TODO Add check if userName/Password are provided
		ServerAuth auth = new ServerAuth(conn);
		auth.setAuthId(BFConstants.serverAuthIdPrefix + name);
		auth.setUsername(userName);
		auth.setLevel(BuildForgeArtifacts.getRlmAdminRoleLevel());
		auth.setPassword(TextUtils.bf_encrypt(password));
		auth.create();
		return auth;

	}

	/**
	 * Update RTLM server agent's Auth
	 * 
	 * @param userName
	 * @param password
	 * @return
	 * @throws IOException
	 * @throws ServiceException
	 */
	private ServerAuth updateRtlmAgentAuth(String userName, String password,
			ServerAuth auth) throws IOException, ServiceException {
		if (userName != null && userName.length() > 0) {
			auth.setUsername(userName);
		}
		if (password != null && password.length() > 0) {
			auth.setPassword(TextUtils.bf_encrypt(password));
		}
		try {
			auth.update();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return auth;
	}

	/**
	 * Get RTLM server's selector.If the selector exists, return the selector.
	 * If selector doesn't exist, just create it.
	 * 
	 * @param apiConn
	 * @param rlmSelectorId
	 * @return
	 * @throws IOException
	 * @throws ServiceException
	 */
	public Selector getRlmSelector(APIClientConnection apiConn,
			String rlmSelectorId) throws IOException, ServiceException {

		Selector sel = null;

		sel = Selector.findById(apiConn, rlmSelectorId);
		if (sel == null) {
			sel = new Selector(apiConn);
			sel.setSelectorId(rlmSelectorId);
			sel.setLevel(getRlmUserRoleLevel());
			SelectorProperty selectorProperty = new SelectorProperty(apiConn,
					sel);
			selectorProperty
					.setPropertyName(BFConstants.SELECTOR_PROPERTY_NAME);
			selectorProperty.setOperator(SelectorPropertyDBO.Operator.EQ);
			selectorProperty.setPropertyValue(rlmSelectorId);
			sel.addProperty(selectorProperty);
			sel.create();

		}
		return sel;

	}

	/**
	 * Crate a new selector and its id is rlmSelectorId
	 * 
	 * @param apiConn
	 * @param rlmSelectorId
	 * @return
	 * @throws IOException
	 * @throws ServiceException
	 */
	public Selector createRlmSelector(APIClientConnection apiConn,
			String rlmSelectorId) throws IOException, ServiceException {
		Selector sel = new Selector(apiConn);
		sel.setSelectorId(rlmSelectorId);
		sel.setLevel(getRlmUserRoleLevel());
		SelectorProperty selectorProperty = new SelectorProperty(apiConn, sel);
		selectorProperty.setPropertyName(BFConstants.SELECTOR_PROPERTY_NAME);
		selectorProperty.setOperator(SelectorPropertyDBO.Operator.EQ);
		selectorProperty.setPropertyValue(rlmSelectorId);
		sel.addProperty(selectorProperty);
		sel.create();
		return sel;
	}

	/**
	 * Update selector's property
	 * 
	 * @param selectorProperty
	 * @param rlmSelectorId
	 * @throws IOException
	 * @throws ServiceException
	 */
	public void updateRlmSelector(SelectorProperty selectorProperty,
			String rlmSelectorId) throws IOException, ServiceException {
		selectorProperty.setPropertyName(BFConstants.SELECTOR_PROPERTY_NAME);
		selectorProperty.setOperator(SelectorPropertyDBO.Operator.EQ);
		selectorProperty.setPropertyValue(rlmSelectorId);
		selectorProperty.update();
	}
	
	/**
	 * Update collector to have all Build in properties
	 * 
	 * @param conn
	 * @param collector Id 
	 * @throws BFAPIException
	 */
	public void updateCollectorForDiscovery(APIClientConnection conn,
			String collectorId) throws BFAPIException {
		final String methName = "updateCollectorForDiscovery"; //$NON-NLS-1$
		try {
			Collector coll = Collector.findById(conn, collectorId);//$NON-NLS-1$
			if (coll != null) {
				String[] properties = Collector.getBuiltIns();
				for (String rtlmProp : properties) {
					
					if (coll.getProperty(rtlmProp) == null){
						CollectorProperty collPro = new CollectorProperty(conn,
								coll);
						collPro.setUserDefined(false);
						collPro.setPropertyName(rtlmProp);
						collPro.setPropertyValue(""); //$NON-NLS-1$
						coll.addCollectorProperty(collPro);
					}					
				}
				coll.update();	
			} 
		} catch (Exception e) {
			throw ConnectionController.bfApiException(CLASS, methName,
					"RTLM can't update collector " + collectorId, e); //$NON-NLS-1$
		}

	}
	
}
